@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2016 Canonical Ltd
+ * Copyright (C) 2015-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -43,6 +43,7 @@ func Test(t *testing.T) { TestingT(t) }
 
 var _ = Suite(&openSuite{})
 var _ = Suite(&revisionErrorSuite{})
+var _ = Suite(&isUnacceptedUpdateSuite{})
 
 type openSuite struct{}
 
@@ -312,6 +313,85 @@ func (chks *checkSuite) TestCheckUnsupportedFormat(c *C) {
 	c.Check(err, ErrorMatches, `proposed "test-only" assertion has format 77 but 1 is latest supported`)
 }
 
+func (chks *checkSuite) TestCheckMismatchedAccountIDandKey(c *C) {
+	trustedKey := testPrivKey0
+
+	cfg := &asserts.DatabaseConfig{
+		Backstore: chks.bs,
+		Trusted:   []asserts.Assertion{asserts.BootstrapAccountKeyForTest("canonical", trustedKey.PublicKey())},
+	}
+	db, err := asserts.OpenDatabase(cfg)
+	c.Assert(err, IsNil)
+
+	headers := map[string]interface{}{
+		"authority-id": "random",
+		"primary-key":  "0",
+	}
+	a, err := asserts.AssembleAndSignInTest(asserts.TestOnlyType, headers, nil, trustedKey)
+	c.Assert(err, IsNil)
+
+	err = db.Check(a)
+	c.Check(err, ErrorMatches, `error finding matching public key for signature: found public key ".*" from "canonical" but expected it from: random`)
+
+	err = asserts.CheckSignature(a, cfg.Trusted[0].(*asserts.AccountKey), db, time.Time{}, time.Time{})
+	c.Check(err, ErrorMatches, `assertion authority "random" does not match public key from "canonical"`)
+}
+
+func (chks *checkSuite) TestCheckAndSetEarliestTime(c *C) {
+	trustedKey := testPrivKey0
+
+	ak := asserts.MakeAccountKeyForTest("canonical", trustedKey.PublicKey(), time.Date(2016, 1, 1, 0, 0, 0, 0, time.UTC), 2)
+
+	cfg := &asserts.DatabaseConfig{
+		Backstore: chks.bs,
+		Trusted:   []asserts.Assertion{ak},
+	}
+	db, err := asserts.OpenDatabase(cfg)
+	c.Assert(err, IsNil)
+
+	headers := map[string]interface{}{
+		"authority-id": "canonical",
+		"primary-key":  "0",
+	}
+	a, err := asserts.AssembleAndSignInTest(asserts.TestOnlyType, headers, nil, trustedKey)
+	c.Assert(err, IsNil)
+
+	// now is since + 1 year, key is valid
+	r := asserts.MockTimeNow(ak.Since().AddDate(1, 0, 0))
+	defer r()
+
+	err = db.Check(a)
+	c.Check(err, IsNil)
+
+	// now is since - 1 year, key is invalid
+	pastTime := ak.Since().AddDate(-1, 0, 0)
+	asserts.MockTimeNow(pastTime)
+
+	err = db.Check(a)
+	c.Check(err, ErrorMatches, `assertion is signed with expired public key .*`)
+
+	// now is ignored but known to be at least >= pastTime
+	// key is considered valid
+	db.SetEarliestTime(pastTime)
+	err = db.Check(a)
+	c.Check(err, IsNil)
+
+	// move earliest after until
+	db.SetEarliestTime(ak.Until().AddDate(0, 0, 1))
+	err = db.Check(a)
+	c.Check(err, ErrorMatches, `assertion is signed with expired public key .*`)
+
+	// check using now = since - 1 year again
+	db.SetEarliestTime(time.Time{})
+	err = db.Check(a)
+	c.Check(err, ErrorMatches, `assertion is signed with expired public key .*`)
+
+	// now is since + 1 month, key is valid
+	asserts.MockTimeNow(ak.Since().AddDate(0, 1, 0))
+	err = db.Check(a)
+	c.Check(err, IsNil)
+}
+
 type signAddFindSuite struct {
 	signingDB    *asserts.Database
 	signingKeyID string
@@ -1238,6 +1318,117 @@ func (safs *signAddFindSuite) TestWithStackedBackstoreSafety(c *C) {
 	})
 }
 
+func (safs *signAddFindSuite) TestFindSequence(c *C) {
+	headers := map[string]interface{}{
+		"authority-id": "canonical",
+		"n":            "s1",
+		"sequence":     "1",
+	}
+	sq1f0, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"n":            "s1",
+		"sequence":     "2",
+	}
+	sq2f0, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "1",
+		"n":            "s1",
+		"sequence":     "2",
+		"revision":     "1",
+	}
+	sq2f1, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "1",
+		"n":            "s1",
+		"sequence":     "3",
+	}
+	sq3f1, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "2",
+		"n":            "s1",
+		"sequence":     "3",
+		"revision":     "1",
+	}
+	sq3f2, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	for _, a := range []asserts.Assertion{sq1f0, sq2f0, sq2f1, sq3f1} {
+
+		err = safs.db.Add(a)
+		c.Assert(err, IsNil)
+	}
+
+	// stack a backstore, for test completeness, this is an unlikely
+	// scenario atm
+	bs := asserts.NewMemoryBackstore()
+	db := safs.db.WithStackedBackstore(bs)
+	err = db.Add(sq3f2)
+	c.Assert(err, IsNil)
+
+	seqHeaders := map[string]string{
+		"n": "s1",
+	}
+	tests := []struct {
+		after     int
+		maxFormat int
+		sequence  int
+		format    int
+		revision  int
+	}{
+		{after: 0, maxFormat: 0, sequence: 1, format: 0, revision: 0},
+		{after: 0, maxFormat: 2, sequence: 1, format: 0, revision: 0},
+		{after: 1, maxFormat: 0, sequence: 2, format: 0, revision: 0},
+		{after: 1, maxFormat: 1, sequence: 2, format: 1, revision: 1},
+		{after: 1, maxFormat: 2, sequence: 2, format: 1, revision: 1},
+		{after: 2, maxFormat: 0, sequence: -1},
+		{after: 2, maxFormat: 1, sequence: 3, format: 1, revision: 0},
+		{after: 2, maxFormat: 2, sequence: 3, format: 2, revision: 1},
+		{after: 3, maxFormat: 0, sequence: -1},
+		{after: 3, maxFormat: 2, sequence: -1},
+		{after: 4, maxFormat: 2, sequence: -1},
+		{after: -1, maxFormat: 0, sequence: 2, format: 0, revision: 0},
+		{after: -1, maxFormat: 1, sequence: 3, format: 1, revision: 0},
+		{after: -1, maxFormat: 2, sequence: 3, format: 2, revision: 1},
+	}
+
+	for _, t := range tests {
+		a, err := db.FindSequence(asserts.TestOnlySeqType, seqHeaders, t.after, t.maxFormat)
+		if t.sequence == -1 {
+			c.Check(err, DeepEquals, &asserts.NotFoundError{
+				Type:    asserts.TestOnlySeqType,
+				Headers: seqHeaders,
+			})
+		} else {
+			c.Assert(err, IsNil)
+			c.Assert(a.HeaderString("n"), Equals, "s1")
+			c.Check(a.Sequence(), Equals, t.sequence)
+			c.Check(a.Format(), Equals, t.format)
+			c.Check(a.Revision(), Equals, t.revision)
+		}
+	}
+
+	seqHeaders = map[string]string{
+		"n": "s2",
+	}
+	_, err = db.FindSequence(asserts.TestOnlySeqType, seqHeaders, -1, 2)
+	c.Check(err, DeepEquals, &asserts.NotFoundError{
+		Type: asserts.TestOnlySeqType, Headers: seqHeaders,
+	})
+
+}
+
 type revisionErrorSuite struct{}
 
 func (res *revisionErrorSuite) TestErrorText(c *C) {
