@@ -27,7 +27,7 @@ import (
 )
 
 // A Grouping identifies opaquely a grouping of assertions.
-// Pool uses it to label the interesection between a set of groups.
+// Pool uses it to label the intersection between a set of groups.
 type Grouping string
 
 // A pool helps holding and tracking a set of assertions and their
@@ -39,10 +39,13 @@ type Grouping string
 // of more than one group are tracked properly only once.
 //
 // Typical usage involves specifying the initial assertions needing to
-// be resolved or updated using AddUnresolved and AddToUpdate. At this
-// point ToResolve can be called to get them organized in groupings
-// ready for fetching. Fetched assertions can then be provided with
-// Add or AddBatch. Because these can have prerequisites calling
+// be resolved or updated using AddUnresolved and AddToUpdate.
+// AddUnresolvedSequence and AddSequenceToUpdate exist parallel to
+// AddUnresolved/AddToUpdate to handle sequence-forming assertions,
+// which cannot be used with the latter.
+// At this point ToResolve can be called to get them organized in
+// groupings ready for fetching. Fetched assertions can then be provided
+// with Add or AddBatch. Because these can have prerequisites calling
 // ToResolve and fetching needs to be repeated until ToResolve's
 // result is empty.  Between any two ToResolve invocations but after
 // any Add or AddBatch AddUnresolved/AddToUpdate can also be used
@@ -71,10 +74,13 @@ type Pool struct {
 	numbering map[string]uint16
 	groupings *internal.Groupings
 
-	unresolved    map[string]*unresolvedRec
-	prerequisites map[string]*unresolvedRec
+	unresolved          map[string]unresolvedAssertRecord
+	unresolvedSequences map[string]unresolvedAssertRecord
+	prerequisites       map[string]unresolvedAssertRecord
+
+	bs        Backstore
+	unchanged map[string]bool
 
-	bs     Backstore
 	groups map[uint16]*groupRec
 
 	curPhase poolPhase
@@ -90,13 +96,15 @@ func NewPool(groundDB RODatabase, n int) *Pool {
 		panic(fmt.Sprintf("NewPool: %v", err))
 	}
 	return &Pool{
-		groundDB:      groundDB,
-		numbering:     make(map[string]uint16),
-		groupings:     groupings,
-		unresolved:    make(map[string]*unresolvedRec),
-		prerequisites: make(map[string]*unresolvedRec),
-		bs:            NewMemoryBackstore(),
-		groups:        make(map[uint16]*groupRec),
+		groundDB:            groundDB,
+		numbering:           make(map[string]uint16),
+		groupings:           groupings,
+		unresolved:          make(map[string]unresolvedAssertRecord),
+		unresolvedSequences: make(map[string]unresolvedAssertRecord),
+		prerequisites:       make(map[string]unresolvedAssertRecord),
+		bs:                  NewMemoryBackstore(),
+		unchanged:           make(map[string]bool),
+		groups:              make(map[uint16]*groupRec),
 	}
 }
 
@@ -118,8 +126,9 @@ func (p *Pool) ensureGroup(group string) (gnum uint16, err error) {
 		return 0, err
 	}
 	if gRec := p.groups[gnum]; gRec == nil {
-		gRec = new(groupRec)
-		p.groups[gnum] = gRec
+		p.groups[gnum] = &groupRec{
+			name: group,
+		}
 	}
 	return gnum, nil
 }
@@ -138,6 +147,14 @@ func (p *Pool) Singleton(group string) (Grouping, error) {
 	return Grouping(p.groupings.Serialize(&grouping)), nil
 }
 
+type unresolvedAssertRecord interface {
+	isAssertionNewer(a Assertion) bool
+	groupingPtr() *internal.Grouping
+	label() Grouping
+	isRevisionNotKnown() bool
+	error() error
+}
+
 // An unresolvedRec tracks a single unresolved assertion until it is
 // resolved or there is an error doing so. The field 'grouping' will
 // grow to contain all the groups requiring this assertion while it
@@ -151,6 +168,26 @@ type unresolvedRec struct {
 	err error
 }
 
+func (u *unresolvedRec) isAssertionNewer(a Assertion) bool {
+	return a.Revision() > u.at.Revision
+}
+
+func (u *unresolvedRec) groupingPtr() *internal.Grouping {
+	return &u.grouping
+}
+
+func (u *unresolvedRec) label() Grouping {
+	return u.serializedLabel
+}
+
+func (u *unresolvedRec) isRevisionNotKnown() bool {
+	return u.at.Revision == RevisionNotKnown
+}
+
+func (u *unresolvedRec) error() error {
+	return u.err
+}
+
 func (u *unresolvedRec) exportTo(r map[Grouping][]*AtRevision, gr *internal.Groupings) {
 	serLabel := Grouping(gr.Serialize(&u.grouping))
 	// remember serialized label
@@ -166,9 +203,58 @@ func (u *unresolvedRec) merge(at *AtRevision, gnum uint16, gr *internal.Grouping
 	}
 }
 
+type unresolvedSeqRec struct {
+	at       *AtSequence
+	grouping internal.Grouping
+
+	serializedLabel Grouping
+
+	err error
+}
+
+func (u *unresolvedSeqRec) groupingPtr() *internal.Grouping {
+	return &u.grouping
+}
+
+func (u *unresolvedSeqRec) label() Grouping {
+	return u.serializedLabel
+}
+
+func (u *unresolvedSeqRec) isAssertionNewer(a Assertion) bool {
+	seqf, ok := a.(SequenceMember)
+	if !ok {
+		// This should never happen because resolveWith() compares correct types.
+		panic(fmt.Sprintf("internal error: cannot compare assertion %v with unresolved sequence-forming assertion (wrong type)", a.Ref()))
+	}
+	if u.at.Pinned {
+		return seqf.Sequence() == u.at.Sequence && a.Revision() > u.at.Revision
+	}
+	// not pinned
+	if seqf.Sequence() == u.at.Sequence {
+		return a.Revision() > u.at.Revision
+	}
+	return seqf.Sequence() > u.at.Sequence
+}
+
+func (u *unresolvedSeqRec) isRevisionNotKnown() bool {
+	return u.at.Revision == RevisionNotKnown
+}
+
+func (u *unresolvedSeqRec) error() error {
+	return u.err
+}
+
+func (u *unresolvedSeqRec) exportTo(r map[Grouping][]*AtSequence, gr *internal.Groupings) {
+	serLabel := Grouping(gr.Serialize(&u.grouping))
+	// remember serialized label
+	u.serializedLabel = serLabel
+	r[serLabel] = append(r[serLabel], u.at)
+}
+
 // A groupRec keeps track of all the resolved assertions in a group
 // or whether the group should be considered in error (err != nil).
 type groupRec struct {
+	name     string
 	err      error
 	resolved []Ref
 }
@@ -224,6 +310,9 @@ func (p *Pool) isPredefined(ref *Ref) (bool, error) {
 }
 
 func (p *Pool) isResolved(ref *Ref) (bool, error) {
+	if p.unchanged[ref.Unique()] {
+		return true, nil
+	}
 	_, err := p.bs.Get(ref.Type, ref.PrimaryKey, ref.Type.MaxSupportedFormat())
 	if err == nil {
 		return true, nil
@@ -245,6 +334,17 @@ func (p *Pool) curRevision(ref *Ref) (int, error) {
 	return RevisionNotKnown, nil
 }
 
+func (p *Pool) curSeqRevision(seq *AtSequence) (int, error) {
+	a, err := seq.Resolve(p.groundDB.Find)
+	if err != nil && !IsNotFound(err) {
+		return 0, err
+	}
+	if err == nil {
+		return a.Revision(), nil
+	}
+	return RevisionNotKnown, nil
+}
+
 type poolPhase int
 
 const (
@@ -269,6 +369,10 @@ func (p *Pool) phase(ph poolPhase) error {
 // AtRevision to the Pool as unresolved and as required by the given group.
 // Usually unresolved.Revision will have been set to RevisionNotKnown.
 func (p *Pool) AddUnresolved(unresolved *AtRevision, group string) error {
+	if unresolved.Type.SequenceForming() {
+		return fmt.Errorf("internal error: AddUnresolved requested for sequence-forming assertion")
+	}
+
 	if err := p.phase(poolPhaseAddUnresolved); err != nil {
 		return err
 	}
@@ -288,6 +392,26 @@ func (p *Pool) AddUnresolved(unresolved *AtRevision, group string) error {
 	return p.addUnresolved(&u, gnum)
 }
 
+// AddUnresolvedSequence adds the assertion referenced by unresolved
+// AtSequence to the Pool as unresolved and as required by the given group.
+// Usually unresolved.Revision will have been set to RevisionNotKnown.
+// Given sequence can only be added once to the Pool.
+func (p *Pool) AddUnresolvedSequence(unresolved *AtSequence, group string) error {
+	if err := p.phase(poolPhaseAddUnresolved); err != nil {
+		return err
+	}
+	if p.unresolvedSequences[unresolved.Unique()] != nil {
+		return fmt.Errorf("internal error: sequence %v is already being resolved", unresolved.SequenceKey)
+	}
+	gnum, err := p.ensureGroup(group)
+	if err != nil {
+		return err
+	}
+	u := *unresolved
+	p.addUnresolvedSeq(&u, gnum)
+	return nil
+}
+
 func (p *Pool) addUnresolved(unresolved *AtRevision, gnum uint16) error {
 	ok, err := p.isResolved(&unresolved.Ref)
 	if err != nil {
@@ -306,17 +430,28 @@ func (p *Pool) addUnresolved(unresolved *AtRevision, gnum uint16) error {
 		return nil
 	}
 	uniq := unresolved.Ref.Unique()
-	var u *unresolvedRec
+	var u unresolvedAssertRecord
 	if u = p.unresolved[uniq]; u == nil {
 		u = &unresolvedRec{
 			at: unresolved,
 		}
 		p.unresolved[uniq] = u
 	}
-	u.merge(unresolved, gnum, p.groupings)
+
+	urec := u.(*unresolvedRec)
+	urec.merge(unresolved, gnum, p.groupings)
 	return nil
 }
 
+func (p *Pool) addUnresolvedSeq(unresolved *AtSequence, gnum uint16) error {
+	uniq := unresolved.Unique()
+	u := &unresolvedSeqRec{
+		at: unresolved,
+	}
+	p.unresolvedSequences[uniq] = u
+	return p.groupings.AddTo(&u.grouping, gnum)
+}
+
 // ToResolve returns all the currently unresolved assertions in the
 // Pool, organized in opaque groupings based on which set of groups
 // requires each of them.
@@ -326,26 +461,42 @@ func (p *Pool) addUnresolved(unresolved *AtRevision, gnum uint16) error {
 // Conversely, the remaining unresolved assertions originally added
 // via AddToUpdate will be assumed to still be at their current
 // revisions.
-func (p *Pool) ToResolve() (map[Grouping][]*AtRevision, error) {
+func (p *Pool) ToResolve() (map[Grouping][]*AtRevision, map[Grouping][]*AtSequence, error) {
 	if p.curPhase == poolPhaseAdd {
 		p.unresolvedBookkeeping()
 	} else {
 		p.curPhase = poolPhaseAdd
 	}
-	r := make(map[Grouping][]*AtRevision)
-	for _, u := range p.unresolved {
+	atr := make(map[Grouping][]*AtRevision)
+	for _, ur := range p.unresolved {
+		u := ur.(*unresolvedRec)
 		if u.at.Revision == RevisionNotKnown {
 			rev, err := p.curRevision(&u.at.Ref)
 			if err != nil {
-				return nil, err
+				return nil, nil, err
 			}
 			if rev != RevisionNotKnown {
 				u.at.Revision = rev
 			}
 		}
-		u.exportTo(r, p.groupings)
+		u.exportTo(atr, p.groupings)
 	}
-	return r, nil
+
+	ats := make(map[Grouping][]*AtSequence)
+	for _, u := range p.unresolvedSequences {
+		seq := u.(*unresolvedSeqRec)
+		if seq.at.Revision == RevisionNotKnown {
+			rev, err := p.curSeqRevision(seq.at)
+			if err != nil {
+				return nil, nil, err
+			}
+			if rev != RevisionNotKnown {
+				seq.at.Revision = rev
+			}
+		}
+		seq.exportTo(ats, p.groupings)
+	}
+	return atr, ats, nil
 }
 
 func (p *Pool) addPrerequisite(pref *Ref, g *internal.Grouping) error {
@@ -361,7 +512,8 @@ func (p *Pool) addPrerequisite(pref *Ref, g *internal.Grouping) error {
 	if u != nil {
 		gr := p.groupings
 		gr.Iter(g, func(gnum uint16) error {
-			u.merge(at, gnum, gr)
+			urec := u.(*unresolvedRec)
+			urec.merge(at, gnum, gr)
 			return nil
 		})
 		return nil
@@ -415,12 +567,12 @@ func (p *Pool) add(a Assertion, g *internal.Grouping) error {
 	return nil
 }
 
-func (p *Pool) resolveWith(unresolved map[string]*unresolvedRec, uniq string, u *unresolvedRec, a Assertion, extrag *internal.Grouping) error {
-	if a.Revision() > u.at.Revision {
+func (p *Pool) resolveWith(unresolved map[string]unresolvedAssertRecord, uniq string, u unresolvedAssertRecord, a Assertion, extrag *internal.Grouping) (ok bool, err error) {
+	if u.isAssertionNewer(a) {
 		if extrag == nil {
-			extrag = &u.grouping
+			extrag = u.groupingPtr()
 		} else {
-			p.groupings.Iter(&u.grouping, func(gnum uint16) error {
+			p.groupings.Iter(u.groupingPtr(), func(gnum uint16) error {
 				p.groupings.AddTo(extrag, gnum)
 				return nil
 			})
@@ -436,11 +588,11 @@ func (p *Pool) resolveWith(unresolved map[string]*unresolvedRec, uniq string, u
 			delete(unresolved, uniq)
 			if err := p.add(a, extrag); err != nil {
 				p.setErr(extrag, err)
-				return err
+				return false, err
 			}
 		}
 	}
-	return nil
+	return true, nil
 }
 
 // Add adds the given assertion associated with the given grouping to the
@@ -452,56 +604,136 @@ func (p *Pool) resolveWith(unresolved map[string]*unresolvedRec, uniq string, u
 // requiring the assertion plus the groups in grouping will be considered.
 // The latter is mostly relevant in scenarios where the server is pushing
 // assertions.
-func (p *Pool) Add(a Assertion, grouping Grouping) error {
+// If an error is returned it refers to an immediate or local error.
+// Errors related to the assertions are associated with the relevant groups
+// and can be retrieved with Err, in which case ok is set to false.
+func (p *Pool) Add(a Assertion, grouping Grouping) (ok bool, err error) {
 	if err := p.phase(poolPhaseAdd); err != nil {
-		return err
+		return false, err
 	}
 
 	if !a.SupportedFormat() {
-		return &UnsupportedFormatError{Ref: a.Ref(), Format: a.Format()}
+		e := &UnsupportedFormatError{Ref: a.Ref(), Format: a.Format()}
+		p.AddGroupingError(e, grouping)
+		return false, nil
 	}
 
-	uniq := a.Ref().Unique()
-	var u *unresolvedRec
+	return p.addToGrouping(a, grouping, p.groupings.Deserialize)
+}
+
+func (p *Pool) addToGrouping(a Assertion, grouping Grouping, deserializeGrouping func(string) (*internal.Grouping, error)) (ok bool, err error) {
+	var uniq string
+	ref := a.Ref()
+	var u unresolvedAssertRecord
 	var extrag *internal.Grouping
-	var unresolved map[string]*unresolvedRec
-	if u = p.unresolved[uniq]; u != nil {
-		unresolved = p.unresolved
-	} else if u = p.prerequisites[uniq]; u != nil {
-		unresolved = p.prerequisites
+	var unresolved map[string]unresolvedAssertRecord
+
+	if !ref.Type.SequenceForming() {
+		uniq = ref.Unique()
+		if u = p.unresolved[uniq]; u != nil {
+			unresolved = p.unresolved
+		} else if u = p.prerequisites[uniq]; u != nil {
+			unresolved = p.prerequisites
+		} else {
+			ok, err := p.isPredefined(a.Ref())
+			if err != nil {
+				return false, err
+			}
+			if ok {
+				// nothing to do
+				return true, nil
+			}
+			// a is not tracked as unresolved in any way so far,
+			// this is an atypical scenario where something gets
+			// pushed but we still want to add it to the resolved
+			// lists of the relevant groups; in case it is
+			// actually already resolved most of resolveWith below will
+			// be a nop
+			rec := &unresolvedRec{
+				at: a.At(),
+			}
+			rec.at.Revision = RevisionNotKnown
+			u = rec
+		}
 	} else {
-		ok, err := p.isPredefined(a.Ref())
-		if err != nil {
-			return err
+		atseq := AtSequence{
+			Type:        ref.Type,
+			SequenceKey: ref.PrimaryKey[:len(ref.PrimaryKey)-1],
 		}
-		if ok {
-			// nothing to do
-			return nil
+		uniq = atseq.Unique()
+		if u = p.unresolvedSequences[uniq]; u != nil {
+			unresolved = p.unresolvedSequences
+		} else {
+			// note: sequence-forming assertions are never prerequisites.
+			at := a.At()
+			// a is not tracked as unresolved in any way so far,
+			// this is an atypical scenario where something gets
+			// pushed but we still want to add it to the resolved
+			// lists of the relevant groups; in case it is
+			// actually already resolved most of resolveWith below will
+			// be a nop
+			rec := &unresolvedSeqRec{
+				at: &AtSequence{
+					Type:        a.Type(),
+					SequenceKey: at.PrimaryKey[:len(at.PrimaryKey)-1],
+				},
+			}
+			rec.at.Revision = RevisionNotKnown
+			u = rec
 		}
-		// a is not tracked as unresolved in any way so far,
-		// this is an atypical scenario where something gets
-		// pushed but we still want to add it to the resolved
-		// lists of the relevant groups; in case it is
-		// actually already resolved most of resolveWith below will
-		// be a nop
-		u = &unresolvedRec{
-			at: a.At(),
-		}
-		u.at.Revision = RevisionNotKnown
 	}
 
-	if u.serializedLabel != grouping {
+	if u.label() != grouping {
 		var err error
-		extrag, err = p.groupings.Deserialize(string(grouping))
+		extrag, err = deserializeGrouping(string(grouping))
 		if err != nil {
-			return err
+			return false, err
 		}
 	}
 
 	return p.resolveWith(unresolved, uniq, u, a, extrag)
 }
 
-// TODO: AddBatch
+// AddBatch adds all the assertions in the Batch to the Pool,
+// associated with the given grouping and as resolved in all the
+// groups requiring them. It is equivalent to using Add on each of them.
+// If an error is returned it refers to an immediate or local error.
+// Errors related to the assertions are associated with the relevant groups
+// and can be retrieved with Err, in which case ok set to false.
+func (p *Pool) AddBatch(b *Batch, grouping Grouping) (ok bool, err error) {
+	if err := p.phase(poolPhaseAdd); err != nil {
+		return false, err
+	}
+
+	// b dealt with unsupported formats already
+
+	// deserialize grouping if needed only once
+	var cachedGrouping *internal.Grouping
+	deser := func(_ string) (*internal.Grouping, error) {
+		if cachedGrouping != nil {
+			// do a copy as addToGrouping and resolveWith
+			// might add to their input
+			g := cachedGrouping.Copy()
+			return &g, nil
+		}
+		var err error
+		cachedGrouping, err = p.groupings.Deserialize(string(grouping))
+		return cachedGrouping, err
+	}
+
+	inError := false
+	for _, a := range b.added {
+		ok, err := p.addToGrouping(a, grouping, deser)
+		if err != nil {
+			return false, err
+		}
+		if !ok {
+			inError = true
+		}
+	}
+
+	return !inError, nil
+}
 
 var (
 	ErrUnresolved       = errors.New("unresolved assertion")
@@ -523,22 +755,30 @@ func (p *Pool) unresolvedBookkeeping() {
 	//  * in error
 	//  * unchanged
 	//  * or unresolved
-	for uniq, u := range p.unresolved {
-		e := u.err
-		if e == nil {
-			if u.at.Revision == RevisionNotKnown {
-				e = ErrUnresolved
+	processUnresolved := func(unresolved map[string]unresolvedAssertRecord) {
+		for uniq, ur := range unresolved {
+			e := ur.error()
+			if e == nil {
+				if ur.isRevisionNotKnown() {
+					e = ErrUnresolved
+				} else {
+					// unchanged
+					p.unchanged[uniq] = true
+				}
 			}
+			if e != nil {
+				p.setErr(ur.groupingPtr(), e)
+			}
+			delete(unresolved, uniq)
 		}
-		if e != nil {
-			p.setErr(&u.grouping, e)
-		}
-		delete(p.unresolved, uniq)
 	}
+	processUnresolved(p.unresolved)
+	processUnresolved(p.unresolvedSequences)
 
 	// prerequisites will become the new unresolved but drop them
 	// if all their groups are in error
-	for uniq, prereq := range p.prerequisites {
+	for uniq, pr := range p.prerequisites {
+		prereq := pr.(*unresolvedRec)
 		useful := false
 		p.groupings.Iter(&prereq.grouping, func(gnum uint16) error {
 			if !p.groups[gnum].hasErr() {
@@ -570,6 +810,20 @@ func (p *Pool) Err(group string) error {
 	return gRec.err
 }
 
+// Errors returns a mapping of groups in error to their errors.
+func (p *Pool) Errors() map[string]error {
+	res := make(map[string]error)
+	for _, gRec := range p.groups {
+		if err := gRec.err; err != nil {
+			res[gRec.name] = err
+		}
+	}
+	if len(res) == 0 {
+		return nil
+	}
+	return res
+}
+
 // AddError associates error e with the unresolved assertion.
 // The error will be propagated to all the affected groups at
 // the next ToResolve.
@@ -578,8 +832,23 @@ func (p *Pool) AddError(e error, ref *Ref) error {
 		return err
 	}
 	uniq := ref.Unique()
-	if u := p.unresolved[uniq]; u != nil && u.err == nil {
-		u.err = e
+	if u := p.unresolved[uniq]; u != nil && u.(*unresolvedRec).err == nil {
+		u.(*unresolvedRec).err = e
+	}
+	return nil
+}
+
+// AddSequenceError associates error e with the unresolved sequence-forming
+// assertion.
+// The error will be propagated to all the affected groups at
+// the next ToResolve.
+func (p *Pool) AddSequenceError(e error, atSeq *AtSequence) error {
+	if err := p.phase(poolPhaseAdd); err != nil {
+		return err
+	}
+	uniq := atSeq.Unique()
+	if u := p.unresolvedSequences[uniq]; u != nil && u.(*unresolvedSeqRec).err == nil {
+		u.(*unresolvedSeqRec).err = e
 	}
 	return nil
 }
@@ -608,6 +877,9 @@ func (p *Pool) AddGroupingError(e error, grouping Grouping) error {
 // otherwise if ultimately unresolved they will be assumed to still be
 // at their current ones.
 func (p *Pool) AddToUpdate(toUpdate *Ref, group string) error {
+	if toUpdate.Type.SequenceForming() {
+		return fmt.Errorf("internal error: AddToUpdate requested for sequence-forming assertion")
+	}
 	if err := p.phase(poolPhaseAddUnresolved); err != nil {
 		return err
 	}
@@ -628,9 +900,58 @@ func (p *Pool) AddToUpdate(toUpdate *Ref, group string) error {
 	return nil
 }
 
+// AddSequenceToUpdate adds the assertion referenced by toUpdate and all its
+// prerequisites to the Pool as unresolved and as required by the
+// given group. It is assumed that the assertion is currently in the
+// ground database of the Pool, otherwise this will error.
+// The current revisions of the assertion and its prerequisites will
+// be recorded and only higher revisions will then resolve them,
+// otherwise if ultimately unresolved they will be assumed to still be
+// at their current ones. If toUpdate is pinned, then it will be resolved
+// to the highest revision with same sequence point (toUpdate.Sequence).
+func (p *Pool) AddSequenceToUpdate(toUpdate *AtSequence, group string) error {
+	if err := p.phase(poolPhaseAddUnresolved); err != nil {
+		return err
+	}
+	if toUpdate.Sequence <= 0 {
+		return fmt.Errorf("internal error: sequence to update must have a sequence number set")
+	}
+	if p.unresolvedSequences[toUpdate.Unique()] != nil {
+		return fmt.Errorf("internal error: sequence %v is already being resolved", toUpdate.SequenceKey)
+	}
+	gnum, err := p.ensureGroup(group)
+	if err != nil {
+		return err
+	}
+
+	u := *toUpdate
+	retrieve := func(ref *Ref) (Assertion, error) {
+		return ref.Resolve(p.groundDB.Find)
+	}
+	add := func(a Assertion) error {
+		if !a.Type().SequenceForming() {
+			return p.addUnresolved(a.At(), gnum)
+		}
+		// sequence forming assertions are never predefined, so no check for it.
+		// final add corresponding to toUpdate itself.
+		u.Revision = a.Revision()
+		return p.addUnresolvedSeq(&u, gnum)
+	}
+	f := NewFetcher(p.groundDB, retrieve, add)
+	ref := &Ref{
+		Type:       toUpdate.Type,
+		PrimaryKey: append(u.SequenceKey, fmt.Sprintf("%d", u.Sequence)),
+	}
+	if err := f.Fetch(ref); err != nil {
+		return err
+	}
+	return nil
+}
+
 // CommitTo adds the assertions from groups without errors to the
 // given assertion database. Commit errors can be retrieved via Err
-// per group.
+// per group. An error is returned directly only if CommitTo is called
+// with possible pending unresolved assertions.
 func (p *Pool) CommitTo(db *Database) error {
 	if p.curPhase == poolPhaseAddUnresolved {
 		return fmt.Errorf("internal error: cannot commit Pool during add unresolved phase")
@@ -677,3 +998,25 @@ NextGroup:
 
 	return nil
 }
+
+// ClearGroups clears the pool in terms of information associated with groups
+// while preserving information about already resolved or unchanged assertions.
+// It is useful for reusing a pool once the maximum of usable groups
+// that was set with NewPool has been exhausted. Group errors must be
+// queried before calling it otherwise they are lost. It is an error
+// to call it when there are still pending unresolved assertions in
+// the pool.
+func (p *Pool) ClearGroups() error {
+	if len(p.unresolved) != 0 || len(p.prerequisites) != 0 {
+		return fmt.Errorf("internal error: trying to clear groups of asserts.Pool with pending unresolved or prerequisites")
+	}
+
+	p.numbering = make(map[string]uint16)
+	// use a fresh Groupings as well so that max group tracking starts
+	// from scratch.
+	// NewGroupings cannot fail on a value accepted by it previously
+	p.groupings, _ = internal.NewGroupings(p.groupings.N())
+	p.groups = make(map[uint16]*groupRec)
+	p.curPhase = poolPhaseAdd
+	return nil
+}
