@@ -1094,17 +1094,16 @@ func (s *bootchainSuite) TestBootAssetsToLoadChainWithAlternativeChains(c *C) {
 	}
 
 	// mock relevant files in cache
-	for _, name := range []string{
-		"recovery-bl/shim-hash0", "recovery-bl/shim-hash1",
-		"recovery-bl/loader-recovery-hash0",
-		"recovery-bl/loader-recovery-hash1",
-		"run-bl/loader-run-hash0",
-		"run-bl/loader-run-hash1",
-	} {
-		p := filepath.Join(dirs.SnapBootAssetsDir, name)
-		c.Assert(os.MkdirAll(filepath.Dir(p), 0755), IsNil)
-		c.Assert(ioutil.WriteFile(p, nil, 0644), IsNil)
-	}
+	mockAssetsCache(c, s.rootDir, "recovery-bl", []string{
+		"shim-hash0",
+		"shim-hash1",
+		"loader-recovery-hash0",
+		"loader-recovery-hash1",
+	})
+	mockAssetsCache(c, s.rootDir, "run-bl", []string{
+		"loader-run-hash0",
+		"loader-run-hash1",
+	})
 
 	blNames := map[bootloader.Role]string{
 		bootloader.RoleRecovery: "recovery-bl",
