@@ -24,6 +24,7 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/bootloader"
+	"github.com/snapcore/snapd/kernel/fde"
 	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/seed"
 	"github.com/snapcore/snapd/snap"
@@ -65,9 +66,12 @@ var (
 	ResealKeyToModeenv              = resealKeyToModeenv
 	RecoveryBootChainsForSystems    = recoveryBootChainsForSystems
 	SealKeyModelParams              = sealKeyModelParams
+
+	BootVarsForTrustedCommandLineFromGadget = bootVarsForTrustedCommandLineFromGadget
 )
 
 type BootAssetsMap = bootAssetsMap
+type BootCommandLines = bootCommandLines
 type TrackedAsset = trackedAsset
 
 func (t *TrackedAsset) Equals(blName, name, hash string) error {
@@ -104,6 +108,14 @@ func MockSecbootSealKeys(f func(keys []secboot.SealKeyRequest, params *secboot.S
 	}
 }
 
+func MockSecbootSealKeysWithFDESetupHook(f func(runHook fde.RunSetupHookFunc, keys []secboot.SealKeyRequest, params *secboot.SealKeysWithFDESetupHookParams) error) (restore func()) {
+	old := secbootSealKeysWithFDESetupHook
+	secbootSealKeysWithFDESetupHook = f
+	return func() {
+		secbootSealKeysWithFDESetupHook = old
+	}
+}
+
 func MockSecbootResealKeys(f func(params *secboot.ResealKeysParams) error) (restore func()) {
 	old := secbootResealKeys
 	secbootResealKeys = f
@@ -153,8 +165,32 @@ var (
 	WriteBootChains                     = writeBootChains
 	ReadBootChains                      = readBootChains
 	IsResealNeeded                      = isResealNeeded
+
+	SetImageBootFlags = setImageBootFlags
+	NextBootFlags     = nextBootFlags
+	SetNextBootFlags  = setNextBootFlags
 )
 
+func SetBootFlagsInBootloader(flags []string, rootDir string) error {
+	blVars := make(map[string]string, 1)
+
+	if err := setImageBootFlags(flags, blVars); err != nil {
+		return err
+	}
+
+	// now find the recovery bootloader in the system dir and set the value on
+	// it
+	opts := &bootloader.Options{
+		Role: bootloader.RoleRecovery,
+	}
+	bl, err := bootloader.Find(rootDir, opts)
+	if err != nil {
+		return err
+	}
+
+	return bl.SetBootVars(blVars)
+}
+
 func (b *bootChain) SetModelAssertion(model *asserts.Model) {
 	b.model = model
 }
@@ -175,7 +211,7 @@ func MockHasFDESetupHook(f func() (bool, error)) (restore func()) {
 	}
 }
 
-func MockRunFDESetupHook(f func(string, *FDESetupHookParams) ([]byte, error)) (restore func()) {
+func MockRunFDESetupHook(f fde.RunSetupHookFunc) (restore func()) {
 	oldRunFDESetupHook := RunFDESetupHook
 	RunFDESetupHook = f
 	return func() { RunFDESetupHook = oldRunFDESetupHook }
@@ -188,3 +224,11 @@ func MockResealKeyToModeenvUsingFDESetupHook(f func(string, *asserts.Model, *Mod
 		resealKeyToModeenvUsingFDESetupHook = old
 	}
 }
+
+func MockAdditionalBootFlags(bootFlags []string) (restore func()) {
+	old := understoodBootFlags
+	understoodBootFlags = append(understoodBootFlags, bootFlags...)
+	return func() {
+		understoodBootFlags = old
+	}
+}
