@@ -38,6 +38,10 @@ var (
 	// during the initramfs, typically used in recover mode.
 	InitramfsHostUbuntuDataDir string
 
+	// InitramfsHostWritableDir is the location of the host writable
+	// partition during the initramfs, typically used in recover mode.
+	InitramfsHostWritableDir string
+
 	// InitramfsUbuntuBootDir is the location of ubuntu-boot during the
 	// initramfs.
 	InitramfsUbuntuBootDir string
@@ -46,6 +50,10 @@ var (
 	// initramfs.
 	InitramfsUbuntuSeedDir string
 
+	// InitramfsUbuntuSaveDir is the location of ubuntu-save during the
+	// initramfs.
+	InitramfsUbuntuSaveDir string
+
 	// InitramfsWritableDir is the location of the writable partition during the
 	// initramfs. Note that this may refer to a temporary filesystem or a
 	// physical partition depending on what system mode the system is in.
@@ -59,21 +67,40 @@ var (
 	// InstallHostFDEDataDir is the location of the FDE data during install mode.
 	InstallHostFDEDataDir string
 
-	// InitramfsEncryptionKeyDir is the location of the encrypted partition keys
-	// during the initramfs.
-	InitramfsEncryptionKeyDir string
+	// InstallHostFDESaveDir is the directory of the FDE data on the
+	// ubuntu-save partition during install mode. For other modes,
+	// use dirs.SnapSaveFDEDirUnder().
+	InstallHostFDESaveDir string
+
+	// InitramfsSeedEncryptionKeyDir is the location of the encrypted partition
+	// keys during the initramfs on ubuntu-seed.
+	InitramfsSeedEncryptionKeyDir string
+
+	// InitramfsBootEncryptionKeyDir is the location of the encrypted partition
+	// keys during the initramfs on ubuntu-boot.
+	InitramfsBootEncryptionKeyDir string
+
+	// snapBootFlagsFile is the location of the file that is used
+	// internally for saving the current boot flags active for this boot.
+	snapBootFlagsFile string
 )
 
 func setInitramfsDirVars(rootdir string) {
 	InitramfsRunMntDir = filepath.Join(rootdir, "run/mnt")
 	InitramfsDataDir = filepath.Join(InitramfsRunMntDir, "data")
 	InitramfsHostUbuntuDataDir = filepath.Join(InitramfsRunMntDir, "host", "ubuntu-data")
+	InitramfsHostWritableDir = filepath.Join(InitramfsHostUbuntuDataDir, "system-data")
 	InitramfsUbuntuBootDir = filepath.Join(InitramfsRunMntDir, "ubuntu-boot")
 	InitramfsUbuntuSeedDir = filepath.Join(InitramfsRunMntDir, "ubuntu-seed")
+	InitramfsUbuntuSaveDir = filepath.Join(InitramfsRunMntDir, "ubuntu-save")
 	InstallHostWritableDir = filepath.Join(InitramfsRunMntDir, "ubuntu-data", "system-data")
 	InstallHostFDEDataDir = dirs.SnapFDEDirUnder(InstallHostWritableDir)
+	InstallHostFDESaveDir = filepath.Join(InitramfsUbuntuSaveDir, "device/fde")
 	InitramfsWritableDir = filepath.Join(InitramfsDataDir, "system-data")
-	InitramfsEncryptionKeyDir = filepath.Join(InitramfsUbuntuSeedDir, "device/fde")
+	InitramfsSeedEncryptionKeyDir = filepath.Join(InitramfsUbuntuSeedDir, "device/fde")
+	InitramfsBootEncryptionKeyDir = filepath.Join(InitramfsUbuntuBootDir, "device/fde")
+
+	snapBootFlagsFile = filepath.Join(dirs.SnapRunDir, "boot-flags")
 }
 
 func init() {
