@@ -22,6 +22,7 @@ package client
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
@@ -29,6 +30,8 @@ import (
 	"strconv"
 	"strings"
 	"time"
+
+	"github.com/snapcore/snapd/snap"
 )
 
 // AppActivator is a thing that activates the app that is a service in the
@@ -43,14 +46,15 @@ type AppActivator struct {
 
 // AppInfo describes a single snap application.
 type AppInfo struct {
-	Snap        string         `json:"snap,omitempty"`
-	Name        string         `json:"name"`
-	DesktopFile string         `json:"desktop-file,omitempty"`
-	Daemon      string         `json:"daemon,omitempty"`
-	Enabled     bool           `json:"enabled,omitempty"`
-	Active      bool           `json:"active,omitempty"`
-	CommonID    string         `json:"common-id,omitempty"`
-	Activators  []AppActivator `json:"activators,omitempty"`
+	Snap        string           `json:"snap,omitempty"`
+	Name        string           `json:"name"`
+	DesktopFile string           `json:"desktop-file,omitempty"`
+	Daemon      string           `json:"daemon,omitempty"`
+	DaemonScope snap.DaemonScope `json:"daemon-scope,omitempty"`
+	Enabled     bool             `json:"enabled,omitempty"`
+	Active      bool             `json:"active,omitempty"`
+	CommonID    string           `json:"common-id,omitempty"`
+	Activators  []AppActivator   `json:"activators,omitempty"`
 }
 
 // IsService returns true if the application is a background daemon.
@@ -119,7 +123,7 @@ func (client *Client) Logs(names []string, opts LogOptions) (<-chan Log, error)
 		query.Set("follow", strconv.FormatBool(opts.Follow))
 	}
 
-	rsp, err := client.raw("GET", "/v2/logs", query, nil, nil)
+	rsp, err := client.raw(context.Background(), "GET", "/v2/logs", query, nil, nil)
 	if err != nil {
 		return nil, err
 	}
