@@ -64,6 +64,7 @@ var SeccompSyscalls = []string{
 	"clone",
 	"clone3",
 	"close",
+	"close_range",
 	"connect",
 	"copy_file_range",
 	"creat",
@@ -77,6 +78,7 @@ var SeccompSyscalls = []string{
 	"epoll_ctl",
 	"epoll_ctl_old",
 	"epoll_pwait",
+	"epoll_pwait2",
 	"epoll_wait",
 	"epoll_wait_old",
 	"eventfd",
@@ -86,6 +88,7 @@ var SeccompSyscalls = []string{
 	"exit",
 	"exit_group",
 	"faccessat",
+	"faccessat2",
 	"fadvise64",
 	"fadvise64_64",
 	"fallocate",
@@ -223,6 +226,7 @@ var SeccompSyscalls = []string{
 	"mmap2",
 	"modify_ldt",
 	"mount",
+	"mount_setattr",
 	"move_mount",
 	"move_pages",
 	"mprotect",
@@ -283,6 +287,7 @@ var SeccompSyscalls = []string{
 	"preadv",
 	"preadv2",
 	"prlimit64",
+	"process_madvise",
 	"process_vm_readv",
 	"process_vm_writev",
 	"prof",
