@@ -51,7 +51,7 @@ type snapOpTestServer struct {
 	channel         string
 	trackingChannel string
 	confinement     string
-	rebooting       bool
+	restart         string
 	snap            string
 }
 
@@ -71,10 +71,10 @@ func (t *snapOpTestServer) handle(w http.ResponseWriter, r *http.Request) {
 	case 1:
 		t.c.Check(r.Method, check.Equals, "GET")
 		t.c.Check(r.URL.Path, check.Equals, "/v2/changes/42")
-		if !t.rebooting {
+		if t.restart == "" {
 			fmt.Fprintln(w, `{"type": "sync", "result": {"status": "Doing"}}`)
 		} else {
-			fmt.Fprintln(w, `{"type": "sync", "result": {"status": "Doing"}, "maintenance": {"kind": "system-restart", "message": "system is restarting"}}}`)
+			fmt.Fprintln(w, fmt.Sprintf(`{"type": "sync", "result": {"status": "Doing"}, "maintenance": {"kind": "system-restart", "message": "system is %sing", "value": {"op": %q}}}}`, t.restart, t.restart))
 		}
 	case 2:
 		t.c.Check(r.Method, check.Equals, "GET")
@@ -101,7 +101,7 @@ type SnapOpSuite struct {
 func (s *SnapOpSuite) SetUpTest(c *check.C) {
 	s.BaseSnapSuite.SetUpTest(c)
 
-	restoreClientRetry := client.MockDoTimings(time.Millisecond, 100*time.Millisecond)
+	restoreClientRetry := client.MockDoTimings(time.Millisecond, time.Second)
 	restorePollTime := snap.MockPollTime(time.Millisecond)
 	s.restoreAll = func() {
 		restoreClientRetry()
@@ -1322,7 +1322,7 @@ func (s *SnapOpSuite) TestRefreshOneRebooting(c *check.C) {
 			"action": "refresh",
 		})
 	}
-	s.srv.rebooting = true
+	s.srv.restart = "reboot"
 
 	restore := mockArgs("snap", "refresh", "core")
 	defer restore()
@@ -1330,7 +1330,44 @@ func (s *SnapOpSuite) TestRefreshOneRebooting(c *check.C) {
 	err := snap.RunMain()
 	c.Check(err, check.IsNil)
 	c.Check(s.Stderr(), check.Equals, "snapd is about to reboot the system\n")
+}
 
+func (s *SnapOpSuite) TestRefreshOneHalting(c *check.C) {
+	s.RedirectClientToTestServer(s.srv.handle)
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.Method, check.Equals, "POST")
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/core")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action": "refresh",
+		})
+	}
+	s.srv.restart = "halt"
+
+	restore := mockArgs("snap", "refresh", "core")
+	defer restore()
+
+	err := snap.RunMain()
+	c.Check(err, check.IsNil)
+	c.Check(s.Stderr(), check.Equals, "snapd is about to halt the system\n")
+}
+
+func (s *SnapOpSuite) TestRefreshOnePoweringOff(c *check.C) {
+	s.RedirectClientToTestServer(s.srv.handle)
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.Method, check.Equals, "POST")
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/core")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action": "refresh",
+		})
+	}
+	s.srv.restart = "poweroff"
+
+	restore := mockArgs("snap", "refresh", "core")
+	defer restore()
+
+	err := snap.RunMain()
+	c.Check(err, check.IsNil)
+	c.Check(s.Stderr(), check.Equals, "snapd is about to power off the system\n")
 }
 
 func (s *SnapOpSuite) TestRefreshOneChanDeprecated(c *check.C) {
