@@ -98,6 +98,10 @@ func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.AddCleanup(snap.MockIsStdinTTY(false))
 
 	s.AddCleanup(snap.MockSELinuxIsEnabled(func() (bool, error) { return false, nil }))
+
+	// mock an empty cmdline since we check the cmdline to check whether we are
+	// in install mode or not and we don't want to use the host's proc/cmdline
+	s.AddCleanup(osutil.MockProcCmdline(filepath.Join(c.MkDir(), "proc/cmdline")))
 }
 
 func (s *BaseSnapSuite) TearDownTest(c *C) {
@@ -338,9 +342,15 @@ func (s *SnapSuite) TestLintDesc(c *C) {
 	c.Check(log.String(), HasLen, 0)
 	log.Reset()
 
-	// LintDesc complains about lowercase description.
+	// LintDesc complains about lowercase description and mentions the locale
+	// that the system is currently in.
+	prevValue := os.Getenv("LC_MESSAGES")
+	os.Setenv("LC_MESSAGES", "en_US")
+	defer func() {
+		os.Setenv("LC_MESSAGES", prevValue)
+	}()
 	snap.LintDesc("command", "<option>", "description", "")
-	c.Check(log.String(), testutil.Contains, `description of command's "<option>" is lowercase: "description"`)
+	c.Check(log.String(), testutil.Contains, `description of command's "<option>" is lowercase in locale "en_US": "description"`)
 	log.Reset()
 
 	// LintDesc does not complain about lowercase description starting with login.ubuntu.com
