@@ -32,32 +32,26 @@ import (
 
 var (
 	serialModelCmd = &Command{
-		Path:   "/v2/model/serial",
-		GET:    getSerial,
-		UserOK: true,
+		Path:       "/v2/model/serial",
+		GET:        getSerial,
+		ReadAccess: openAccess{},
 	}
 	modelCmd = &Command{
-		Path:   "/v2/model",
-		POST:   postModel,
-		GET:    getModel,
-		UserOK: true,
+		Path:        "/v2/model",
+		POST:        postModel,
+		GET:         getModel,
+		ReadAccess:  openAccess{},
+		WriteAccess: rootAccess{},
 	}
 )
 
-type assertType int
-
-const (
-	serialType assertType = iota
-	modelType
-)
-
 var devicestateRemodel = devicestate.Remodel
 
 type postModelData struct {
 	NewModel string `json:"new-model"`
 }
 
-type modelAssertJSONResponse struct {
+type modelAssertJSON struct {
 	Headers map[string]interface{} `json:"headers,omitempty"`
 	Body    string                 `json:"body,omitempty"`
 }
@@ -124,7 +118,7 @@ func getModel(c *Command, r *http.Request, _ *auth.UserState) Response {
 	}
 
 	if opts.jsonResult {
-		modelJSON := modelAssertJSONResponse{}
+		modelJSON := modelAssertJSON{}
 
 		modelJSON.Headers = model.Headers()
 		if !opts.headersOnly {
@@ -134,7 +128,7 @@ func getModel(c *Command, r *http.Request, _ *auth.UserState) Response {
 		return SyncResponse(modelJSON, nil)
 	}
 
-	return AssertResponse([]asserts.Assertion{model}, true)
+	return AssertResponse([]asserts.Assertion{model}, false)
 }
 
 // getSerial gets the current serial assertion using the DeviceManager
@@ -169,7 +163,7 @@ func getSerial(c *Command, r *http.Request, _ *auth.UserState) Response {
 	}
 
 	if opts.jsonResult {
-		serialJSON := modelAssertJSONResponse{}
+		serialJSON := modelAssertJSON{}
 
 		serialJSON.Headers = serial.Headers()
 		if !opts.headersOnly {
@@ -179,5 +173,5 @@ func getSerial(c *Command, r *http.Request, _ *auth.UserState) Response {
 		return SyncResponse(serialJSON, nil)
 	}
 
-	return AssertResponse([]asserts.Assertion{serial}, true)
+	return AssertResponse([]asserts.Assertion{serial}, false)
 }
