@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2020 Canonical Ltd
+ * Copyright (C) 2015-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,14 +20,15 @@
 package daemon
 
 import (
+	"bytes"
 	"context"
+	"encoding/json"
 	"fmt"
 	"net"
 	"net/http"
 	"os"
 	"os/exec"
 	"os/signal"
-	"strconv"
 	"strings"
 	"sync"
 	"time"
@@ -35,7 +36,6 @@ import (
 	"github.com/gorilla/mux"
 	"gopkg.in/tomb.v2"
 
-	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/logger"
@@ -45,7 +45,6 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/standby"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/polkit"
 	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/systemd"
@@ -55,6 +54,14 @@ var ErrRestartSocket = fmt.Errorf("daemon stop requested to wait for socket acti
 
 var systemdSdNotify = systemd.SdNotify
 
+const (
+	daemonRestartMsg  = "daemon is restarting"
+	systemRestartMsg  = "system is restarting"
+	systemHaltMsg     = "system is halting"
+	systemPoweroffMsg = "system is powering off"
+	socketRestartMsg  = "daemon is stopping to wait for socket activation"
+)
+
 // A Daemon listens for requests and routes them to the right command
 type Daemon struct {
 	Version         string
@@ -68,8 +75,8 @@ type Daemon struct {
 	router          *mux.Router
 	standbyOpinions *standby.StandbyOpinions
 
-	// set to remember we need to restart the system
-	restartSystem state.RestartType
+	// set to what kind of restart was requested if any
+	requestedRestart state.RestartType
 	// set to remember that we need to exit the daemon in a way that
 	// prevents systemd from restarting it
 	restartSocket bool
@@ -92,130 +99,19 @@ type Command struct {
 	GET  ResponseFunc
 	PUT  ResponseFunc
 	POST ResponseFunc
-	// can guest GET?
-	GuestOK bool
-	// can non-admin GET?
-	UserOK bool
-	// is this path accessible on the snapd-snap socket?
-	SnapOK bool
-	// this path is only accessible to root
-	RootOnly bool
 
-	// can polkit grant access? set to polkit action ID if so
-	PolkitOK string
+	// Access control.
+	ReadAccess  accessChecker
+	WriteAccess accessChecker
 
 	d *Daemon
 }
 
-type accessResult int
-
-const (
-	accessOK accessResult = iota
-	accessUnauthorized
-	accessForbidden
-	accessCancelled
-)
-
-var polkitCheckAuthorization = polkit.CheckAuthorization
-
-// canAccess checks the following properties:
-//
-// - if the user is `root` everything is allowed
-// - if a user is logged in (via `snap login`) and the command doesn't have RootOnly, everything is allowed
-// - POST/PUT all require `root`, or just `snap login` if not RootOnly
-//
-// Otherwise for GET requests the following parameters are honored:
-// - GuestOK: anyone can access GET
-// - UserOK: any uid on the local system can access GET
-// - RootOnly: only root can access this
-// - SnapOK: a snap can access this via `snapctl`
-func (c *Command) canAccess(r *http.Request, user *auth.UserState) accessResult {
-	if c.RootOnly && (c.UserOK || c.GuestOK || c.SnapOK) {
-		// programming error
-		logger.Panicf("Command can't have RootOnly together with any *OK flag")
-	}
-
-	if user != nil && !c.RootOnly {
-		// Authenticated users do anything not requiring explicit root.
-		return accessOK
-	}
-
-	// isUser means we have a UID for the request
-	isUser := false
-	pid, uid, socket, err := ucrednetGet(r.RemoteAddr)
-	if err == nil {
-		isUser = true
-	} else if err != errNoID {
-		logger.Noticef("unexpected error when attempting to get UID: %s", err)
-		return accessForbidden
-	}
-	isSnap := (socket == dirs.SnapSocket)
-
-	// ensure that snaps can only access SnapOK things
-	if isSnap {
-		if c.SnapOK {
-			return accessOK
-		}
-		return accessUnauthorized
-	}
-
-	// the !RootOnly check is redundant, but belt-and-suspenders
-	if r.Method == "GET" && !c.RootOnly {
-		// Guest and user access restricted to GET requests
-		if c.GuestOK {
-			return accessOK
-		}
-
-		if isUser && c.UserOK {
-			return accessOK
-		}
-	}
-
-	// Remaining admin checks rely on identifying peer uid
-	if !isUser {
-		return accessUnauthorized
-	}
-
-	if uid == 0 {
-		// Superuser does anything.
-		return accessOK
-	}
-
-	if c.RootOnly {
-		return accessUnauthorized
-	}
-
-	if c.PolkitOK != "" {
-		var flags polkit.CheckFlags
-		allowHeader := r.Header.Get(client.AllowInteractionHeader)
-		if allowHeader != "" {
-			if allow, err := strconv.ParseBool(allowHeader); err != nil {
-				logger.Noticef("error parsing %s header: %s", client.AllowInteractionHeader, err)
-			} else if allow {
-				flags |= polkit.CheckAllowInteraction
-			}
-		}
-		// Pass both pid and uid from the peer ucred to avoid pid race
-		if authorized, err := polkitCheckAuthorization(pid, uid, c.PolkitOK, nil, flags); err == nil {
-			if authorized {
-				// polkit says user is authorised
-				return accessOK
-			}
-		} else if err == polkit.ErrDismissed {
-			return accessCancelled
-		} else {
-			logger.Noticef("polkit error: %s", err)
-		}
-	}
-
-	return accessUnauthorized
-}
-
 func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	st := c.d.state
 	st.Lock()
 	// TODO Look at the error and fail if there's an attempt to authenticate with invalid data.
-	user, _ := UserFromRequest(st, r)
+	user, _ := userFromRequest(st, r)
 	st.Unlock()
 
 	// check if we are in degradedMode
@@ -224,17 +120,10 @@ func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	switch c.canAccess(r, user) {
-	case accessOK:
-		// nothing
-	case accessUnauthorized:
-		Unauthorized("access denied").ServeHTTP(w, r)
-		return
-	case accessForbidden:
-		Forbidden("forbidden").ServeHTTP(w, r)
-		return
-	case accessCancelled:
-		AuthCancelled("cancelled").ServeHTTP(w, r)
+	ucred, err := ucrednetGet(r.RemoteAddr)
+	if err != nil && err != errNoID {
+		logger.Noticef("unexpected error when attempting to get UID: %s", err)
+		InternalError(err.Error()).ServeHTTP(w, r)
 		return
 	}
 
@@ -242,31 +131,47 @@ func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	r = r.WithContext(ctx)
 
 	var rspf ResponseFunc
-	var rsp = MethodNotAllowed("method %q not allowed", r.Method)
+	var access accessChecker
 
 	switch r.Method {
 	case "GET":
 		rspf = c.GET
+		access = c.ReadAccess
 	case "PUT":
 		rspf = c.PUT
+		access = c.WriteAccess
 	case "POST":
 		rspf = c.POST
+		access = c.WriteAccess
 	}
 
-	if rspf != nil {
-		rsp = rspf(c, r, user)
+	if rspf == nil {
+		MethodNotAllowed("method %q not allowed", r.Method).ServeHTTP(w, r)
+		return
 	}
 
+	switch access.CheckAccess(r, ucred, user) {
+	case accessOK:
+		// nothing
+	case accessUnauthorized:
+		Unauthorized("access denied").ServeHTTP(w, r)
+		return
+	case accessForbidden:
+		Forbidden("access denied").ServeHTTP(w, r)
+		return
+	case accessCancelled:
+		AuthCancelled("cancelled").ServeHTTP(w, r)
+		return
+	}
+
+	rsp := rspf(c, r, user)
+
 	if rsp, ok := rsp.(*resp); ok {
 		_, rst := st.Restarting()
-		switch rst {
-		case state.RestartSystem, state.RestartSystemNow:
-			rsp.transmitMaintenance(client.ErrorKindSystemRestart, "system is restarting")
-		case state.RestartDaemon:
-			rsp.transmitMaintenance(client.ErrorKindDaemonRestart, "daemon is restarting")
-		case state.RestartSocket:
-			rsp.transmitMaintenance(client.ErrorKindDaemonRestart, "daemon is stopping to wait for socket activation")
+		if rst != state.RestartUnset {
+			rsp.Maintenance = maintenanceForRestartType(rst)
 		}
+
 		if rsp.Type != ResponseTypeError {
 			st.Lock()
 			count, stamp := st.WarningsSummary()
@@ -307,7 +212,7 @@ func logit(handler http.Handler) http.Handler {
 		ww := &wrappedWriter{w: w}
 		t0 := time.Now()
 		handler.ServeHTTP(ww, r)
-		t := time.Now().Sub(t0)
+		t := time.Since(t0)
 		url := r.URL.String()
 		if !strings.Contains(url, "/changes/") {
 			logger.Debugf("%s %s %s %s %d", r.RemoteAddr, r.Method, r.URL, t, ww.s)
@@ -450,6 +355,13 @@ func (d *Daemon) Start() error {
 	// enable standby handling
 	d.initStandbyHandling()
 
+	// before serving actual connections remove the maintenance.json file as we
+	// are no longer down for maintenance, this state most closely corresponds
+	// to state.RestartUnset
+	if err := d.updateMaintenanceFile(state.RestartUnset); err != nil {
+		return err
+	}
+
 	// the loop runs in its own goroutine
 	d.overlord.Loop()
 
@@ -478,29 +390,43 @@ func (d *Daemon) Start() error {
 
 // HandleRestart implements overlord.RestartBehavior.
 func (d *Daemon) HandleRestart(t state.RestartType) {
+	d.mu.Lock()
+	defer d.mu.Unlock()
+
+	scheduleFallback := func(a rebootAction) {
+		if err := reboot(a, rebootWaitTimeout); err != nil {
+			logger.Noticef("%s", err)
+		}
+	}
+
 	// die when asked to restart (systemd should get us back up!) etc
 	switch t {
 	case state.RestartDaemon:
+		// save the restart kind to write out a maintenance.json in a bit
+		d.requestedRestart = t
 	case state.RestartSystem, state.RestartSystemNow:
 		// try to schedule a fallback slow reboot already here
 		// in case we get stuck shutting down
-		if err := reboot(rebootWaitTimeout); err != nil {
-			logger.Noticef("%s", err)
-		}
 
-		d.mu.Lock()
-		defer d.mu.Unlock()
-		// remember we need to restart the system
-		d.restartSystem = t
+		// save the restart kind to write out a maintenance.json in a bit
+		scheduleFallback(rebootReboot)
+		d.requestedRestart = t
+	case state.RestartSystemHaltNow:
+		scheduleFallback(rebootHalt)
+		d.requestedRestart = t
+	case state.RestartSystemPoweroffNow:
+		scheduleFallback(rebootPoweroff)
+		d.requestedRestart = t
 	case state.RestartSocket:
-		d.mu.Lock()
-		defer d.mu.Unlock()
+		// save the restart kind to write out a maintenance.json in a bit
+		d.requestedRestart = t
 		d.restartSocket = true
 	case state.StopDaemon:
 		logger.Noticef("stopping snapd as requested")
 	default:
 		logger.Noticef("internal error: restart handler called with unknown restart type: %v", t)
 	}
+
 	d.tomb.Kill(nil)
 }
 
@@ -511,13 +437,34 @@ var (
 	rebootMaxTentatives    = 3
 )
 
+func (d *Daemon) updateMaintenanceFile(rst state.RestartType) error {
+	// for unset restart, just remove the maintenance.json file
+	if rst == state.RestartUnset {
+		err := os.Remove(dirs.SnapdMaintenanceFile)
+		// only return err if the error was something other than the file not
+		// existing
+		if err != nil && !os.IsNotExist(err) {
+			return err
+		}
+		return nil
+	}
+
+	// otherwise marshal and write it out appropriately
+	b, err := json.Marshal(maintenanceForRestartType(rst))
+	if err != nil {
+		return err
+	}
+
+	return osutil.AtomicWrite(dirs.SnapdMaintenanceFile, bytes.NewBuffer(b), 0644, 0)
+}
+
 // Stop shuts down the Daemon
 func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 	// we need to schedule/wait for a system restart again
 	if d.expectedRebootDidNotHappen {
 		// make the reboot retry immediate
 		immediateReboot := true
-		return d.doReboot(sigCh, immediateReboot, rebootRetryWaitTimeout)
+		return d.doReboot(sigCh, state.RestartSystem, immediateReboot, rebootRetryWaitTimeout)
 	}
 	if d.overlord == nil {
 		return fmt.Errorf("internal error: no Overlord")
@@ -525,12 +472,31 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 
 	d.tomb.Kill(nil)
 
+	// check the state associated with a potential restart with the lock to
+	// prevent races
 	d.mu.Lock()
-	restartSystem := d.restartSystem != state.RestartUnset
-	immediateReboot := d.restartSystem == state.RestartSystemNow
+	// needsFullShutdown is whether the entire system will
+	// shutdown or not as a consequence of this request
+	needsFullShutdown := false
+	switch d.requestedRestart {
+	case state.RestartSystem, state.RestartSystemNow, state.RestartSystemHaltNow, state.RestartSystemPoweroffNow:
+		needsFullShutdown = true
+	}
+	immediateShutdown := false
+	switch d.requestedRestart {
+	case state.RestartSystemNow, state.RestartSystemHaltNow, state.RestartSystemPoweroffNow:
+		immediateShutdown = true
+	}
 	restartSocket := d.restartSocket
 	d.mu.Unlock()
 
+	// before not accepting any new client connections we need to write the
+	// maintenance.json file for potential clients to see after the daemon stops
+	// responding so they can read it correctly and handle the maintenance
+	if err := d.updateMaintenanceFile(d.requestedRestart); err != nil {
+		logger.Noticef("error writing maintenance file: %v", err)
+	}
+
 	d.snapdListener.Close()
 	d.standbyOpinions.Stop()
 
@@ -547,7 +513,7 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 		d.snapListener.Close()
 	}
 
-	if restartSystem {
+	if needsFullShutdown {
 		// give time to polling clients to notice restart
 		time.Sleep(rebootNoticeWait)
 	}
@@ -559,10 +525,9 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 	d.tomb.Kill(d.serve.Shutdown(ctx))
 	cancel()
 
-	if !restartSystem {
+	if !needsFullShutdown {
 		// tell systemd that we are stopping
 		systemdSdNotify("STOPPING=1")
-
 	}
 
 	if restartSocket {
@@ -580,8 +545,7 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 	}
 	d.overlord.Stop()
 
-	err := d.tomb.Wait()
-	if err != nil {
+	if err := d.tomb.Wait(); err != nil {
 		if err == context.DeadlineExceeded {
 			logger.Noticef("WARNING: cannot gracefully shut down in-flight snapd API activity within: %v", shutdownTimeout)
 			// the process is shutting down anyway, so we may just
@@ -592,7 +556,7 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 			// because we already scheduled a slow shutdown and
 			// exiting here will just restart snapd (via systemd)
 			// which will lead to confusing results.
-			if restartSystem {
+			if needsFullShutdown {
 				logger.Noticef("WARNING: cannot stop daemon: %v", err)
 			} else {
 				return err
@@ -600,8 +564,8 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 		}
 	}
 
-	if restartSystem {
-		return d.doReboot(sigCh, immediateReboot, rebootWaitTimeout)
+	if needsFullShutdown {
+		return d.doReboot(sigCh, d.requestedRestart, immediateShutdown, rebootWaitTimeout)
 	}
 
 	if d.restartSocket {
@@ -641,18 +605,25 @@ func (d *Daemon) rebootDelay(immediate bool) (time.Duration, error) {
 	return rebootDelay, nil
 }
 
-func (d *Daemon) doReboot(sigCh chan<- os.Signal, immediate bool, waitTimeout time.Duration) error {
+func (d *Daemon) doReboot(sigCh chan<- os.Signal, rst state.RestartType, immediate bool, waitTimeout time.Duration) error {
 	rebootDelay, err := d.rebootDelay(immediate)
 	if err != nil {
 		return err
 	}
+	action := rebootReboot
+	switch rst {
+	case state.RestartSystemHaltNow:
+		action = rebootHalt
+	case state.RestartSystemPoweroffNow:
+		action = rebootPoweroff
+	}
 	// ask for shutdown and wait for it to happen.
 	// if we exit snapd will be restared by systemd
-	if err := reboot(rebootDelay); err != nil {
+	if err := reboot(action, rebootDelay); err != nil {
 		return err
 	}
 	// wait for reboot to happen
-	logger.Noticef("Waiting for system reboot")
+	logger.Noticef("Waiting for %s", action)
 	if sigCh != nil {
 		signal.Stop(sigCh)
 		if len(sigCh) > 0 {
@@ -662,17 +633,56 @@ func (d *Daemon) doReboot(sigCh chan<- os.Signal, immediate bool, waitTimeout ti
 		close(sigCh)
 	}
 	time.Sleep(waitTimeout)
-	return fmt.Errorf("expected reboot did not happen")
+	return fmt.Errorf("expected %s did not happen", action)
 }
 
-var shutdownMsg = i18n.G("reboot scheduled to update the system")
+var (
+	shutdownMsg = i18n.G("reboot scheduled to update the system")
+	haltMsg     = i18n.G("system halt scheduled")
+	poweroffMsg = i18n.G("system poweroff scheduled")
+)
 
-func rebootImpl(rebootDelay time.Duration) error {
+type rebootAction int
+
+func (a rebootAction) String() string {
+	switch a {
+	case rebootReboot:
+		return "system reboot"
+	case rebootHalt:
+		return "system halt"
+	case rebootPoweroff:
+		return "system poweroff"
+	default:
+		panic(fmt.Sprintf("unknown reboot action %d", a))
+	}
+}
+
+const (
+	rebootReboot rebootAction = iota
+	rebootHalt
+	rebootPoweroff
+)
+
+func rebootImpl(action rebootAction, rebootDelay time.Duration) error {
 	if rebootDelay < 0 {
 		rebootDelay = 0
 	}
 	mins := int64(rebootDelay / time.Minute)
-	cmd := exec.Command("shutdown", "-r", fmt.Sprintf("+%d", mins), shutdownMsg)
+	var arg, msg string
+	switch action {
+	case rebootReboot:
+		arg = "-r"
+		msg = shutdownMsg
+	case rebootHalt:
+		arg = "--halt"
+		msg = haltMsg
+	case rebootPoweroff:
+		arg = "--poweroff"
+		msg = poweroffMsg
+	default:
+		return fmt.Errorf("unknown reboot action: %v", action)
+	}
+	cmd := exec.Command("shutdown", arg, fmt.Sprintf("+%d", mins), msg)
 	if out, err := cmd.CombinedOutput(); err != nil {
 		return osutil.OutputErr(out, err)
 	}
