@@ -32,7 +32,9 @@ import (
 	"github.com/snapcore/snapd/snap"
 )
 
-var MinLane = minLane
+func APICommands() []*Command {
+	return api
+}
 
 func NewAndAddRoutes() (*Daemon, error) {
 	d, err := New()
@@ -61,7 +63,9 @@ func (d *Daemon) RequestedRestart() state.RestartType {
 	return d.requestedRestart
 }
 
-func MockUcrednetGet(mock func(remoteAddr string) (pid int32, uid uint32, socket string, err error)) (restore func()) {
+type Ucrednet = ucrednet
+
+func MockUcrednetGet(mock func(remoteAddr string) (ucred *Ucrednet, err error)) (restore func()) {
 	oldUcrednetGet := ucrednetGet
 	ucrednetGet = mock
 	return func() {
@@ -125,6 +129,14 @@ func MockSnapstateInstallPath(mock func(*state.State, *snap.SideInfo, string, st
 	}
 }
 
+func MockSnapstateUpdate(mock func(*state.State, string, *snapstate.RevisionOptions, int, snapstate.Flags) (*state.TaskSet, error)) (restore func()) {
+	oldSnapstateUpdate := snapstateUpdate
+	snapstateUpdate = mock
+	return func() {
+		snapstateUpdate = oldSnapstateUpdate
+	}
+}
+
 func MockSnapstateTryPath(mock func(*state.State, string, string, snapstate.Flags) (*state.TaskSet, error)) (restore func()) {
 	oldSnapstateTryPath := snapstateTryPath
 	snapstateTryPath = mock
@@ -133,6 +145,30 @@ func MockSnapstateTryPath(mock func(*state.State, string, string, snapstate.Flag
 	}
 }
 
+func MockSnapstateSwitch(mock func(*state.State, string, *snapstate.RevisionOptions) (*state.TaskSet, error)) (restore func()) {
+	oldSnapstateSwitch := snapstateSwitch
+	snapstateSwitch = mock
+	return func() {
+		snapstateSwitch = oldSnapstateSwitch
+	}
+}
+
+func MockSnapstateRevert(mock func(*state.State, string, snapstate.Flags) (*state.TaskSet, error)) (restore func()) {
+	oldSnapstateRevert := snapstateRevert
+	snapstateRevert = mock
+	return func() {
+		snapstateRevert = oldSnapstateRevert
+	}
+}
+
+func MockSnapstateRevertToRevision(mock func(*state.State, string, snap.Revision, snapstate.Flags) (*state.TaskSet, error)) (restore func()) {
+	oldSnapstateRevertToRevision := snapstateRevertToRevision
+	snapstateRevertToRevision = mock
+	return func() {
+		snapstateRevertToRevision = oldSnapstateRevertToRevision
+	}
+}
+
 func MockSnapstateInstallMany(mock func(*state.State, []string, int) ([]string, []*state.TaskSet, error)) (restore func()) {
 	oldSnapstateInstallMany := snapstateInstallMany
 	snapstateInstallMany = mock
@@ -159,10 +195,17 @@ func MockSnapstateRemoveMany(mock func(*state.State, []string) ([]string, []*sta
 
 type (
 	Resp            = resp
+	RespJSON        = respJSON
+	FileResponse    = fileResponse
 	ErrorResult     = errorResult
 	SnapInstruction = snapInstruction
 )
 
+// XXX this is not used very consistently
+func (rsp *resp) ErrorResult() *errorResult {
+	return rsp.Result.(*errorResult)
+}
+
 func (inst *snapInstruction) Dispatch() snapActionFunc {
 	return inst.dispatch()
 }
@@ -170,3 +213,23 @@ func (inst *snapInstruction) Dispatch() snapActionFunc {
 func (inst *snapInstruction) DispatchForMany() snapManyActionFunc {
 	return inst.dispatchForMany()
 }
+
+func (inst *snapInstruction) SetUserID(userID int) {
+	inst.userID = userID
+}
+
+func (inst *snapInstruction) ModeFlags() (snapstate.Flags, error) {
+	return inst.modeFlags()
+}
+
+func (inst *snapInstruction) ErrToResponse(err error) Response {
+	return inst.errToResponse(err)
+}
+
+var (
+	UserFromRequest = userFromRequest
+	IsTrue          = isTrue
+
+	MakeErrorResponder = makeErrorResponder
+	ErrToResponse      = errToResponse
+)
