@@ -59,6 +59,11 @@ const (
 	CheckDiskSpaceInstall
 	// CheckDiskSpaceRefresh controls free disk space check on snap refresh.
 	CheckDiskSpaceRefresh
+	// GateAutoRefreshHook enables refresh control from snaps via gate-auto-refresh hook.
+	GateAutoRefreshHook
+
+	// QuotaGroups enable creating resource quota groups for snaps via the rest API and cli.
+	QuotaGroups
 
 	// lastFeature is the final known feature, it is only used for testing.
 	lastFeature
@@ -94,6 +99,10 @@ var featureNames = map[SnapdFeature]string{
 	CheckDiskSpaceInstall: "check-disk-space-install",
 	CheckDiskSpaceRefresh: "check-disk-space-refresh",
 	CheckDiskSpaceRemove:  "check-disk-space-remove",
+
+	GateAutoRefreshHook: "gate-auto-refresh-hook",
+
+	QuotaGroups: "quota-groups",
 }
 
 // featuresEnabledWhenUnset contains a set of features that are enabled when not explicitly configured.
