@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2019 Canonical Ltd
+ * Copyright (C) 2019-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,6 +20,8 @@
 package gadget
 
 import (
+	"bufio"
+	"bytes"
 	"errors"
 	"fmt"
 	"io/ioutil"
@@ -35,8 +37,10 @@ import (
 	"github.com/snapcore/snapd/gadget/edition"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/metautil"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/naming"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -52,13 +56,31 @@ const (
 	SystemSeed = "system-seed"
 	SystemSave = "system-save"
 
-	bootImage  = "system-boot-image"
+	// extracted kernels for all uc systems
+	bootImage = "system-boot-image"
+
+	// extracted kernels for recovery kernels for uc20 specifically
+	seedBootImage = "system-seed-image"
+
+	// bootloader specific partition which stores bootloader environment vars
+	// for purposes of booting normal run mode on uc20 and all modes on
+	// uc16 and uc18
 	bootSelect = "system-boot-select"
 
+	// bootloader specific partition which stores bootloader environment vars
+	// for purposes of booting recovery systems on uc20, i.e. recover or install
+	seedBootSelect = "system-seed-select"
+
 	// implicitSystemDataLabel is the implicit filesystem label of structure
 	// of system-data role
 	implicitSystemDataLabel = "writable"
 
+	// UC20 filesystem labels for roles
+	ubuntuBootLabel = "ubuntu-boot"
+	ubuntuSeedLabel = "ubuntu-seed"
+	ubuntuDataLabel = "ubuntu-data"
+	ubuntuSaveLabel = "ubuntu-save"
+
 	// only supported for legacy reasons
 	legacyBootImage  = "bootimg"
 	legacyBootSelect = "bootselect"
@@ -71,7 +93,7 @@ var (
 )
 
 type Info struct {
-	Volumes map[string]Volume `yaml:"volumes,omitempty"`
+	Volumes map[string]*Volume `yaml:"volumes,omitempty"`
 
 	// Default configuration for snaps (snap-id => key => value).
 	Defaults map[string]map[string]interface{} `yaml:"defaults,omitempty"`
@@ -92,13 +114,6 @@ type Volume struct {
 	Structure []VolumeStructure `yaml:"structure"`
 }
 
-func (v *Volume) EffectiveSchema() string {
-	if v.Schema == "" {
-		return schemaGPT
-	}
-	return v.Schema
-}
-
 // VolumeStructure describes a single structure inside a volume. A structure can
 // represent a partition, Master Boot Record, or any other contiguous range
 // within the volume.
@@ -108,7 +123,7 @@ type VolumeStructure struct {
 	// Label provides the filesystem label
 	Label string `yaml:"filesystem-label"`
 	// Offset defines a starting offset of the structure
-	Offset *quantity.Size `yaml:"offset"`
+	Offset *quantity.Offset `yaml:"offset"`
 	// OffsetWrite describes a 32-bit address, within the volume, at which
 	// the offset of current structure will be written. The position may be
 	// specified as a byte offset relative to the start of a named structure
@@ -125,7 +140,7 @@ type VolumeStructure struct {
 	Type string `yaml:"type"`
 	// Role describes the role of given structure, can be one of
 	// 'mbr', 'system-data', 'system-boot', 'system-boot-image',
-	// 'system-boot-select'. Structures of type 'mbr', must have a
+	// 'system-boot-select' or 'system-recovery-select'. Structures of type 'mbr', must have a
 	// size of 446 bytes and must start at 0 offset.
 	Role string `yaml:"role"`
 	// ID is the GPT partition ID
@@ -146,40 +161,16 @@ func (vs *VolumeStructure) HasFilesystem() bool {
 // IsPartition returns true when the structure describes a partition in a block
 // device.
 func (vs *VolumeStructure) IsPartition() bool {
-	return vs.Type != "bare" && vs.EffectiveRole() != schemaMBR
-}
-
-// EffectiveRole returns the role of given structure
-func (vs *VolumeStructure) EffectiveRole() string {
-	if vs.Role != "" {
-		return vs.Role
-	}
-	if vs.Role == "" && vs.Type == schemaMBR {
-		return schemaMBR
-	}
-	if vs.Label == SystemBoot {
-		// for gadgets that only specify a filesystem-label, eg. pc
-		return SystemBoot
-	}
-	return ""
-}
-
-// EffectiveFilesystemLabel returns the effective filesystem label, either
-// explicitly provided or implied by the structure's role
-func (vs *VolumeStructure) EffectiveFilesystemLabel() string {
-	if vs.EffectiveRole() == SystemData {
-		return implicitSystemDataLabel
-	}
-	return vs.Label
+	return vs.Type != "bare" && vs.Role != schemaMBR
 }
 
 // VolumeContent defines the contents of the structure. The content can be
 // either files within a filesystem described by the structure or raw images
 // written into the area of a bare structure.
 type VolumeContent struct {
-	// Source is the data of the partition relative to the gadget base
-	// directory
-	Source string `yaml:"source"`
+	// UnresovedSource is the data of the partition relative to
+	// the gadget base directory
+	UnresolvedSource string `yaml:"source"`
 	// Target is the location of the data inside the root filesystem
 	Target string `yaml:"target"`
 
@@ -187,7 +178,7 @@ type VolumeContent struct {
 	// for a 'bare' type structure
 	Image string `yaml:"image"`
 	// Offset the image is written at
-	Offset *quantity.Size `yaml:"offset"`
+	Offset *quantity.Offset `yaml:"offset"`
 	// OffsetWrite describes a 32-bit address, within the volume, at which
 	// the offset of current image will be written. The position may be
 	// specified as a byte offset relative to the start of a named structure
@@ -203,7 +194,7 @@ func (vc VolumeContent) String() string {
 	if vc.Image != "" {
 		return fmt.Sprintf("image:%s", vc.Image)
 	}
-	return fmt.Sprintf("source:%s", vc.Source)
+	return fmt.Sprintf("source:%s", vc.UnresolvedSource)
 }
 
 type VolumeUpdate struct {
@@ -290,14 +281,14 @@ func systemOrSnapID(s string) bool {
 	return true
 }
 
-// Model carries information about the model that is relevant to gadget.
+// Model carries characteristics about the model that are relevant to gadget.
 // Note *asserts.Model implements this, and that's the expected use case.
 type Model interface {
 	Classic() bool
 	Grade() asserts.ModelGrade
 }
 
-func classicOrUnconstrained(m Model) bool {
+func classicOrUndetermined(m Model) bool {
 	return m == nil || m.Classic()
 }
 
@@ -305,9 +296,11 @@ func wantsSystemSeed(m Model) bool {
 	return m != nil && m.Grade() != asserts.ModelGradeUnset
 }
 
-// InfoFromGadgetYaml reads the provided gadget metadata. If constraints is nil, only the
-// self-consistency checks are performed, otherwise rules for the classic or
-// system seed cases are enforced.
+// InfoFromGadgetYaml parses the provided gadget metadata.
+// If model is nil only self-consistency checks are performed.
+// If model is not nil implied values for filesystem labels will be set
+// as well, based on whether the model is for classic, UC16/18 or UC20.
+// UC gadget metadata is expected to have volumes definitions.
 func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 	var gi Info
 
@@ -336,7 +329,7 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 		}
 	}
 
-	if len(gi.Volumes) == 0 && classicOrUnconstrained(model) {
+	if len(gi.Volumes) == 0 && classicOrUndetermined(model) {
 		// volumes can be left out on classic
 		// can still specify defaults though
 		return &gi, nil
@@ -344,8 +337,9 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 
 	// basic validation
 	var bootloadersFound int
+	knownFsLabelsPerVolume := make(map[string]map[string]bool, len(gi.Volumes))
 	for name, v := range gi.Volumes {
-		if err := validateVolume(name, &v, model); err != nil {
+		if err := validateVolume(name, v, model, knownFsLabelsPerVolume); err != nil {
 			return nil, fmt.Errorf("invalid volume %q: %v", name, err)
 		}
 
@@ -365,12 +359,85 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 		return nil, fmt.Errorf("too many (%d) bootloaders declared", bootloadersFound)
 	}
 
+	for name, v := range gi.Volumes {
+		if err := setImplicitForVolume(name, v, model, knownFsLabelsPerVolume[name]); err != nil {
+			return nil, fmt.Errorf("invalid volume %q: %v", name, err)
+		}
+	}
+
 	return &gi, nil
 }
 
+type volRuleset int
+
+const (
+	volRulesetUnknown volRuleset = iota
+	volRuleset16
+	volRuleset20
+)
+
+func whichVolRuleset(model Model) volRuleset {
+	if model == nil {
+		return volRulesetUnknown
+	}
+	if model.Grade() != asserts.ModelGradeUnset {
+		return volRuleset20
+	}
+	return volRuleset16
+}
+
+func setImplicitForVolume(name string, vol *Volume, model Model, knownFsLabels map[string]bool) error {
+	rs := whichVolRuleset(model)
+	if vol.Schema == "" {
+		// default for schema is gpt
+		vol.Schema = schemaGPT
+	}
+	for i := range vol.Structure {
+		if err := setImplicitForVolumeStructure(&vol.Structure[i], rs, knownFsLabels); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func setImplicitForVolumeStructure(vs *VolumeStructure, rs volRuleset, knownFsLabels map[string]bool) error {
+	if vs.Role == "" && vs.Type == schemaMBR {
+		vs.Role = schemaMBR
+		return nil
+	}
+	if rs == volRuleset16 && vs.Role == "" && vs.Label == SystemBoot {
+		// legacy behavior, for gadgets that only specify a filesystem-label, eg. pc
+		vs.Role = SystemBoot
+		return nil
+	}
+	if vs.Label == "" {
+		var implicitLabel string
+		switch {
+		case rs == volRuleset16 && vs.Role == SystemData:
+			implicitLabel = implicitSystemDataLabel
+		case rs == volRuleset20 && vs.Role == SystemData:
+			implicitLabel = ubuntuDataLabel
+		case rs == volRuleset20 && vs.Role == SystemSeed:
+			implicitLabel = ubuntuSeedLabel
+		case rs == volRuleset20 && vs.Role == SystemBoot:
+			implicitLabel = ubuntuBootLabel
+		case rs == volRuleset20 && vs.Role == SystemSave:
+			implicitLabel = ubuntuSaveLabel
+		}
+		if implicitLabel != "" {
+			if knownFsLabels[implicitLabel] {
+				return fmt.Errorf("filesystem label %q is implied by %s role but was already set elsewhere", implicitLabel, vs.Role)
+			}
+			knownFsLabels[implicitLabel] = true
+			vs.Label = implicitLabel
+		}
+	}
+	return nil
+}
+
 func readInfo(f func(string) ([]byte, error), gadgetYamlFn string, model Model) (*Info, error) {
 	gmeta, err := f(gadgetYamlFn)
-	if classicOrUnconstrained(model) && os.IsNotExist(err) {
+	if classicOrUndetermined(model) && os.IsNotExist(err) {
 		// gadget.yaml is optional for classic gadgets
 		return &Info{}, nil
 	}
@@ -383,16 +450,60 @@ func readInfo(f func(string) ([]byte, error), gadgetYamlFn string, model Model)
 
 // ReadInfo reads the gadget specific metadata from meta/gadget.yaml in the snap
 // root directory.
+// See ReadInfoAndValidate for a variant that does role-usage consistency
+// validation like Validate.
 func ReadInfo(gadgetSnapRootDir string, model Model) (*Info, error) {
 	gadgetYamlFn := filepath.Join(gadgetSnapRootDir, "meta", "gadget.yaml")
-	return readInfo(ioutil.ReadFile, gadgetYamlFn, model)
+	ginfo, err := readInfo(ioutil.ReadFile, gadgetYamlFn, model)
+	if err != nil {
+		return nil, err
+	}
+	return ginfo, nil
+}
+
+// ReadInfoAndValidate reads the gadget specific metadata from
+// meta/gadget.yaml in the snap root directory.
+// It also performs role-usage consistency validation as Validate does
+// using the given constraints. See ReadInfo for a variant that does not.
+// See also ValidateContent for further validating the content itself
+// instead of the metadata.
+func ReadInfoAndValidate(gadgetSnapRootDir string, model Model, validationConstraints *ValidationConstraints) (*Info, error) {
+	ginfo, err := ReadInfo(gadgetSnapRootDir, model)
+	if err != nil {
+		return nil, err
+	}
+	if err := Validate(ginfo, model, validationConstraints); err != nil {
+		return nil, err
+	}
+	return ginfo, err
 }
 
 // ReadInfoFromSnapFile reads the gadget specific metadata from
 // meta/gadget.yaml in the given snap container.
+// It also performs role-usage consistency validation as Validate does.
+// See ReadInfoFromSnapFileNoValidate for a variant that does not.
 func ReadInfoFromSnapFile(snapf snap.Container, model Model) (*Info, error) {
+	ginfo, err := ReadInfoFromSnapFileNoValidate(snapf, model)
+	if err != nil {
+		return nil, err
+	}
+	if err := Validate(ginfo, model, nil); err != nil {
+		return nil, err
+	}
+	return ginfo, nil
+}
+
+// ReadInfoFromSnapFileNoValidate reads the gadget specific metadata from
+// meta/gadget.yaml in the given snap container.
+// See ReadInfoFromSnapFile for a variant that does role-usage consistency
+// validation like Validate as well.
+func ReadInfoFromSnapFileNoValidate(snapf snap.Container, model Model) (*Info, error) {
 	gadgetYamlFn := "meta/gadget.yaml"
-	return readInfo(snapf.ReadFile, gadgetYamlFn, model)
+	ginfo, err := readInfo(snapf.ReadFile, gadgetYamlFn, model)
+	if err != nil {
+		return nil, err
+	}
+	return ginfo, nil
 }
 
 func fmtIndexAndName(idx int, name string) string {
@@ -402,14 +513,7 @@ func fmtIndexAndName(idx int, name string) string {
 	return fmt.Sprintf("#%v", idx)
 }
 
-type validationState struct {
-	SystemSeed *VolumeStructure
-	SystemData *VolumeStructure
-	SystemBoot *VolumeStructure
-	SystemSave *VolumeStructure
-}
-
-func validateVolume(name string, vol *Volume, model Model) error {
+func validateVolume(name string, vol *Volume, model Model, knownFsLabelsPerVolume map[string]map[string]bool) error {
 	if !validVolumeName.MatchString(name) {
 		return errors.New("invalid name")
 	}
@@ -424,19 +528,25 @@ func validateVolume(name string, vol *Volume, model Model) error {
 	// for validating structure overlap
 	structures := make([]LaidOutStructure, len(vol.Structure))
 
-	state := &validationState{}
-	previousEnd := quantity.Size(0)
+	if knownFsLabelsPerVolume != nil {
+		knownFsLabelsPerVolume[name] = knownFsLabels
+	}
+
+	previousEnd := quantity.Offset(0)
+	// TODO: should we also validate that if there is a system-recovery-select
+	// role there should also be at least 2 system-recovery-image roles and
+	// same for system-boot-select and at least 2 system-boot-image roles?
 	for idx, s := range vol.Structure {
 		if err := validateVolumeStructure(&s, vol); err != nil {
 			return fmt.Errorf("invalid structure %v: %v", fmtIndexAndName(idx, s.Name), err)
 		}
-		var start quantity.Size
+		var start quantity.Offset
 		if s.Offset != nil {
 			start = *s.Offset
 		} else {
 			start = previousEnd
 		}
-		end := start + s.Size
+		end := start + quantity.Offset(s.Size)
 		ps := LaidOutStructure{
 			VolumeStructure: &vol.Structure[idx],
 			StartOffset:     start,
@@ -457,135 +567,34 @@ func validateVolume(name string, vol *Volume, model Model) error {
 			knownFsLabels[s.Label] = true
 		}
 
-		switch s.Role {
-		case SystemSeed:
-			if state.SystemSeed != nil {
-				return fmt.Errorf("cannot have more than one partition with system-seed role")
-			}
-			state.SystemSeed = &vol.Structure[idx]
-		case SystemData:
-			if state.SystemData != nil {
-				return fmt.Errorf("cannot have more than one partition with system-data role")
-			}
-			state.SystemData = &vol.Structure[idx]
-		case SystemBoot:
-			if state.SystemBoot != nil {
-				return fmt.Errorf("cannot have more than one partition with system-boot role")
-			}
-			state.SystemBoot = &vol.Structure[idx]
-		case SystemSave:
-			if state.SystemSave != nil {
-				return fmt.Errorf("cannot have more than one partition with system-save role")
-			}
-			state.SystemSave = &vol.Structure[idx]
-		}
-
 		previousEnd = end
 	}
 
-	if err := ensureVolumeConsistency(state, model); err != nil {
-		return err
-	}
-
 	// sort by starting offset
 	sort.Sort(byStartOffset(structures))
 
 	return validateCrossVolumeStructure(structures, knownStructures)
 }
 
-func ensureVolumeConsistencyNoConstraints(state *validationState) error {
-	switch {
-	case state.SystemSeed == nil && state.SystemData == nil:
-		// happy so far
-	case state.SystemSeed != nil && state.SystemData == nil:
-		return fmt.Errorf("the system-seed role requires system-data to be defined")
-	case state.SystemSeed == nil && state.SystemData != nil:
-		if state.SystemData.Label != "" && state.SystemData.Label != implicitSystemDataLabel {
-			return fmt.Errorf("system-data structure must have an implicit label or %q, not %q", implicitSystemDataLabel, state.SystemData.Label)
-		}
-	case state.SystemSeed != nil && state.SystemData != nil:
-		if err := ensureSeedDataLabelsUnset(state); err != nil {
-			return err
-		}
+// isMBR returns whether the structure is the MBR and can be used before setImplicitForVolume
+func isMBR(vs *VolumeStructure) bool {
+	if vs.Role == schemaMBR {
+		return true
 	}
-	if state.SystemSave != nil {
-		if err := ensureSystemSaveConsistency(state); err != nil {
-			return err
-		}
+	if vs.Role == "" && vs.Type == schemaMBR {
+		return true
 	}
-	return nil
-}
-
-func ensureVolumeConsistencyWithConstraints(state *validationState, model Model) error {
-	switch {
-	case state.SystemSeed == nil && state.SystemData == nil:
-		if wantsSystemSeed(model) {
-			return fmt.Errorf("model requires system-seed partition, but no system-seed or system-data partition found")
-		}
-	case state.SystemSeed != nil && state.SystemData == nil:
-		return fmt.Errorf("the system-seed role requires system-data to be defined")
-	case state.SystemSeed == nil && state.SystemData != nil:
-		// error if we have the SystemSeed constraint but no actual system-seed structure
-		if wantsSystemSeed(model) {
-			return fmt.Errorf("model requires system-seed structure, but none was found")
-		}
-		// without SystemSeed, system-data label must be implicit or writable
-		if state.SystemData.Label != "" && state.SystemData.Label != implicitSystemDataLabel {
-			return fmt.Errorf("system-data structure must have an implicit label or %q, not %q",
-				implicitSystemDataLabel, state.SystemData.Label)
-		}
-	case state.SystemSeed != nil && state.SystemData != nil:
-		// error if we don't have the SystemSeed constraint but we have a system-seed structure
-		if !wantsSystemSeed(model) {
-			return fmt.Errorf("model does not support the system-seed role")
-		}
-		if err := ensureSeedDataLabelsUnset(state); err != nil {
-			return err
-		}
-	}
-	if state.SystemSave != nil {
-		if err := ensureSystemSaveConsistency(state); err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
-func ensureVolumeConsistency(state *validationState, model Model) error {
-	if model == nil {
-		return ensureVolumeConsistencyNoConstraints(state)
-	}
-	return ensureVolumeConsistencyWithConstraints(state, model)
-}
-
-func ensureSeedDataLabelsUnset(state *validationState) error {
-	if state.SystemData.Label != "" {
-		return fmt.Errorf("system-data structure must not have a label")
-	}
-	if state.SystemSeed.Label != "" {
-		return fmt.Errorf("system-seed structure must not have a label")
-	}
-	return nil
-}
-
-func ensureSystemSaveConsistency(state *validationState) error {
-	if state.SystemData == nil || state.SystemSeed == nil {
-		return fmt.Errorf("system-save requires system-seed and system-data structures")
-	}
-	if state.SystemSave.Label != "" {
-		return fmt.Errorf("system-save structure must not have a label")
-	}
-	return nil
+	return false
 }
 
 func validateCrossVolumeStructure(structures []LaidOutStructure, knownStructures map[string]*LaidOutStructure) error {
-	previousEnd := quantity.Size(0)
+	previousEnd := quantity.Offset(0)
 	// cross structure validation:
 	// - relative offsets that reference other structures by name
 	// - laid out structure overlap
 	// use structures laid out within the volume
 	for pidx, ps := range structures {
-		if ps.EffectiveRole() == schemaMBR {
+		if isMBR(ps.VolumeStructure) {
 			if ps.StartOffset != 0 {
 				return fmt.Errorf(`structure %v has "mbr" role and must start at offset 0`, ps)
 			}
@@ -603,7 +612,7 @@ func validateCrossVolumeStructure(structures []LaidOutStructure, knownStructures
 			previous := structures[pidx-1]
 			return fmt.Errorf("structure %v overlaps with the preceding structure %v", ps, previous)
 		}
-		previousEnd = ps.StartOffset + ps.Size
+		previousEnd = ps.StartOffset + quantity.Offset(ps.Size)
 
 		if ps.HasFilesystem() {
 			// content relative offset only possible if it's a bare structure
@@ -623,31 +632,6 @@ func validateCrossVolumeStructure(structures []LaidOutStructure, knownStructures
 	return nil
 }
 
-var (
-	reservedLabels = []string{
-		// 2020-12-02 disabled because of customer gadget hotfix
-		/*ubuntuBootLabel,*/
-		ubuntuSeedLabel,
-		ubuntuDataLabel,
-		ubuntuSaveLabel,
-	}
-)
-
-func validateReservedLabels(vs *VolumeStructure) error {
-	if vs.Role != "" {
-		// structure specifies a role, its labels will be checked later
-		return nil
-	}
-	if vs.Label == "" {
-		return nil
-	}
-	if strutil.ListContains(reservedLabels, vs.Label) {
-		// a structure without a role uses one of reserved labels
-		return fmt.Errorf("label %q is reserved", vs.Label)
-	}
-	return nil
-}
-
 func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 	if vs.Size == 0 {
 		return errors.New("missing size")
@@ -685,10 +669,6 @@ func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 		return err
 	}
 
-	if err := validateReservedLabels(vs); err != nil {
-		return err
-	}
-
 	// TODO: validate structure size against sector-size; ubuntu-image uses
 	// a tmp file to find out the default sector size of the device the tmp
 	// file is created on
@@ -790,7 +770,7 @@ func validateRole(vs *VolumeStructure, vol *Volume) error {
 		if vs.Filesystem != "" && vs.Filesystem != "none" {
 			return errors.New("mbr structures must not specify a file system")
 		}
-	case SystemBoot, bootImage, bootSelect, "":
+	case SystemBoot, bootImage, bootSelect, seedBootSelect, seedBootImage, "":
 		// noop
 	case legacyBootImage, legacyBootSelect:
 		// noop
@@ -803,7 +783,7 @@ func validateRole(vs *VolumeStructure, vol *Volume) error {
 }
 
 func validateBareContent(vc *VolumeContent) error {
-	if vc.Source != "" || vc.Target != "" {
+	if vc.UnresolvedSource != "" || vc.Target != "" {
 		return fmt.Errorf("cannot use non-image content for bare file system")
 	}
 	if vc.Image == "" {
@@ -816,8 +796,11 @@ func validateFilesystemContent(vc *VolumeContent) error {
 	if vc.Image != "" || vc.Offset != nil || vc.OffsetWrite != nil || vc.Size != 0 {
 		return fmt.Errorf("cannot use image content for non-bare file system")
 	}
-	if vc.Source == "" || vc.Target == "" {
-		return fmt.Errorf("missing source or target")
+	if vc.UnresolvedSource == "" {
+		return fmt.Errorf("missing source")
+	}
+	if vc.Target == "" {
+		return fmt.Errorf("missing target")
 	}
 	return nil
 }
@@ -853,7 +836,7 @@ type RelativeOffset struct {
 	// address write will be calculated.
 	RelativeTo string
 	// Offset is a 32-bit value
-	Offset quantity.Size
+	Offset quantity.Offset
 }
 
 func (r *RelativeOffset) String() string {
@@ -867,31 +850,31 @@ func (r *RelativeOffset) String() string {
 }
 
 // parseRelativeOffset parses a string describing an offset that can be
-// expressed relative to a named structure, with the format: [<name>+]<size>.
+// expressed relative to a named structure, with the format: [<name>+]<offset>.
 func parseRelativeOffset(grs string) (*RelativeOffset, error) {
 	toWhat := ""
-	sizeSpec := grs
+	offsSpec := grs
 	if idx := strings.IndexRune(grs, '+'); idx != -1 {
-		toWhat, sizeSpec = grs[:idx], grs[idx+1:]
+		toWhat, offsSpec = grs[:idx], grs[idx+1:]
 		if toWhat == "" {
 			return nil, errors.New("missing volume name")
 		}
 	}
-	if sizeSpec == "" {
+	if offsSpec == "" {
 		return nil, errors.New("missing offset")
 	}
 
-	size, err := quantity.ParseSize(sizeSpec)
+	offset, err := quantity.ParseOffset(offsSpec)
 	if err != nil {
-		return nil, fmt.Errorf("cannot parse offset %q: %v", sizeSpec, err)
+		return nil, fmt.Errorf("cannot parse offset %q: %v", offsSpec, err)
 	}
-	if size > 4*quantity.SizeGiB {
+	if offset > 4*1024*quantity.OffsetMiB {
 		return nil, fmt.Errorf("offset above 4G limit")
 	}
 
 	return &RelativeOffset{
 		RelativeTo: toWhat,
-		Offset:     size,
+		Offset:     offset,
 	}, nil
 }
 
@@ -926,14 +909,18 @@ func IsCompatible(current, new *Info) error {
 		return err
 	}
 
+	if currentVol.Schema == "" || newVol.Schema == "" {
+		return fmt.Errorf("internal error: unset volume schemas: old: %q new: %q", currentVol.Schema, newVol.Schema)
+	}
+
 	// layout both volumes partially, without going deep into the layout of
 	// structure content, we only want to make sure that structures are
 	// comapatible
-	pCurrent, err := LayoutVolumePartially(currentVol, defaultConstraints)
+	pCurrent, err := LayoutVolumePartially(currentVol, DefaultConstraints)
 	if err != nil {
 		return fmt.Errorf("cannot lay out the current volume: %v", err)
 	}
-	pNew, err := LayoutVolumePartially(newVol, defaultConstraints)
+	pNew, err := LayoutVolumePartially(newVol, DefaultConstraints)
 	if err != nil {
 		return fmt.Errorf("cannot lay out the new volume: %v", err)
 	}
@@ -943,32 +930,46 @@ func IsCompatible(current, new *Info) error {
 	return nil
 }
 
-// PositionedVolumeFromGadget takes a gadget rootdir and positions the
-// partitions as specified.
-func PositionedVolumeFromGadget(gadgetRoot string) (*LaidOutVolume, error) {
-	info, err := ReadInfo(gadgetRoot, nil)
+// LaidOutSystemVolumeFromGadget takes a gadget rootdir and lays out the
+// partitions as specified. It returns one specific volume, which is the volume
+// on which system-* roles/partitions exist, all other volumes are assumed to
+// already be flashed and managed separately at image build/flash time, while
+// the system-* roles can be manipulated on the returned volume during install
+// mode.
+func LaidOutSystemVolumeFromGadget(gadgetRoot, kernelRoot string, model Model) (*LaidOutVolume, error) {
+	// model should never be nil here
+	if model == nil {
+		return nil, fmt.Errorf("internal error: must have model to lay out system volumes from a gadget")
+	}
+	// rely on the basic validation from ReadInfo to ensure that the system-*
+	// roles are all on the same volume for example
+	info, err := ReadInfoAndValidate(gadgetRoot, model, nil)
 	if err != nil {
 		return nil, err
 	}
-	// Limit ourselves to just one volume for now.
-	if len(info.Volumes) != 1 {
-		return nil, fmt.Errorf("cannot position multiple volumes yet")
-	}
 
 	constraints := LayoutConstraints{
-		NonMBRStartOffset: 1 * quantity.SizeMiB,
-		SectorSize:        512,
+		NonMBRStartOffset: 1 * quantity.OffsetMiB,
 	}
 
+	// find the volume with the system-boot role on it, we already validated
+	// that the system-* roles are all on the same volume
 	for _, vol := range info.Volumes {
-		pvol, err := LayoutVolume(gadgetRoot, &vol, constraints)
-		if err != nil {
-			return nil, err
+		for _, structure := range vol.Structure {
+			// use the system-boot role to identify the system volume
+			if structure.Role == SystemBoot {
+				pvol, err := LayoutVolume(gadgetRoot, kernelRoot, vol, constraints)
+				if err != nil {
+					return nil, err
+				}
+
+				return pvol, nil
+			}
 		}
-		// we know  info.Volumes map has size 1 so we can return here
-		return pvol, nil
 	}
-	return nil, fmt.Errorf("internal error in PositionedVolumeFromGadget: this line cannot be reached")
+
+	// this should be impossible, the validation above should ensure this
+	return nil, fmt.Errorf("internal error: gadget passed validation but does not have system-* roles on any volume")
 }
 
 func flatten(path string, cfg interface{}, out map[string]interface{}) {
@@ -996,3 +997,106 @@ func SystemDefaults(gadgetDefaults map[string]map[string]interface{}) map[string
 	}
 	return nil
 }
+
+// See https://www.kernel.org/doc/html/latest/admin-guide/kernel-parameters.html
+var disallowedKernelArguments = []string{
+	"root", "nfsroot",
+	"init",
+}
+
+// isKernelArgumentAllowed checks whether the kernel command line argument is
+// allowed. Prohibits all arguments listed explicitly in
+// disallowedKernelArguments list and those prefixed with snapd, with exception
+// of snapd.debug. All other arguments are allowed.
+func isKernelArgumentAllowed(arg string) bool {
+	if strings.HasPrefix(arg, "snapd") && arg != "snapd.debug" {
+		return false
+	}
+	if strutil.ListContains(disallowedKernelArguments, arg) {
+		return false
+	}
+	return true
+}
+
+var ErrNoKernelCommandline = errors.New("no kernel command line in the gadget")
+
+// KernelCommandLineFromGadget returns the desired kernel command line provided by the
+// gadget. The full flag indicates whether the gadget provides a full command
+// line or just the extra parameters that will be appended to the static ones.
+// An ErrNoKernelCommandline is returned when thea gadget does not set any
+// kernel command line.
+func KernelCommandLineFromGadget(gadgetDirOrSnapPath string) (cmdline string, full bool, err error) {
+	sf, err := snapfile.Open(gadgetDirOrSnapPath)
+	if err != nil {
+		return "", false, fmt.Errorf("cannot open gadget snap: %v", err)
+	}
+	contentExtra, err := sf.ReadFile("cmdline.extra")
+	if err != nil && !os.IsNotExist(err) {
+		return "", false, err
+	}
+	contentFull, err := sf.ReadFile("cmdline.full")
+	if err != nil && !os.IsNotExist(err) {
+		return "", false, err
+	}
+	content := contentExtra
+	whichFile := "cmdline.extra"
+	switch {
+	case contentExtra != nil && contentFull != nil:
+		return "", false, fmt.Errorf("cannot support both extra and full kernel command lines")
+	case contentExtra == nil && contentFull == nil:
+		return "", false, ErrNoKernelCommandline
+	case contentFull != nil:
+		content = contentFull
+		whichFile = "cmdline.full"
+		full = true
+	}
+	parsed, err := parseCommandLineFromGadget(content, whichFile)
+	if err != nil {
+		return "", full, fmt.Errorf("invalid kernel command line in %v: %v", whichFile, err)
+	}
+	return parsed, full, nil
+}
+
+// parseCommandLineFromGadget parses the command line file and returns a
+// reassembled kernel command line as a single string. The file can be multi
+// line, where only lines stating with # are treated as comments, eg.
+//
+// foo
+// # this is a comment
+//
+// According to https://elixir.bootlin.com/linux/latest/source/Documentation/admin-guide/kernel-parameters.txt
+// the # character can appear as part of a valid kernel command line argument,
+// specifically in the following argument:
+//   memmap=nn[KMG]#ss[KMG]
+//   memmap=100M@2G,100M#3G,1G!1024G
+// Thus a lone # or a token starting with # are treated as errors.
+func parseCommandLineFromGadget(content []byte, whichFile string) (string, error) {
+	s := bufio.NewScanner(bytes.NewBuffer(content))
+	filtered := &bytes.Buffer{}
+	for s.Scan() {
+		line := s.Text()
+		if len(line) > 0 && line[0] == '#' {
+			// comment
+			continue
+		}
+		filtered.WriteRune(' ')
+		filtered.WriteString(line)
+	}
+	if err := s.Err(); err != nil {
+		return "", err
+	}
+	kargs, err := osutil.KernelCommandLineSplit(filtered.String())
+	if err != nil {
+		return "", err
+	}
+	for _, argValue := range kargs {
+		if strings.HasPrefix(argValue, "#") {
+			return "", fmt.Errorf("unexpected or invalid use of # in argument %q", argValue)
+		}
+		split := strings.SplitN(argValue, "=", 2)
+		if !isKernelArgumentAllowed(split[0]) {
+			return "", fmt.Errorf("disallowed kernel argument %q", argValue)
+		}
+	}
+	return strings.Join(kargs, " "), nil
+}
