@@ -2,7 +2,7 @@
 // +build !nosecboot
 
 /*
- * Copyright (C) 2019 Canonical Ltd
+ * Copyright (C) 2019-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -24,7 +24,6 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
-	"testing"
 
 	. "gopkg.in/check.v1"
 
@@ -35,8 +34,6 @@ import (
 	"github.com/snapcore/snapd/testutil"
 )
 
-func TestInstall(t *testing.T) { TestingT(t) }
-
 type installSuite struct {
 	testutil.BaseTest
 
@@ -58,7 +55,7 @@ func (s *installSuite) SetUpTest(c *C) {
 }
 
 func (s *installSuite) TestInstallRunError(c *C) {
-	sys, err := install.Run("", "", install.Options{}, nil)
+	sys, err := install.Run(nil, "", "", "", install.Options{}, nil)
 	c.Assert(err, ErrorMatches, "cannot use empty gadget root directory")
 	c.Check(sys, IsNil)
 }
@@ -104,7 +101,7 @@ var mockDeviceLayout = gadget.OnDiskVolume{
 					Name: "BIOS Boot",
 					Size: 1 * quantity.SizeMiB,
 				},
-				StartOffset: 1 * quantity.SizeMiB,
+				StartOffset: 1 * quantity.OffsetMiB,
 			},
 			Node: "/dev/node2",
 		},
@@ -118,12 +115,26 @@ var mockDeviceLayout = gadget.OnDiskVolume{
 
 func (s *installSuite) TestLayoutCompatibility(c *C) {
 	// same contents (the locally created structure should be ignored)
-	gadgetLayout := layoutFromYaml(c, mockGadgetYaml)
+	gadgetLayout := layoutFromYaml(c, mockGadgetYaml, nil)
 	err := install.EnsureLayoutCompatibility(gadgetLayout, &mockDeviceLayout)
 	c.Assert(err, IsNil)
 
+	// layout still compatible with a larger disk sector size
+	mockDeviceLayout.SectorSize = 4096
+	err = install.EnsureLayoutCompatibility(gadgetLayout, &mockDeviceLayout)
+	c.Assert(err, IsNil)
+
+	// layout not compatible with a sector size that's not a factor of the
+	// structure sizes
+	mockDeviceLayout.SectorSize = 513
+	err = install.EnsureLayoutCompatibility(gadgetLayout, &mockDeviceLayout)
+	c.Assert(err, ErrorMatches, `gadget volume structure #1 \(\"BIOS Boot\"\) size is not a multiple of disk sector size 513`)
+
+	// rest for the rest of the test
+	mockDeviceLayout.SectorSize = 512
+
 	// missing structure (that's ok)
-	gadgetLayoutWithExtras := layoutFromYaml(c, mockGadgetYaml+mockExtraStructure)
+	gadgetLayoutWithExtras := layoutFromYaml(c, mockGadgetYaml+mockExtraStructure, nil)
 	err = install.EnsureLayoutCompatibility(gadgetLayoutWithExtras, &mockDeviceLayout)
 	c.Assert(err, IsNil)
 
@@ -136,7 +147,7 @@ func (s *installSuite) TestLayoutCompatibility(c *C) {
 					Size:  10 * quantity.SizeMiB,
 					Label: "extra",
 				},
-				StartOffset: 2 * quantity.SizeMiB,
+				StartOffset: 2 * quantity.OffsetMiB,
 			},
 			Node: "/dev/node3",
 		},
@@ -152,6 +163,7 @@ func (s *installSuite) TestLayoutCompatibility(c *C) {
 	c.Check(gadgetLayoutWithExtras.Size > smallDeviceLayout.Size, Equals, true)
 	err = install.EnsureLayoutCompatibility(gadgetLayoutWithExtras, &smallDeviceLayout)
 	c.Assert(err, ErrorMatches, `device /dev/node \(100 MiB\) is too small to fit the requested layout \(1\.17 GiB\)`)
+
 }
 
 func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
@@ -191,7 +203,7 @@ func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
 						Name: "different BIOS Boot",
 						Size: 1 * quantity.SizeMiB,
 					},
-					StartOffset: 1 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
 				},
 				Node: "/dev/node2",
 			},
@@ -202,11 +214,11 @@ func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
 		Size:       2 * quantity.SizeGiB,
 		SectorSize: 512,
 	}
-	gadgetLayout := layoutFromYaml(c, mockMBRGadgetYaml)
+	gadgetLayout := layoutFromYaml(c, mockMBRGadgetYaml, nil)
 	err := install.EnsureLayoutCompatibility(gadgetLayout, &mockMBRDeviceLayout)
 	c.Assert(err, IsNil)
 	// structure is missing from disk
-	gadgetLayoutWithExtras := layoutFromYaml(c, mockMBRGadgetYaml+mockExtraStructure)
+	gadgetLayoutWithExtras := layoutFromYaml(c, mockMBRGadgetYaml+mockExtraStructure, nil)
 	err = install.EnsureLayoutCompatibility(gadgetLayoutWithExtras, &mockMBRDeviceLayout)
 	c.Assert(err, IsNil)
 	// add it now
@@ -222,13 +234,26 @@ func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
 					Filesystem: "ext4",
 					Type:       "83",
 				},
-				StartOffset: 2 * quantity.SizeMiB,
+				StartOffset: 2 * quantity.OffsetMiB,
 			},
 			Node: "/dev/node3",
 		},
 	)
 	err = install.EnsureLayoutCompatibility(gadgetLayoutWithExtras, &deviceLayoutWithExtras)
 	c.Assert(err, IsNil)
+
+	// test with a larger sector size that is still an even multiple of the
+	// structure sizes in the gadget
+	mockMBRDeviceLayout.SectorSize = 4096
+	err = install.EnsureLayoutCompatibility(gadgetLayout, &mockMBRDeviceLayout)
+	c.Assert(err, IsNil)
+
+	// but with a sector size that is not an even multiple of the structure size
+	// then we have an error
+	mockMBRDeviceLayout.SectorSize = 513
+	err = install.EnsureLayoutCompatibility(gadgetLayout, &mockMBRDeviceLayout)
+	c.Assert(err, ErrorMatches, `gadget volume structure #1 \(\"BIOS Boot\"\) size is not a multiple of disk sector size 513`)
+
 	// add another structure that's not part of the gadget
 	deviceLayoutWithExtras.Structure = append(deviceLayoutWithExtras.Structure,
 		gadget.OnDiskStructure{
@@ -238,7 +263,7 @@ func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
 					Name: "Extra extra partition",
 					Size: 1 * quantity.SizeMiB,
 				},
-				StartOffset: 1202 * quantity.SizeMiB,
+				StartOffset: 1202 * quantity.OffsetMiB,
 			},
 			Node: "/dev/node4",
 		},
@@ -248,7 +273,7 @@ func (s *installSuite) TestMBRLayoutCompatibility(c *C) {
 }
 
 func (s *installSuite) TestLayoutCompatibilityWithCreatedPartitions(c *C) {
-	gadgetLayoutWithExtras := layoutFromYaml(c, mockGadgetYaml+mockExtraStructure)
+	gadgetLayoutWithExtras := layoutFromYaml(c, mockGadgetYaml+mockExtraStructure, nil)
 	deviceLayout := mockDeviceLayout
 
 	// device matches gadget except for the filesystem type
@@ -261,7 +286,7 @@ func (s *installSuite) TestLayoutCompatibilityWithCreatedPartitions(c *C) {
 					Label:      "writable",
 					Filesystem: "something_else",
 				},
-				StartOffset: 2 * quantity.SizeMiB,
+				StartOffset: 2 * quantity.OffsetMiB,
 			},
 			Node: "/dev/node3",
 		},
@@ -308,7 +333,7 @@ func (s *installSuite) TestLayoutCompatibilityWithCreatedPartitions(c *C) {
 }
 
 func (s *installSuite) TestSchemaCompatibility(c *C) {
-	gadgetLayout := layoutFromYaml(c, mockGadgetYaml)
+	gadgetLayout := layoutFromYaml(c, mockGadgetYaml, nil)
 	deviceLayout := mockDeviceLayout
 
 	error_msg := "disk partitioning.* doesn't match gadget.*"
@@ -346,7 +371,7 @@ func (s *installSuite) TestSchemaCompatibility(c *C) {
 }
 
 func (s *installSuite) TestIDCompatibility(c *C) {
-	gadgetLayout := layoutFromYaml(c, mockGadgetYaml)
+	gadgetLayout := layoutFromYaml(c, mockGadgetYaml, nil)
 	deviceLayout := mockDeviceLayout
 
 	error_msg := "disk ID.* doesn't match gadget volume ID.*"
@@ -374,13 +399,13 @@ func (s *installSuite) TestIDCompatibility(c *C) {
 	c.Logf("-----")
 }
 
-func layoutFromYaml(c *C, gadgetYaml string) *gadget.LaidOutVolume {
+func layoutFromYaml(c *C, gadgetYaml string, model gadget.Model) *gadget.LaidOutVolume {
 	gadgetRoot := filepath.Join(c.MkDir(), "gadget")
 	err := os.MkdirAll(filepath.Join(gadgetRoot, "meta"), 0755)
 	c.Assert(err, IsNil)
 	err = ioutil.WriteFile(filepath.Join(gadgetRoot, "meta", "gadget.yaml"), []byte(gadgetYaml), 0644)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, gadgetRoot, "", model)
 	c.Assert(err, IsNil)
 	return pv
 }
@@ -403,6 +428,11 @@ const mockUC20GadgetYaml = `volumes:
         # UEFI will boot the ESP partition by default first
         type: EF,C12A7328-F81F-11D2-BA4B-00A0C93EC93B
         size: 1200M
+      - name: ubuntu-boot
+        role: system-boot
+        filesystem: ext4
+        type: 83,0FC63DAF-8483-4772-8E79-3D69D8477DE4
+        size: 1200M
       - name: ubuntu-data
         role: system-data
         filesystem: ext4
@@ -429,7 +459,7 @@ func (s *installSuite) setupMockSysfs(c *C) {
 
 func (s *installSuite) TestDeviceFromRoleHappy(c *C) {
 	s.setupMockSysfs(c)
-	lv := layoutFromYaml(c, mockUC20GadgetYaml)
+	lv := layoutFromYaml(c, mockUC20GadgetYaml, uc20Mod)
 
 	device, err := install.DeviceFromRole(lv, gadget.SystemSeed)
 	c.Assert(err, IsNil)
@@ -438,7 +468,7 @@ func (s *installSuite) TestDeviceFromRoleHappy(c *C) {
 
 func (s *installSuite) TestDeviceFromRoleErrorNoMatchingSysfs(c *C) {
 	// note no sysfs mocking
-	lv := layoutFromYaml(c, mockUC20GadgetYaml)
+	lv := layoutFromYaml(c, mockUC20GadgetYaml, uc20Mod)
 
 	_, err := install.DeviceFromRole(lv, gadget.SystemSeed)
 	c.Assert(err, ErrorMatches, `cannot find device for role "system-seed": device not found`)
@@ -446,7 +476,7 @@ func (s *installSuite) TestDeviceFromRoleErrorNoMatchingSysfs(c *C) {
 
 func (s *installSuite) TestDeviceFromRoleErrorNoRole(c *C) {
 	s.setupMockSysfs(c)
-	lv := layoutFromYaml(c, mockGadgetYaml)
+	lv := layoutFromYaml(c, mockGadgetYaml, nil)
 
 	_, err := install.DeviceFromRole(lv, gadget.SystemSeed)
 	c.Assert(err, ErrorMatches, "cannot find role system-seed in gadget")
