@@ -24,16 +24,20 @@ import (
 	"fmt"
 	"io/ioutil"
 	"path/filepath"
+	"testing"
 	"time"
 
 	. "gopkg.in/check.v1"
 
+	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/gadget/install"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/testutil"
 )
 
+func TestInstall(t *testing.T) { TestingT(t) }
+
 type partitionTestSuite struct {
 	testutil.BaseTest
 
@@ -57,6 +61,10 @@ func (s *partitionTestSuite) SetUpTest(c *C) {
 	s.AddCleanup(cmdSfdisk.Restore)
 	cmdLsblk := testutil.MockCommand(c, "lsblk", `echo "lsblk was not mocked"; exit 1`)
 	s.AddCleanup(cmdLsblk.Restore)
+
+	// we test different sector sizes elsewhere, here we always use it to get the sector size
+	cmdBlockdev := testutil.MockCommand(c, "blockdev", blockdevSectorSize512Script)
+	s.AddCleanup(cmdBlockdev.Restore)
 }
 
 const (
@@ -159,6 +167,15 @@ exit 0`)
 	return b.String()
 }
 
+const blockdevSectorSize512Script = `
+if [ "$1" == "--getss" ]; then
+	echo 512
+	exit 0
+fi
+echo "unexpected cmdline opts $*"
+exit 1
+`
+
 var mockOnDiskStructureWritable = gadget.OnDiskStructure{
 	Node: "/dev/node3",
 	LaidOutStructure: gadget.LaidOutStructure{
@@ -177,6 +194,99 @@ var mockOnDiskStructureWritable = gadget.OnDiskStructure{
 	Size: 2*quantity.SizeGiB + 845*quantity.SizeMiB + 1031680,
 }
 
+var mockOnDiskStructureSave = gadget.OnDiskStructure{
+	Node: "/dev/node3",
+	LaidOutStructure: gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Name:       "Save",
+			Size:       128 * quantity.SizeMiB,
+			Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
+			Role:       "system-save",
+			Label:      "ubuntu-save",
+			Filesystem: "ext4",
+		},
+		StartOffset: 1260388352,
+		Index:       3,
+	},
+	Size: 128 * quantity.SizeMiB,
+}
+
+var mockOnDiskStructureWritableAfterSave = gadget.OnDiskStructure{
+	Node: "/dev/node4",
+	LaidOutStructure: gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Name:       "Writable",
+			Size:       1200 * quantity.SizeMiB,
+			Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
+			Role:       "system-data",
+			Label:      "ubuntu-data",
+			Filesystem: "ext4",
+		},
+		StartOffset: 1394606080,
+		Index:       4,
+	},
+	// expanded to fill the disk
+	Size: 2*quantity.SizeGiB + 717*quantity.SizeMiB + 1031680,
+}
+
+// mustLayOutVolumeFromGadget takes a gadget rootdir and lays out the
+// partitions as specified. This function does not handle multiple volumes and
+// is meant for test helpers only. For runtime users, with multiple volumes
+// handled by choosing the ubuntu-* role volume, see LaidOutSystemVolumeFromGadget
+func mustLayOutVolumeFromGadget(c *C, gadgetRoot, kernelRoot string, model gadget.Model) (*gadget.LaidOutVolume, error) {
+	info, err := gadget.ReadInfo(gadgetRoot, model)
+	c.Assert(err, IsNil)
+
+	c.Assert(info.Volumes, HasLen, 1, Commentf("only single volumes supported in test helper"))
+
+	constraints := gadget.LayoutConstraints{
+		NonMBRStartOffset: 1 * quantity.OffsetMiB,
+	}
+
+	for _, vol := range info.Volumes {
+		pvol, err := gadget.LayoutVolume(gadgetRoot, kernelRoot, vol, constraints)
+		c.Assert(err, IsNil)
+		// we know  info.Volumes map has size 1 so we can return here
+		return pvol, nil
+	}
+	// this is impossible to reach, we already asserted that info.Volumes has a
+	// length of 1
+	panic("impossible test error")
+}
+
+type uc20Model struct{}
+
+func (c uc20Model) Classic() bool             { return false }
+func (c uc20Model) Grade() asserts.ModelGrade { return asserts.ModelSigned }
+
+var uc20Mod = uc20Model{}
+
+func (s *partitionTestSuite) TestBuildPartitionList(c *C) {
+	cmdSfdisk := testutil.MockCommand(c, "sfdisk", makeSfdiskScript(scriptPartitionsBiosSeed))
+	defer cmdSfdisk.Restore()
+
+	cmdLsblk := testutil.MockCommand(c, "lsblk", makeLsblkScript(scriptPartitionsBiosSeed))
+	defer cmdLsblk.Restore()
+
+	err := makeMockGadget(s.gadgetRoot, gptGadgetContentWithSave)
+	c.Assert(err, IsNil)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
+	c.Assert(err, IsNil)
+
+	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
+	c.Assert(err, IsNil)
+
+	// the expected expanded writable partition size is:
+	// start offset = (2M + 1200M), expanded size in sectors = (8388575*512 - start offset)/512
+	sfdiskInput, create := install.BuildPartitionList(dl, pv)
+	c.Assert(sfdiskInput.String(), Equals,
+		`/dev/node3 : start=     2461696, size=      262144, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Save"
+/dev/node4 : start=     2723840, size=     5664735, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Writable"
+`)
+	c.Check(create, NotNil)
+	c.Assert(create, DeepEquals, []gadget.OnDiskStructure{mockOnDiskStructureSave, mockOnDiskStructureWritableAfterSave})
+}
+
 func (s *partitionTestSuite) TestCreatePartitions(c *C) {
 	cmdSfdisk := testutil.MockCommand(c, "sfdisk", makeSfdiskScript(scriptPartitionsBiosSeed))
 	defer cmdSfdisk.Restore()
@@ -195,7 +305,7 @@ func (s *partitionTestSuite) TestCreatePartitions(c *C) {
 
 	err := makeMockGadget(s.gadgetRoot, gadgetContent)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
@@ -227,7 +337,7 @@ func (s *partitionTestSuite) TestRemovePartitionsTrivial(c *C) {
 
 	err := makeMockGadget(s.gadgetRoot, gadgetContent)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
@@ -291,7 +401,7 @@ echo '{
 
 	err = makeMockGadget(s.gadgetRoot, gadgetContent)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	err = install.RemoveCreatedPartitions(pv, dl)
@@ -316,7 +426,7 @@ func (s *partitionTestSuite) TestRemovePartitionsError(c *C) {
 
 	err = makeMockGadget(s.gadgetRoot, gadgetContent)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	err = install.RemoveCreatedPartitions(pv, dl)
@@ -478,7 +588,7 @@ echo '{
 
 	err := makeMockGadget(s.gadgetRoot, gptGadgetContentWithSave)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	dl, err := gadget.OnDiskVolumeFromDevice("node")
@@ -591,7 +701,17 @@ echo '{
 }'
 `)
 	defer cmdSfdisk.Restore()
-	cmdBlockdev := testutil.MockCommand(c, "blockdev", `echo '1234567'`)
+	cmdBlockdev := testutil.MockCommand(c, "blockdev", `
+if [ "$1" == "--getss" ]; then
+	echo 512
+	exit 0
+elif [ "$1" == "--getsz" ]; then
+	echo 1234567
+	exit 0
+fi
+echo "unexpected cmdline opts $*"
+exit 1
+`)
 	defer cmdBlockdev.Restore()
 
 	dl, err := gadget.OnDiskVolumeFromDevice("node")
@@ -599,9 +719,16 @@ echo '{
 
 	err = makeMockGadget(s.gadgetRoot, mbrGadgetContentWithSave)
 	c.Assert(err, IsNil)
-	pv, err := gadget.PositionedVolumeFromGadget(s.gadgetRoot)
+	pv, err := mustLayOutVolumeFromGadget(c, s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
 	list := install.CreatedDuringInstall(pv, dl)
 	c.Assert(list, DeepEquals, []string{"/dev/node2", "/dev/node3", "/dev/node4"})
 }
+
+func (s *partitionTestSuite) TestCreationSupported(c *C) {
+	winBasic := "EBD0A0A2-B9E5-4433-87C0-68B6B72699C7"
+
+	c.Check(install.CreationSupported(winBasic), Equals, true)
+	c.Check(install.CreationSupported("invalid-partion-uuid"), Equals, false)
+}
