@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2019 Canonical Ltd
+ * Copyright (C) 2019-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -22,8 +22,10 @@ package gadget
 import (
 	"errors"
 	"fmt"
+	"strings"
 
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/kernel"
 	"github.com/snapcore/snapd/logger"
 )
 
@@ -33,9 +35,8 @@ var (
 
 var (
 	// default positioning constraints that match ubuntu-image
-	defaultConstraints = LayoutConstraints{
-		NonMBRStartOffset: 1 * quantity.SizeMiB,
-		SectorSize:        512,
+	DefaultConstraints = LayoutConstraints{
+		NonMBRStartOffset: 1 * quantity.OffsetMiB,
 	}
 )
 
@@ -43,13 +44,27 @@ var (
 type GadgetData struct {
 	// Info is the gadget metadata
 	Info *Info
+	// XXX: should be GadgetRootDir
 	// RootDir is the root directory of gadget snap data
 	RootDir string
+
+	// KernelRootDir is the root directory of kernel snap data
+	KernelRootDir string
 }
 
-// UpdatePolicyFunc is a callback that evaluates the provided pair of structures
-// and returns true when the pair should be part of an update.
-type UpdatePolicyFunc func(from, to *LaidOutStructure) bool
+// UpdatePolicyFunc is a callback that evaluates the provided pair of
+// (potentially not yet resolved) structures and returns true when the
+// pair should be part of an update. It may also return a filter
+// function for the resolved content when not all of the content
+// should be applied as part of the update (e.g. when updating assets
+// from the kernel snap).
+type UpdatePolicyFunc func(from, to *LaidOutStructure) (bool, ResolvedContentFilterFunc)
+
+// ResolvedContentFilterFunc is a callback that evaluates the given
+// ResolvedContent and returns true if it should be applied as part of
+// an update. This is relevant for e.g. asset updates that come from
+// the kernel snap.
+type ResolvedContentFilterFunc func(*ResolvedContent) bool
 
 // ContentChange carries paths to files containing the content data being
 // modified by the operation.
@@ -121,6 +136,28 @@ type ContentUpdateObserver interface {
 // Data that would be modified during the update is first backed up inside the
 // rollback directory. Should the apply step fail, the modified data is
 // recovered.
+//
+//
+// The rules for gadget/kernel updates with "$kernel:refs":
+//
+// 1. When installing a kernel with assets that have "update: true"
+//    there *must* be a matching entry in gadget.yaml. If not we risk
+//    bricking the system because the kernel tells us that it *needs*
+//    this file to boot but without gadget.yaml we would not put it
+//    anywhere.
+// 2. When installing a gadget with "$kernel:ref" content it is okay
+//    if this content cannot get resolved as long as there is no
+//    "edition" jump. This means adding new "$kernel:ref" without
+//    "edition" updates is always possible.
+//
+// To add a new "$kernel:ref" to gadget/kernel:
+// a. Update gadget and gadget.yaml and add "$kernel:ref" but do not
+//    update edition (if edition update is needed, use epoch)
+// b. Update kernel and kernel.yaml with new assets.
+// c. snapd will refresh gadget (see rule 2) but refuse to take the
+//    new kernel (rule 1)
+// d. After step (c) is completed the kernel refresh will now also
+//    work (no more violation of rule 1)
 func Update(old, new GadgetData, rollbackDirPath string, updatePolicy UpdatePolicyFunc, observer ContentUpdateObserver) error {
 	// TODO: support multi-volume gadgets. But for now we simply
 	//       do not do any gadget updates on those. We cannot error
@@ -136,15 +173,21 @@ func Update(old, new GadgetData, rollbackDirPath string, updatePolicy UpdatePoli
 		return err
 	}
 
+	if oldVol.Schema == "" || newVol.Schema == "" {
+		return fmt.Errorf("internal error: unset volume schemas: old: %q new: %q", oldVol.Schema, newVol.Schema)
+	}
+
 	// layout old partially, without going deep into the layout of structure
 	// content
-	pOld, err := LayoutVolumePartially(oldVol, defaultConstraints)
+	pOld, err := LayoutVolumePartially(oldVol, DefaultConstraints)
 	if err != nil {
 		return fmt.Errorf("cannot lay out the old volume: %v", err)
 	}
 
-	// layout new
-	pNew, err := LayoutVolume(new.RootDir, newVol, defaultConstraints)
+	// Layout new volume, delay resolving of filesystem content
+	constraints := DefaultConstraints
+	constraints.SkipResolveContent = true
+	pNew, err := LayoutVolume(new.RootDir, new.KernelRootDir, newVol, constraints)
 	if err != nil {
 		return fmt.Errorf("cannot lay out the new volume: %v", err)
 	}
@@ -156,8 +199,18 @@ func Update(old, new GadgetData, rollbackDirPath string, updatePolicy UpdatePoli
 	if updatePolicy == nil {
 		updatePolicy = defaultPolicy
 	}
+
+	// ensure all required kernel assets are found in the gadget
+	kernelInfo, err := kernel.ReadInfo(new.KernelRootDir)
+	if err != nil {
+		return err
+	}
+	if err := gadgetVolumeConsumesOneKernelUpdateAsset(pNew.Volume, kernelInfo); err != nil {
+		return err
+	}
+
 	// now we know which structure is which, find which ones need an update
-	updates, err := resolveUpdate(pOld, pNew, updatePolicy)
+	updates, err := resolveUpdate(pOld, pNew, updatePolicy, new.RootDir, new.KernelRootDir, kernelInfo)
 	if err != nil {
 		return err
 	}
@@ -168,7 +221,7 @@ func Update(old, new GadgetData, rollbackDirPath string, updatePolicy UpdatePoli
 
 	// can update old layout to new layout
 	for _, update := range updates {
-		if err := canUpdateStructure(update.from, update.to, pNew.EffectiveSchema()); err != nil {
+		if err := canUpdateStructure(update.from, update.to, pNew.Schema); err != nil {
 			return fmt.Errorf("cannot update volume structure %v: %v", update.to, err)
 		}
 	}
@@ -194,10 +247,10 @@ func resolveVolume(old *Info, new *Info) (oldVol, newVol *Volume, err error) {
 		return nil, nil, fmt.Errorf("cannot find entry for volume %q in updated gadget info", name)
 	}
 
-	return &oldV, &newV, nil
+	return oldV, newV, nil
 }
 
-func isSameOffset(one *quantity.Size, two *quantity.Size) bool {
+func isSameOffset(one *quantity.Offset, two *quantity.Offset) bool {
 	if one == nil && two == nil {
 		return true
 	}
@@ -220,7 +273,7 @@ func isSameRelativeOffset(one *RelativeOffset, two *RelativeOffset) bool {
 func isLegacyMBRTransition(from *LaidOutStructure, to *LaidOutStructure) bool {
 	// legacy MBR could have been specified by setting type: mbr, with no
 	// role
-	return from.Type == schemaMBR && to.EffectiveRole() == schemaMBR
+	return from.Type == schemaMBR && to.Role == schemaMBR
 }
 
 func canUpdateStructure(from *LaidOutStructure, to *LaidOutStructure, schema string) error {
@@ -241,8 +294,8 @@ func canUpdateStructure(from *LaidOutStructure, to *LaidOutStructure, schema str
 	if !isSameRelativeOffset(from.OffsetWrite, to.OffsetWrite) {
 		return fmt.Errorf("cannot change structure offset-write from %v to %v", from.OffsetWrite, to.OffsetWrite)
 	}
-	if from.EffectiveRole() != to.EffectiveRole() {
-		return fmt.Errorf("cannot change structure role from %q to %q", from.EffectiveRole(), to.EffectiveRole())
+	if from.Role != to.Role {
+		return fmt.Errorf("cannot change structure role from %q to %q", from.Role, to.Role)
 	}
 	if from.Type != to.Type {
 		if !isLegacyMBRTransition(from, to) {
@@ -260,7 +313,7 @@ func canUpdateStructure(from *LaidOutStructure, to *LaidOutStructure, schema str
 			return fmt.Errorf("cannot change filesystem from %q to %q",
 				from.Filesystem, to.Filesystem)
 		}
-		if from.EffectiveFilesystemLabel() != to.EffectiveFilesystemLabel() {
+		if from.Label != to.Label {
 			return fmt.Errorf("cannot change filesystem label from %q to %q",
 				from.Label, to.Label)
 		}
@@ -277,8 +330,8 @@ func canUpdateVolume(from *PartiallyLaidOutVolume, to *LaidOutVolume) error {
 	if from.ID != to.ID {
 		return fmt.Errorf("cannot change volume ID from %q to %q", from.ID, to.ID)
 	}
-	if from.EffectiveSchema() != to.EffectiveSchema() {
-		return fmt.Errorf("cannot change volume schema from %q to %q", from.EffectiveSchema(), to.EffectiveSchema())
+	if from.Schema != to.Schema {
+		return fmt.Errorf("cannot change volume schema from %q to %q", from.Schema, to.Schema)
 	}
 	if len(from.LaidOutStructure) != len(to.LaidOutStructure) {
 		return fmt.Errorf("cannot change the number of structures within volume from %v to %v", len(from.LaidOutStructure), len(to.LaidOutStructure))
@@ -291,30 +344,66 @@ type updatePair struct {
 	to   *LaidOutStructure
 }
 
-func defaultPolicy(from, to *LaidOutStructure) bool {
-	return to.Update.Edition > from.Update.Edition
+func defaultPolicy(from, to *LaidOutStructure) (bool, ResolvedContentFilterFunc) {
+	return to.Update.Edition > from.Update.Edition, nil
 }
 
 // RemodelUpdatePolicy implements the update policy of a remodel scenario. The
 // policy selects all non-MBR structures for the update.
-func RemodelUpdatePolicy(from, _ *LaidOutStructure) bool {
-	if from.EffectiveRole() == schemaMBR {
-		return false
+func RemodelUpdatePolicy(from, to *LaidOutStructure) (bool, ResolvedContentFilterFunc) {
+	if from.Role == schemaMBR {
+		return false, nil
 	}
-	return true
+	return true, nil
 }
 
-func resolveUpdate(oldVol *PartiallyLaidOutVolume, newVol *LaidOutVolume, policy UpdatePolicyFunc) (updates []updatePair, err error) {
+// KernelUpdatePolicy implements the update policy for kernel asset updates.
+//
+// This is called when there is a kernel->kernel refresh for kernels that
+// contain bootloader assets. In this case all bootloader assets that are
+// marked as "update: true" in the kernel.yaml need updating.
+//
+// But any non-kernel assets need to be ignored, they will be handled by
+// the regular gadget->gadget update mechanism and policy.
+func KernelUpdatePolicy(from, to *LaidOutStructure) (bool, ResolvedContentFilterFunc) {
+	// The policy function has to work on unresolved content, the
+	// returned filter will make sure that after resolving only the
+	// relevant $kernel:refs are updated.
+	for _, ct := range to.Content {
+		if strings.HasPrefix(ct.UnresolvedSource, "$kernel:") {
+			return true, func(rn *ResolvedContent) bool {
+				return rn.KernelUpdate
+			}
+		}
+	}
+
+	return false, nil
+}
+
+func resolveUpdate(oldVol *PartiallyLaidOutVolume, newVol *LaidOutVolume, policy UpdatePolicyFunc, newGadgetRootDir, newKernelRootDir string, kernelInfo *kernel.Info) (updates []updatePair, err error) {
 	if len(oldVol.LaidOutStructure) != len(newVol.LaidOutStructure) {
 		return nil, errors.New("internal error: the number of structures in new and old volume definitions is different")
 	}
 	for j, oldStruct := range oldVol.LaidOutStructure {
 		newStruct := newVol.LaidOutStructure[j]
-		// update only when new edition is higher than the old one; boot
-		// assets are assumed to be backwards compatible, once deployed
-		// are not rolled back or replaced unless a higher edition is
-		// available
-		if policy(&oldStruct, &newStruct) {
+		// update only when the policy says so; boot assets
+		// are assumed to be backwards compatible, once
+		// deployed they are not rolled back or replaced unless
+		// told by the new policy
+		if update, filter := policy(&oldStruct, &newStruct); update {
+			// Ensure content is resolved and filtered. Filtering
+			// is required for e.g. KernelUpdatePolicy, see above.
+			resolvedContent, err := resolveVolumeContent(newGadgetRootDir, newKernelRootDir, kernelInfo, &newStruct, filter)
+			if err != nil {
+				return nil, err
+			}
+			// Nothing to do after filtering
+			if filter != nil && len(resolvedContent) == 0 && len(newStruct.LaidOutContent) == 0 {
+				continue
+			}
+			newVol.LaidOutStructure[j].ResolvedContent = resolvedContent
+
+			// and add to updates
 			updates = append(updates, updatePair{
 				from: &oldVol.LaidOutStructure[j],
 				to:   &newVol.LaidOutStructure[j],
@@ -419,7 +508,7 @@ func updaterForStructureImpl(ps *LaidOutStructure, newRootDir, rollbackDir strin
 	if !ps.HasFilesystem() {
 		updater, err = newRawStructureUpdater(newRootDir, ps, rollbackDir, findDeviceForStructureWithFallback)
 	} else {
-		updater, err = newMountedFilesystemUpdater(newRootDir, ps, rollbackDir, findMountPointForStructure, observer)
+		updater, err = newMountedFilesystemUpdater(ps, rollbackDir, findMountPointForStructure, observer)
 	}
 	return updater, err
 }
