@@ -20,6 +20,7 @@
 package image
 
 import (
+	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/store"
 )
@@ -29,21 +30,16 @@ func MockToolingStore(sto Store) *ToolingStore {
 }
 
 var (
-	LocalSnaps           = localSnaps
 	DecodeModelAssertion = decodeModelAssertion
+	MakeLabel            = makeLabel
 	SetupSeed            = setupSeed
 	InstallCloudConfig   = installCloudConfig
-	SnapChannel          = snapChannel
 )
 
 func (tsto *ToolingStore) User() *auth.UserState {
 	return tsto.user
 }
 
-func (ls *localInfos) NameToPath() map[string]string {
-	return ls.nameToPath
-}
-
 func ToolingStoreContext() store.DeviceAndAuthContext {
 	return toolingStoreContext{}
 }
@@ -55,4 +51,14 @@ func (opts *DownloadOptions) Validate() error {
 var (
 	ErrRevisionAndCohort = errRevisionAndCohort
 	ErrPathInBase        = errPathInBase
+
+	WriteResolvedContent = writeResolvedContent
 )
+
+func MockWriteResolvedContent(f func(prepareImageDir string, info *gadget.Info, gadgetRoot, kernelRoot string) error) (restore func()) {
+	oldWriteResolvedContent := writeResolvedContent
+	writeResolvedContent = f
+	return func() {
+		writeResolvedContent = oldWriteResolvedContent
+	}
+}
