@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2017 Canonical Ltd
+ * Copyright (C) 2014-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -40,19 +40,21 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/snapasserts"
+	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/strutil"
 )
 
 // A Store can find metadata on snaps, download snaps and fetch assertions.
 type Store interface {
-	SnapAction(context.Context, []*store.CurrentSnap, []*store.SnapAction, *auth.UserState, *store.RefreshOptions) ([]store.SnapActionResult, error)
+	SnapAction(context.Context, []*store.CurrentSnap, []*store.SnapAction, store.AssertionQuery, *auth.UserState, *store.RefreshOptions) ([]store.SnapActionResult, []store.AssertionResult, error)
 	Download(ctx context.Context, name, targetFn string, downloadInfo *snap.DownloadInfo, pbar progress.Meter, user *auth.UserState, dlOpts *store.DownloadOptions) error
 
 	Assertion(assertType *asserts.AssertionType, primaryKey []string, user *auth.UserState) (asserts.Assertion, error)
@@ -127,7 +129,7 @@ func parseAuthFile(authFn string, data []byte) (*authData, error) {
 }
 
 func snapcraftLoginSection() string {
-	if osutil.GetenvBool("SNAPPY_USE_STAGING_STORE") {
+	if snapdenv.UseStagingStore() {
 		return "login.staging.ubuntu.com"
 	}
 	return "login.ubuntu.com"
@@ -260,19 +262,20 @@ func (opts *DownloadOptions) String() string {
 	return strings.Join(spec, " ")
 }
 
-// DownloadSnap downloads the snap with the given name and optionally revision
-// using the provided store and options. It returns the final full path of the
-// snap inside the opts.TargetDir and a snap.Info for the snap.
-func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targetFn string, info *snap.Info, err error) {
+// DownloadSnap downloads the snap with the given name and optionally
+// revision using the provided store and options. It returns the final
+// full path of the snap and a snap.Info for it and optionally a
+// channel the snap got redirected to.
+func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targetFn string, info *snap.Info, redirectChannel string, err error) {
 	if err := opts.validate(); err != nil {
-		return "", nil, err
+		return "", nil, "", err
 	}
 	sto := tsto.sto
 
 	if opts.TargetPathFunc == nil && opts.TargetDir == "" {
 		pwd, err := os.Getwd()
 		if err != nil {
-			return "", nil, err
+			return "", nil, "", err
 		}
 		opts.TargetDir = pwd
 	}
@@ -292,12 +295,13 @@ func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targe
 		Channel:      opts.Channel,
 	}}
 
-	sars, err := sto.SnapAction(context.TODO(), nil, actions, tsto.user, nil)
+	sars, _, err := sto.SnapAction(context.TODO(), nil, actions, nil, tsto.user, nil)
 	if err != nil {
 		// err will be 'cannot download snap "foo": <reasons>'
-		return "", nil, err
+		return "", nil, "", err
 	}
 	snap := sars[0].Info
+	redirectChannel = sars[0].RedirectChannel
 
 	if opts.TargetPathFunc == nil {
 		baseName := opts.Basename
@@ -311,7 +315,7 @@ func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targe
 		var err error
 		targetFn, err = opts.TargetPathFunc(snap)
 		if err != nil {
-			return "", nil, err
+			return "", nil, "", err
 		}
 	}
 
@@ -320,7 +324,7 @@ func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targe
 		sha3_384Dgst, size, err := osutil.FileDigest(targetFn, crypto.SHA3_384)
 		if err == nil && size == uint64(snap.DownloadInfo.Size) && fmt.Sprintf("%x", sha3_384Dgst) == snap.DownloadInfo.Sha3_384 {
 			logger.Debugf("not downloading, using existing file %s", targetFn)
-			return targetFn, snap, nil
+			return targetFn, snap, redirectChannel, nil
 		}
 		logger.Debugf("File exists but has wrong hash, ignoring (here).")
 	}
@@ -339,12 +343,12 @@ func (tsto *ToolingStore) DownloadSnap(name string, opts DownloadOptions) (targe
 
 	dlOpts := &store.DownloadOptions{LeavePartialOnError: opts.LeavePartialOnError}
 	if err = sto.Download(context.TODO(), name, targetFn, &snap.DownloadInfo, pb, tsto.user, dlOpts); err != nil {
-		return "", nil, err
+		return "", nil, "", err
 	}
 
 	signal.Reset(syscall.SIGINT)
 
-	return targetFn, snap, nil
+	return targetFn, snap, redirectChannel, nil
 }
 
 // AssertionFetcher creates an asserts.Fetcher for assertions against the given store using dlOpts for authorization, the fetcher will add assertions in the given database and after that also call save for each of them.
@@ -401,3 +405,55 @@ func (tsto *ToolingStore) Find(at *asserts.AssertionType, headers map[string]str
 	}
 	return tsto.sto.Assertion(at, pk, tsto.user)
 }
+
+// var so that it can be mocked for tests
+var writeResolvedContent = writeResolvedContentImpl
+
+// writeResolvedContent takes gadget.Info and the unpacked
+// gadget/kernel snaps and outputs the resolved content from the
+// {gadget,kernel}.yaml into a filesystem tree with the structure:
+// <prepareImageDir>/resolved-content/<volume-name>/part<structure-nr>/...
+//
+// E.g.
+// /tmp/prep-img/resolved-content/pi/part0/{config.txt,bootcode.bin,...}
+func writeResolvedContentImpl(prepareDir string, info *gadget.Info, gadgetUnpackDir, kernelUnpackDir string) error {
+	fullPrepareDir, err := filepath.Abs(prepareDir)
+	if err != nil {
+		return err
+	}
+	targetDir := filepath.Join(fullPrepareDir, "resolved-content")
+
+	for volName, vol := range info.Volumes {
+		pvol, err := gadget.LayoutVolume(gadgetUnpackDir, kernelUnpackDir, vol, gadget.DefaultConstraints)
+		if err != nil {
+			return err
+		}
+		for i, ps := range pvol.LaidOutStructure {
+			if !ps.HasFilesystem() {
+				continue
+			}
+			mw, err := gadget.NewMountedFilesystemWriter(&ps, nil)
+			if err != nil {
+				return err
+			}
+			// ubuntu-image uses the "part{}" nomenclature
+			dst := filepath.Join(targetDir, volName, fmt.Sprintf("part%d", i))
+			// on UC20, ensure system-seed links back to the
+			// <PrepareDir>/system-seed
+			if ps.Role == gadget.SystemSeed {
+				uc20systemSeedDir := filepath.Join(fullPrepareDir, "system-seed")
+				if err := os.MkdirAll(filepath.Dir(dst), 0755); err != nil {
+					return err
+				}
+				if err := os.Symlink(uc20systemSeedDir, dst); err != nil {
+					return err
+				}
+			}
+			if err := mw.Write(dst, nil); err != nil {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
