@@ -55,7 +55,6 @@ import (
 	"github.com/snapcore/snapd/release"
 	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/snap"
-	"github.com/snapcore/snapd/strutil"
 	"github.com/snapcore/snapd/timings"
 )
 
@@ -65,6 +64,10 @@ var (
 	isRootWritableOverlay = osutil.IsRootWritableOverlay
 	kernelFeatures        = apparmor_sandbox.KernelFeatures
 	parserFeatures        = apparmor_sandbox.ParserFeatures
+
+	// make sure that apparmor profile fulfills the late discarding backend
+	// interface
+	_ interfaces.SecurityBackendDiscardingLate = (*Backend)(nil)
 )
 
 // Backend is responsible for maintaining apparmor profiles for snaps and parts of snapd.
@@ -220,9 +223,24 @@ func snapConfineFromSnapProfile(info *snap.Info) (dir, glob string, content map[
 	//   /snap/core/111/usr/lib/snapd/snap-confine
 	// becomes
 	//   snap-confine.core.111
-	patchedProfileName := fmt.Sprintf("snap-confine.%s.%s", info.InstanceName(), info.Revision)
+	patchedProfileName := snapConfineProfileName(info.InstanceName(), info.Revision)
+	// remove other generated profiles, which is only relevant for the
+	// 'core' snap on classic system where we reexec, on core systems the
+	// profile is already a part of the rootfs snap
 	patchedProfileGlob := fmt.Sprintf("snap-confine.%s.*", info.InstanceName())
 
+	if info.Type() == snap.TypeSnapd {
+		// with the snapd snap, things are a little different, the
+		// profile is discarded only late for the revisions that are
+		// being removed, also on core devices the rootfs snap and the
+		// snapd snap are updated separately, so the profile needs to be
+		// around for as long as the given revision of the snapd snap is
+		// active, so we use the exact match such that we only replace
+		// our own profile, which can happen if system was rebooted
+		// before task calling the backend was finished
+		patchedProfileGlob = patchedProfileName
+	}
+
 	// Return information for EnsureDirState that describes the re-exec profile for snap-confine.
 	content = map[string]osutil.FileState{
 		patchedProfileName: &osutil.MemoryFileState{
@@ -234,6 +252,10 @@ func snapConfineFromSnapProfile(info *snap.Info) (dir, glob string, content map[
 	return dirs.SnapAppArmorDir, patchedProfileGlob, content, nil
 }
 
+func snapConfineProfileName(snapName string, rev snap.Revision) string {
+	return fmt.Sprintf("snap-confine.%s.%s", snapName, rev)
+}
+
 // setupSnapConfineReexec will setup apparmor profiles inside the host's
 // /var/lib/snapd/apparmor/profiles directory. This is needed for
 // running snap-confine from the core or snapd snap.
@@ -357,7 +379,7 @@ func (b *Backend) prepareProfiles(snapInfo *snap.Info, opts interfaces.Confineme
 	// Deal with the "snapd" snap - we do the setup slightly differently
 	// here because this will run both on classic and on Ubuntu Core 18
 	// systems but /etc/apparmor.d is not writable on core18 systems
-	if snapInfo.GetType() == snap.TypeSnapd && apparmor_sandbox.ProbedLevel() != apparmor_sandbox.Unsupported {
+	if snapInfo.Type() == snap.TypeSnapd && apparmor_sandbox.ProbedLevel() != apparmor_sandbox.Unsupported {
 		if err := b.setupSnapConfineReexec(snapInfo); err != nil {
 			return nil, fmt.Errorf("cannot create host snap-confine apparmor configuration: %s", err)
 		}
@@ -368,7 +390,7 @@ func (b *Backend) prepareProfiles(snapInfo *snap.Info, opts interfaces.Confineme
 	// See LP:#1460152 and
 	// https://forum.snapcraft.io/t/core-snap-revert-issues-on-core-devices/
 	//
-	if (snapInfo.GetType() == snap.TypeOS || snapInfo.GetType() == snap.TypeSnapd) && !release.OnClassic {
+	if (snapInfo.Type() == snap.TypeOS || snapInfo.Type() == snap.TypeSnapd) && !release.OnClassic {
 		if li, err := filepath.Glob(filepath.Join(apparmor_sandbox.SystemCacheDir, "*")); err == nil {
 			for _, p := range li {
 				if st, err := os.Stat(p); err == nil && st.Mode().IsRegular() && profileIsRemovableOnCoreSetup(p) {
@@ -542,6 +564,7 @@ func (b *Backend) Remove(snapName string) error {
 	globs := profileGlobs(snapName)
 	cache := apparmor_sandbox.CacheDir
 	_, removed, errEnsure := osutil.EnsureDirStateGlobs(dir, globs, nil)
+	// always try to unload affected profiles
 	errUnload := unloadProfiles(removed, cache)
 	if errEnsure != nil {
 		return fmt.Errorf("cannot synchronize security files for snap %q: %s", snapName, errEnsure)
@@ -549,6 +572,25 @@ func (b *Backend) Remove(snapName string) error {
 	return errUnload
 }
 
+func (b *Backend) RemoveLate(snapName string, rev snap.Revision, typ snap.Type) error {
+	logger.Debugf("remove late for snap %v (%s) type %v", snapName, rev, typ)
+	if typ != snap.TypeSnapd {
+		// late remove is relevant only for snap confine profiles
+		return nil
+	}
+
+	globs := []string{snapConfineProfileName(snapName, rev)}
+	_, removed, errEnsure := osutil.EnsureDirStateGlobs(dirs.SnapAppArmorDir, globs, nil)
+	// XXX: unloadProfiles() does not unload profiles from the kernel, but
+	// only removes profiles from the cache
+	// always try to unload the affected profile
+	errUnload := unloadProfiles(removed, apparmor_sandbox.CacheDir)
+	if errEnsure != nil {
+		return fmt.Errorf("cannot remove security profiles for snap %q (%s): %s", snapName, rev, errEnsure)
+	}
+	return errUnload
+}
+
 var (
 	templatePattern    = regexp.MustCompile("(###[A-Z_]+###)")
 	coreRuntimePattern = regexp.MustCompile("^core([0-9][0-9])?$")
@@ -611,23 +653,6 @@ func addUpdateNSProfile(snapInfo *snap.Info, opts interfaces.ConfinementOptions,
 	}
 }
 
-func downgradeConfinement() bool {
-	kver := osutil.KernelVersion()
-	switch {
-	case release.DistroLike("opensuse-tumbleweed"):
-		if cmp, _ := strutil.VersionCompare(kver, "4.16"); cmp >= 0 {
-			// As a special exception, for openSUSE Tumbleweed which ships Linux
-			// 4.16, do not downgrade the confinement template.
-			return false
-		}
-	case release.DistroLike("arch", "archlinux"):
-		// The default kernel has AppArmor enabled since 4.18.8, the
-		// hardened one since 4.17.4
-		return false
-	}
-	return true
-}
-
 func addContent(securityTag string, snapInfo *snap.Info, cmdName string, opts interfaces.ConfinementOptions, snippetForTag string, content map[string]osutil.FileState, spec *Specification) {
 	// If base is specified and it doesn't match the core snaps (not
 	// specifying a base should use the default core policy since in this
@@ -647,22 +672,6 @@ func addContent(securityTag string, snapInfo *snap.Info, cmdName string, opts in
 		policy = classicTemplate
 		ignoreSnippets = true
 	}
-	// When partial AppArmor is detected, use the classic template for now. We could
-	// use devmode, but that could generate confusing log entries for users running
-	// snaps on systems with partial AppArmor support.
-	if apparmor_sandbox.ProbedLevel() == apparmor_sandbox.Partial {
-		// By default, downgrade confinement to the classic template when
-		// partial AppArmor support is detected. We don't want to use strict
-		// in general yet because older versions of the kernel did not
-		// provide backwards compatible interpretation of confinement
-		// so the meaning of the template would change across kernel
-		// versions and we have not validated that the current template
-		// is operational on older kernels.
-		if downgradeConfinement() {
-			policy = classicTemplate
-			ignoreSnippets = true
-		}
-	}
 	// If a snap is in devmode (or is using classic confinement) then make the
 	// profile non-enforcing where violations are logged but not denied.
 	// This is also done for classic so that no confinement applies. Just in
@@ -774,10 +783,6 @@ func (b *Backend) SandboxFeatures() []string {
 	policy := "default"
 	if apparmor_sandbox.ProbedLevel() == apparmor_sandbox.Partial {
 		level = "partial"
-
-		if downgradeConfinement() {
-			policy = "downgraded"
-		}
 	}
 	tags = append(tags, fmt.Sprintf("support-level:%s", level))
 	tags = append(tags, fmt.Sprintf("policy:%s", policy))
