@@ -20,12 +20,15 @@
 package builtin
 
 import (
+	"fmt"
+
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -110,6 +113,22 @@ dbus (receive)
     member={ActionInvoked,NotificationClosed,NotificationReplied}
     peer=(label=unconfined),
 
+# KDE Plasma's Inhibited property indicating "do not disturb" mode
+# https://invent.kde.org/plasma/plasma-workspace/-/blob/master/libnotificationmanager/dbus/org.freedesktop.Notifications.xml#L42
+dbus (send)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member="Get{,All}"
+    peer=(label=unconfined),
+
+dbus (receive)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member=PropertiesChanged
+    peer=(label=unconfined),
+
 # DesktopAppInfo Launched
 dbus (send)
     bus=session
@@ -284,12 +303,34 @@ type desktopInterface struct {
 	commonInterface
 }
 
-func (iface *desktopInterface) fontconfigDirs() []string {
+func (iface *desktopInterface) shouldMountHostFontCache(attribs interfaces.Attrer) (bool, error) {
+	value, ok := attribs.Lookup("mount-host-font-cache")
+	if !ok {
+		// If the attribute is not present, we mount the font cache
+		return true, nil
+	}
+	shouldMount, ok := value.(bool)
+	if !ok {
+		return false, fmt.Errorf("desktop plug requires bool with 'mount-host-font-cache'")
+	}
+	return shouldMount, nil
+}
+
+func (iface *desktopInterface) fontconfigDirs(plug *interfaces.ConnectedPlug) ([]string, error) {
 	fontDirs := []string{
 		dirs.SystemFontsDir,
 		dirs.SystemLocalFontsDir,
 	}
-	return append(fontDirs, dirs.SystemFontconfigCacheDirs...)
+
+	shouldMountHostFontCache, err := iface.shouldMountHostFontCache(plug)
+	if err != nil {
+		return nil, err
+	}
+	if shouldMountHostFontCache {
+		fontDirs = append(fontDirs, dirs.SystemFontconfigCacheDirs...)
+	}
+
+	return fontDirs, nil
 }
 
 func (iface *desktopInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
@@ -307,7 +348,11 @@ func (iface *desktopInterface) AppArmorConnectedPlug(spec *apparmor.Specificatio
 	}
 
 	// Allow mounting fonts
-	for _, dir := range iface.fontconfigDirs() {
+	fontDirs, err := iface.fontconfigDirs(plug)
+	if err != nil {
+		return err
+	}
+	for _, dir := range fontDirs {
 		source := "/var/lib/snapd/hostfs" + dir
 		target := dirs.StripRootDir(dir)
 		emit("  # Read-only access to %s\n", target)
@@ -332,7 +377,11 @@ func (iface *desktopInterface) MountConnectedPlug(spec *mount.Specification, plu
 		return nil
 	}
 
-	for _, dir := range iface.fontconfigDirs() {
+	fontDirs, err := iface.fontconfigDirs(plug)
+	if err != nil {
+		return err
+	}
+	for _, dir := range fontDirs {
 		if !osutil.IsDirectory(dir) {
 			continue
 		}
@@ -363,6 +412,11 @@ func (iface *desktopInterface) MountConnectedPlug(spec *mount.Specification, plu
 	return nil
 }
 
+func (iface *desktopInterface) BeforePreparePlug(plug *snap.PlugInfo) error {
+	_, err := iface.shouldMountHostFontCache(plug)
+	return err
+}
+
 func init() {
 	registerIface(&desktopInterface{
 		commonInterface: commonInterface{
