@@ -27,7 +27,6 @@ import (
 	"github.com/snapcore/snapd/interfaces/kmod"
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/release"
-	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -145,7 +144,7 @@ pivot_root,
 
 # Docker needs to be able to create and load the profile it applies to
 # containers ("docker-default")
-/sbin/apparmor_parser ixr,
+/{,usr/}sbin/apparmor_parser ixr,
 /etc/apparmor.d/cache/ r,            # apparmor 2.12 and below
 /etc/apparmor.d/cache/.features r,
 /etc/apparmor.d/{,cache/}docker* rw,
@@ -203,6 +202,15 @@ ptrace (read, trace) peer=cri-containerd.apparmor.d,
 # snap), but in deployments where the control plane is not a snap, it will tell
 # containerd to use this path for various account information for pods.
 /run/secrets/kubernetes.io/{,**} rk,
+
+# Allow using the 'autobind' feature of bind() (eg, for journald via go-systemd)
+# unix (bind) type=dgram addr=auto,
+# TODO: when snapd vendors in AppArmor userspace, then enable the new syntax
+# above which allows only "empty"/automatic addresses, for now we simply permit
+# all addresses with SOCK_DGRAM type, which leaks info for other addresses than
+# what docker tries to use
+# see https://bugs.launchpad.net/snapd/+bug/1867216
+unix (bind) type=dgram,
 `
 
 const dockerSupportConnectedPlugSecComp = `
@@ -644,10 +652,6 @@ type dockerSupportInterface struct {
 	commonInterface
 }
 
-var (
-	parserFeatures = apparmor_sandbox.ParserFeatures
-)
-
 func (iface *dockerSupportInterface) KModConnectedPlug(spec *kmod.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	// https://kubernetes.io/docs/setup/production-environment/container-runtimes/
 	if err := spec.AddModule("overlay"); err != nil {
