@@ -55,11 +55,9 @@ capability sys_tty_config,
 owner /run/user/[0-9]*/wayland-[0-9]* rwk,
 # Allow access to common client Wayland sockets from non-snap clients
 /run/user/[0-9]*/{mesa,mutter,sdl,wayland-cursor,weston,xwayland}-shared-* rw,
-
-# Allow reading an Xwayland Xauth file
-# (see https://gitlab.gnome.org/GNOME/mutter/merge_requests/626)
-/run/user/[0-9]*/.mutter-Xwaylandauth.* r,
-/run/user/[0-9]*/mutter/Xauthority r,
+# Some Wayland based toolkits (Qt, GTK3, SDL2) and Xwayland create shm files to pass
+# to the server. Although they are passed by FD we still need rw access to the file.
+/run/user/[0-9]*/snap.*/{wayland-cursor,xwayland}-shared-* rw,
 
 # Allow write access to create /run/user/* to create XDG_RUNTIME_DIR (until
 # lp:1738197 is fixed). Note this is not needed if creating a session using
