@@ -203,6 +203,8 @@ func (s *baseDeclSuite) TestAutoConnectPlugSlot(c *C) {
 		"content":         true,
 		"home":            true,
 		"lxd-support":     true,
+		// netlink-driver needs the family-name attributes to match
+		"netlink-driver": true,
 	}
 
 	for _, iface := range all {
@@ -580,8 +582,6 @@ plugs:
 // ATM a nil entry means even stricter rules that would need be tested
 // separately and whose implementation is in flux for now
 var (
-	unconstrained = []string{"core", "kernel", "gadget", "app"}
-
 	slotInstallation = map[string][]string{
 		// other
 		"adb-support":             {"core"},
@@ -599,6 +599,7 @@ var (
 		"cups-control":            {"app", "core"},
 		"dbus":                    {"app"},
 		"docker-support":          {"core"},
+		"dsp":                     {"core", "gadget"},
 		"dummy":                   {"app"},
 		"fwupd":                   {"app", "core"},
 		"gpio":                    {"core", "gadget"},
@@ -616,6 +617,7 @@ var (
 		"mir":                     {"app"},
 		"modem-manager":           {"app", "core"},
 		"mpris":                   {"app"},
+		"netlink-driver":          {"core", "gadget"},
 		"network-manager":         {"app", "core"},
 		"network-manager-observe": {"app", "core"},
 		"network-status":          {"core"},
@@ -624,6 +626,7 @@ var (
 		"power-control":           {"core"},
 		"ppp":                     {"core"},
 		"pulseaudio":              {"app", "core"},
+		"pwm":                     {"core", "gadget"},
 		"raw-volume":              {"core", "gadget"},
 		"serial-port":             {"core", "gadget"},
 		"spi":                     {"core", "gadget"},
@@ -702,6 +705,7 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 	restricted := map[string]bool{
 		"block-devices":         true,
 		"classic-support":       true,
+		"dm-crypt":              true,
 		"docker-support":        true,
 		"greengrass-support":    true,
 		"gpio-control":          true,
@@ -935,6 +939,7 @@ func (s *baseDeclSuite) TestSanity(c *C) {
 		"audio-playback":        true,
 		"classic-support":       true,
 		"core-support":          true,
+		"dm-crypt":              true,
 		"docker-support":        true,
 		"greengrass-support":    true,
 		"gpio-control":          true,
