@@ -17,11 +17,12 @@
  *
  */
 
-package osutil
+package osutil_test
 
 import (
 	"fmt"
 
+	"github.com/snapcore/snapd/osutil"
 	. "gopkg.in/check.v1"
 )
 
@@ -32,14 +33,14 @@ var _ = Suite(&outputErrSuite{})
 func (ts *outputErrSuite) TestOutputErrOutputWithoutNewlines(c *C) {
 	output := "test output"
 	err := fmt.Errorf("test error")
-	formattedErr := OutputErr([]byte(output), err)
+	formattedErr := osutil.OutputErr([]byte(output), err)
 	c.Check(formattedErr, ErrorMatches, output)
 }
 
 func (ts *outputErrSuite) TestOutputErrOutputWithNewlines(c *C) {
 	output := "output line1\noutput line2"
 	err := fmt.Errorf("test error")
-	formattedErr := OutputErr([]byte(output), err)
+	formattedErr := osutil.OutputErr([]byte(output), err)
 	c.Check(formattedErr.Error(), Equals, `
 -----
 output line1
@@ -49,6 +50,6 @@ output line2
 
 func (ts *outputErrSuite) TestOutputErrNoOutput(c *C) {
 	err := fmt.Errorf("test error")
-	formattedErr := OutputErr([]byte{}, err)
+	formattedErr := osutil.OutputErr([]byte{}, err)
 	c.Check(formattedErr, Equals, err)
 }
