@@ -17,7 +17,7 @@
  *
  */
 
-package osutil
+package osutil_test
 
 import (
 	"fmt"
@@ -27,6 +27,7 @@ import (
 	"strings"
 	"syscall"
 
+	"github.com/snapcore/snapd/osutil"
 	. "gopkg.in/check.v1"
 )
 
@@ -35,7 +36,7 @@ type StatTestSuite struct{}
 var _ = Suite(&StatTestSuite{})
 
 func (ts *StatTestSuite) TestFileDoesNotExist(c *C) {
-	c.Assert(FileExists("/i-do-not-exist"), Equals, false)
+	c.Assert(osutil.FileExists("/i-do-not-exist"), Equals, false)
 }
 
 func (ts *StatTestSuite) TestFileExistsSimple(c *C) {
@@ -43,7 +44,7 @@ func (ts *StatTestSuite) TestFileExistsSimple(c *C) {
 	err := ioutil.WriteFile(fname, []byte(fname), 0644)
 	c.Assert(err, IsNil)
 
-	c.Assert(FileExists(fname), Equals, true)
+	c.Assert(osutil.FileExists(fname), Equals, true)
 }
 
 func (ts *StatTestSuite) TestFileExistsExistsOddPermissions(c *C) {
@@ -51,11 +52,11 @@ func (ts *StatTestSuite) TestFileExistsExistsOddPermissions(c *C) {
 	err := ioutil.WriteFile(fname, []byte(fname), 0100)
 	c.Assert(err, IsNil)
 
-	c.Assert(FileExists(fname), Equals, true)
+	c.Assert(osutil.FileExists(fname), Equals, true)
 }
 
 func (ts *StatTestSuite) TestIsDirectoryDoesNotExist(c *C) {
-	c.Assert(IsDirectory("/i-do-not-exist"), Equals, false)
+	c.Assert(osutil.IsDirectory("/i-do-not-exist"), Equals, false)
 }
 
 func (ts *StatTestSuite) TestIsDirectorySimple(c *C) {
@@ -63,7 +64,7 @@ func (ts *StatTestSuite) TestIsDirectorySimple(c *C) {
 	err := os.Mkdir(dname, 0700)
 	c.Assert(err, IsNil)
 
-	c.Assert(IsDirectory(dname), Equals, true)
+	c.Assert(osutil.IsDirectory(dname), Equals, true)
 }
 
 func (ts *StatTestSuite) TestIsSymlink(c *C) {
@@ -71,11 +72,11 @@ func (ts *StatTestSuite) TestIsSymlink(c *C) {
 	err := os.Symlink("/", sname)
 	c.Assert(err, IsNil)
 
-	c.Assert(IsSymlink(sname), Equals, true)
+	c.Assert(osutil.IsSymlink(sname), Equals, true)
 }
 
 func (ts *StatTestSuite) TestIsSymlinkNoSymlink(c *C) {
-	c.Assert(IsSymlink(c.MkDir()), Equals, false)
+	c.Assert(osutil.IsSymlink(c.MkDir()), Equals, false)
 }
 
 func (ts *StatTestSuite) TestExecutableExists(c *C) {
@@ -83,24 +84,26 @@ func (ts *StatTestSuite) TestExecutableExists(c *C) {
 	defer os.Setenv("PATH", oldPath)
 	d := c.MkDir()
 	os.Setenv("PATH", d)
-	c.Check(ExecutableExists("xyzzy"), Equals, false)
+	c.Check(osutil.ExecutableExists("xyzzy"), Equals, false)
 
 	fname := filepath.Join(d, "xyzzy")
 	c.Assert(ioutil.WriteFile(fname, []byte{}, 0644), IsNil)
-	c.Check(ExecutableExists("xyzzy"), Equals, false)
+	c.Check(osutil.ExecutableExists("xyzzy"), Equals, false)
 
 	c.Assert(os.Chmod(fname, 0755), IsNil)
-	c.Check(ExecutableExists("xyzzy"), Equals, true)
+	c.Check(osutil.ExecutableExists("xyzzy"), Equals, true)
 }
 
-func (s *StatTestSuite) TestLookPathDefaultGivesCorrectPath(c *C) {
-	lookPath = func(name string) (string, error) { return "/bin/true", nil }
-	c.Assert(LookPathDefault("true", "/bin/foo"), Equals, "/bin/true")
+func (ts *StatTestSuite) TestLookPathDefaultGivesCorrectPath(c *C) {
+	r := osutil.MockLookPath(func(name string) (string, error) { return "/bin/true", nil })
+	defer r()
+	c.Assert(osutil.LookPathDefault("true", "/bin/foo"), Equals, "/bin/true")
 }
 
-func (s *StatTestSuite) TestLookPathDefaultReturnsDefaultWhenNotFound(c *C) {
-	lookPath = func(name string) (string, error) { return "", fmt.Errorf("Not found") }
-	c.Assert(LookPathDefault("bar", "/bin/bla"), Equals, "/bin/bla")
+func (ts *StatTestSuite) TestLookPathDefaultReturnsDefaultWhenNotFound(c *C) {
+	r := osutil.MockLookPath(func(name string) (string, error) { return "", fmt.Errorf("Not found") })
+	defer r()
+	c.Assert(osutil.LookPathDefault("bar", "/bin/bla"), Equals, "/bin/bla")
 }
 
 func makeTestPath(c *C, path string, mode os.FileMode) string {
@@ -123,7 +126,7 @@ func makeTestPathInDir(c *C, dir, path string, mode os.FileMode) string {
 	return path
 }
 
-func (s *StatTestSuite) TestIsWritableDir(c *C) {
+func (ts *StatTestSuite) TestIsWritableDir(c *C) {
 	for _, t := range []struct {
 		path       string
 		mode       os.FileMode
@@ -143,12 +146,12 @@ func (s *StatTestSuite) TestIsWritableDir(c *C) {
 		{"file", 0600, true},
 		{"file", 0400, false},
 	} {
-		writable := IsWritable(makeTestPath(c, t.path, t.mode))
+		writable := osutil.IsWritable(makeTestPath(c, t.path, t.mode))
 		c.Check(writable, Equals, t.isWritable, Commentf("incorrect result for %q (%s), got %v, expected %v", t.path, t.mode, writable, t.isWritable))
 	}
 }
 
-func (s *StatTestSuite) TestIsDirNotExist(c *C) {
+func (ts *StatTestSuite) TestIsDirNotExist(c *C) {
 	for _, e := range []error{
 		os.ErrNotExist,
 		syscall.ENOENT,
@@ -160,18 +163,18 @@ func (s *StatTestSuite) TestIsDirNotExist(c *C) {
 		&os.SyscallError{Err: syscall.ENOENT},
 		&os.SyscallError{Err: syscall.ENOTDIR},
 	} {
-		c.Check(IsDirNotExist(e), Equals, true, Commentf("%#v (%v)", e, e))
+		c.Check(osutil.IsDirNotExist(e), Equals, true, Commentf("%#v (%v)", e, e))
 	}
 
 	for _, e := range []error{
 		nil,
 		fmt.Errorf("hello"),
 	} {
-		c.Check(IsDirNotExist(e), Equals, false)
+		c.Check(osutil.IsDirNotExist(e), Equals, false)
 	}
 }
 
-func (s *StatTestSuite) TestDirExists(c *C) {
+func (ts *StatTestSuite) TestDirExists(c *C) {
 	for _, t := range []struct {
 		make   string
 		path   string
@@ -189,7 +192,7 @@ func (s *StatTestSuite) TestDirExists(c *C) {
 		if t.make != "" {
 			makeTestPathInDir(c, base, t.make, 0755)
 		}
-		exists, isDir, err := DirExists(filepath.Join(base, t.path))
+		exists, isDir, err := osutil.DirExists(filepath.Join(base, t.path))
 		c.Check(exists, Equals, t.exists, comm)
 		c.Check(isDir, Equals, t.isDir, comm)
 		c.Check(err, IsNil, comm)
@@ -198,17 +201,17 @@ func (s *StatTestSuite) TestDirExists(c *C) {
 	p := makeTestPath(c, "foo/bar", 0)
 	c.Assert(os.Chmod(filepath.Dir(p), 0), IsNil)
 	defer os.Chmod(filepath.Dir(p), 0755)
-	exists, isDir, err := DirExists(p)
+	exists, isDir, err := osutil.DirExists(p)
 	c.Check(exists, Equals, false)
 	c.Check(isDir, Equals, false)
 	c.Check(err, NotNil)
 }
 
-func (s *StatTestSuite) TestIsExecutable(c *C) {
-	c.Check(IsExecutable("non-existent"), Equals, false)
-	c.Check(IsExecutable("."), Equals, false)
+func (ts *StatTestSuite) TestIsExecutable(c *C) {
+	c.Check(osutil.IsExecutable("non-existent"), Equals, false)
+	c.Check(osutil.IsExecutable("."), Equals, false)
 	dir := c.MkDir()
-	c.Check(IsExecutable(dir), Equals, false)
+	c.Check(osutil.IsExecutable(dir), Equals, false)
 
 	for _, tc := range []struct {
 		mode os.FileMode
@@ -230,6 +233,64 @@ func (s *StatTestSuite) TestIsExecutable(c *C) {
 
 		err = ioutil.WriteFile(p, []byte(""), tc.mode)
 		c.Assert(err, IsNil)
-		c.Check(IsExecutable(p), Equals, tc.is)
+		c.Check(osutil.IsExecutable(p), Equals, tc.is)
+	}
+}
+
+func (ts *StatTestSuite) TestRegularFileExists(c *C) {
+	tt := []struct {
+		make           bool
+		makeNonRegular bool
+		path           string
+		expExists      bool
+		expIsReg       bool
+		expErr         string
+		comment        string
+	}{
+		{
+			make:      true,
+			path:      "foo",
+			expExists: true,
+			expIsReg:  true,
+			comment:   "file is regular",
+		},
+		{
+			make:           true,
+			makeNonRegular: true,
+			path:           "bar",
+			expExists:      true,
+			comment:        "file is symlink",
+		},
+		{
+			path:      "not-exists",
+			expExists: false,
+			expErr:    ".*no such file or directory",
+			comment:   "file doesn't exist",
+		},
+	}
+
+	for _, t := range tt {
+		fullpath := filepath.Join(c.MkDir(), t.path)
+		comment := Commentf(t.comment)
+
+		if t.make {
+			if t.makeNonRegular {
+				// make it a symlink
+				err := os.Symlink("foo", fullpath)
+				c.Assert(err, IsNil, comment)
+			} else {
+				// make it a normal file
+				err := ioutil.WriteFile(fullpath, nil, 0644)
+				c.Assert(err, IsNil, comment)
+			}
+		}
+
+		exists, isReg, err := osutil.RegularFileExists(fullpath)
+		if t.expErr != "" {
+			c.Assert(err, ErrorMatches, t.expErr, comment)
+			continue
+		}
+		c.Assert(exists, Equals, t.expExists, comment)
+		c.Assert(isReg, Equals, t.expIsReg, comment)
 	}
 }
