@@ -34,6 +34,7 @@ import (
 
 	"golang.org/x/crypto/sha3"
 
+	"gopkg.in/check.v1"
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/asserts"
@@ -67,6 +68,7 @@ type assertMgrSuite struct {
 
 	storeSigning *assertstest.StoreStack
 	dev1Acct     *asserts.Account
+	dev1AcctKey  *asserts.AccountKey
 	dev1Signing  *assertstest.SigningDB
 
 	fakeStore        snapstate.StoreService
@@ -77,9 +79,10 @@ var _ = Suite(&assertMgrSuite{})
 
 type fakeStore struct {
 	storetest.Store
-	state                  *state.State
-	db                     asserts.RODatabase
-	maxDeclSupportedFormat int
+	state                           *state.State
+	db                              asserts.RODatabase
+	maxDeclSupportedFormat          int
+	maxValidationSetSupportedFormat int
 
 	requestedTypes [][]string
 
@@ -111,7 +114,7 @@ func (sto *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.Curren
 		panic("only assertion query supported")
 	}
 
-	toResolve, err := assertQuery.ToResolve()
+	toResolve, toResolveSeq, err := assertQuery.ToResolve()
 	if err != nil {
 		return nil, nil, err
 	}
@@ -123,8 +126,11 @@ func (sto *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.Curren
 	restore := asserts.MockMaxSupportedFormat(asserts.SnapDeclarationType, sto.maxDeclSupportedFormat)
 	defer restore()
 
+	restoreSeq := asserts.MockMaxSupportedFormat(asserts.ValidationSetType, sto.maxValidationSetSupportedFormat)
+	defer restoreSeq()
+
 	reqTypes := make(map[string]bool)
-	ares := make([]store.AssertionResult, 0, len(toResolve))
+	ares := make([]store.AssertionResult, 0, len(toResolve)+len(toResolveSeq))
 	for g, ats := range toResolve {
 		urls := make([]string, 0, len(ats))
 		for _, at := range ats {
@@ -143,6 +149,36 @@ func (sto *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.Curren
 			StreamURLs: urls,
 		})
 	}
+
+	for g, ats := range toResolveSeq {
+		urls := make([]string, 0, len(ats))
+		for _, at := range ats {
+			reqTypes[at.Type.Name] = true
+			var a asserts.Assertion
+			headers, err := asserts.HeadersFromSequenceKey(at.Type, at.SequenceKey)
+			if err != nil {
+				return nil, nil, err
+			}
+			if !at.Pinned {
+				a, err = sto.db.FindSequence(at.Type, headers, -1, asserts.ValidationSetType.MaxSupportedFormat())
+			} else {
+				a, err = at.Resolve(sto.db.Find)
+			}
+			if err != nil {
+				assertQuery.AddSequenceError(err, at)
+				continue
+			}
+			storeVs := a.(*asserts.ValidationSet)
+			if storeVs.Sequence() > at.Sequence || (storeVs.Sequence() == at.Sequence && storeVs.Revision() >= at.Revision) {
+				urls = append(urls, fmt.Sprintf("/assertions/%s/%s", a.Type().Name, strings.Join(a.At().PrimaryKey, "/")))
+			}
+		}
+		ares = append(ares, store.AssertionResult{
+			Grouping:   asserts.Grouping(g),
+			StreamURLs: urls,
+		})
+	}
+
 	// behave like the actual SnapAction if there are no results
 	if len(ares) == 0 {
 		return nil, ares, &store.SnapActionError{
@@ -170,6 +206,9 @@ func (sto *fakeStore) DownloadAssertions(urls []string, b *asserts.Batch, user *
 	resolve := func(ref *asserts.Ref) (asserts.Assertion, error) {
 		restore := asserts.MockMaxSupportedFormat(asserts.SnapDeclarationType, sto.maxDeclSupportedFormat)
 		defer restore()
+
+		restoreSeq := asserts.MockMaxSupportedFormat(asserts.ValidationSetType, sto.maxValidationSetSupportedFormat)
+		defer restoreSeq()
 		return ref.Resolve(sto.db.Find)
 	}
 
@@ -210,8 +249,8 @@ func (s *assertMgrSuite) SetUpTest(c *C) {
 	c.Assert(err, IsNil)
 
 	// developer signing
-	dev1AcctKey := assertstest.NewAccountKey(s.storeSigning, s.dev1Acct, nil, dev1PrivKey.PublicKey(), "")
-	err = s.storeSigning.Add(dev1AcctKey)
+	s.dev1AcctKey = assertstest.NewAccountKey(s.storeSigning, s.dev1Acct, nil, dev1PrivKey.PublicKey(), "")
+	err = s.storeSigning.Add(s.dev1AcctKey)
 	c.Assert(err, IsNil)
 
 	s.dev1Signing = assertstest.NewSigningDB(s.dev1Acct.AccountID(), dev1PrivKey)
@@ -229,7 +268,8 @@ func (s *assertMgrSuite) SetUpTest(c *C) {
 	s.fakeStore = &fakeStore{
 		state: s.state,
 		db:    s.storeSigning,
-		maxDeclSupportedFormat: asserts.SnapDeclarationType.MaxSupportedFormat(),
+		maxDeclSupportedFormat:          asserts.SnapDeclarationType.MaxSupportedFormat(),
+		maxValidationSetSupportedFormat: asserts.ValidationSetType.MaxSupportedFormat(),
 	}
 	s.trivialDeviceCtx = &snapstatetest.TrivialDeviceContext{
 		CtxStore: s.fakeStore,
@@ -841,6 +881,29 @@ func (s *assertMgrSuite) TestValidateSnapCrossCheckFail(c *C) {
 	c.Assert(chg.Err(), ErrorMatches, `(?s).*cannot install "f", snap "f" is undergoing a rename to "foo".*`)
 }
 
+func (s *assertMgrSuite) validationSetAssert(c *C, name, sequence, revision string) *asserts.ValidationSet {
+	snaps := []interface{}{map[string]interface{}{
+		"id":       "qOqKhntON3vR7kwEbVPsILm7bUViPDzz",
+		"name":     "foo",
+		"presence": "required",
+		"revision": "1",
+	}}
+	headers := map[string]interface{}{
+		"series":       "16",
+		"account-id":   s.dev1Acct.AccountID(),
+		"authority-id": s.dev1Acct.AccountID(),
+		"publisher-id": s.dev1Acct.AccountID(),
+		"name":         name,
+		"sequence":     sequence,
+		"snaps":        snaps,
+		"timestamp":    time.Now().Format(time.RFC3339),
+		"revision":     revision,
+	}
+	a, err := s.dev1Signing.Sign(asserts.ValidationSetType, headers, nil, "")
+	c.Assert(err, IsNil)
+	return a.(*asserts.ValidationSet)
+}
+
 func (s *assertMgrSuite) snapDecl(c *C, name string, extraHeaders map[string]interface{}) *asserts.SnapDeclaration {
 	headers := map[string]interface{}{
 		"series":       "16",
@@ -2076,3 +2139,428 @@ func (s *assertMgrSuite) TestStore(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(store.Store(), Equals, "foo")
 }
+
+// validation-sets related tests
+
+func (s *assertMgrSuite) TestRefreshValidationSetAssertionsNop(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	err := assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionsAutoRefresh(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), IsNil)
+
+	// store key already present
+	c.Assert(assertstate.Add(s.state, s.storeSigning.StoreAccountKey("")), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1Acct), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1AcctKey), IsNil)
+
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(s.state, vsetAs1), IsNil)
+
+	vsetAs2 := s.validationSetAssert(c, "bar", "2", "3")
+	c.Assert(s.storeSigning.Add(vsetAs2), IsNil)
+
+	tr := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   1,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr)
+
+	c.Assert(assertstate.AutoRefreshAssertions(s.state, 0), IsNil)
+
+	a, err := assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "2",
+	})
+	c.Assert(err, IsNil)
+	c.Check(a.Revision(), Equals, 3)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionsAutoRefreshError(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), IsNil)
+
+	tr := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   1,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr)
+	err := assertstate.AutoRefreshAssertions(s.state, 0)
+	c.Assert(asserts.IsNotFound(err), Equals, true)
+}
+
+func (s *assertMgrSuite) TestRefreshValidationSetAssertionsStoreError(c *C) {
+	s.fakeStore.(*fakeStore).snapActionErr = &store.UnexpectedHTTPStatusError{StatusCode: 400}
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	// store key already present
+	c.Assert(assertstate.Add(s.state, s.storeSigning.StoreAccountKey("")), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1Acct), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1AcctKey), IsNil)
+
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(s.state, vsetAs1), IsNil)
+
+	tr := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   1,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr)
+
+	err := assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, ErrorMatches, `cannot refresh validation set assertions: cannot : got unexpected HTTP status code 400.*`)
+}
+
+func (s *assertMgrSuite) TestRefreshValidationSetAssertions(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	// store key already present
+	c.Assert(assertstate.Add(s.state, s.storeSigning.StoreAccountKey("")), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1Acct), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1AcctKey), IsNil)
+
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(s.state, vsetAs1), IsNil)
+
+	vsetAs2 := s.validationSetAssert(c, "bar", "1", "2")
+	err = s.storeSigning.Add(vsetAs2)
+	c.Assert(err, IsNil)
+
+	tr := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   1,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr)
+
+	err = assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+
+	a, err := assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "1",
+	})
+	c.Assert(err, IsNil)
+	c.Check(a.(*asserts.ValidationSet).Name(), Equals, "bar")
+	c.Check(a.Revision(), Equals, 2)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "validation-set"},
+	})
+
+	// sequence changed in the store to 4
+	vsetAs3 := s.validationSetAssert(c, "bar", "4", "3")
+	err = s.storeSigning.Add(vsetAs3)
+	c.Assert(err, IsNil)
+
+	// sanity check - sequence 4 not available locally yet
+	_, err = assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "4",
+	})
+	c.Assert(asserts.IsNotFound(err), Equals, true)
+
+	s.fakeStore.(*fakeStore).requestedTypes = nil
+	err = assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "validation-set"},
+	})
+
+	// new sequence is available in the db
+	a, err = assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "4",
+	})
+	c.Assert(err, IsNil)
+	c.Check(a.(*asserts.ValidationSet).Name(), Equals, "bar")
+
+	// tracking current was updated
+	c.Assert(assertstate.GetValidationSet(s.state, s.dev1Acct.AccountID(), "bar", &tr), IsNil)
+	c.Check(tr.Current, Equals, 4)
+}
+
+func (s *assertMgrSuite) TestRefreshValidationSetAssertionsPinned(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	// store key already present
+	err = assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+
+	c.Assert(assertstate.Add(s.state, s.dev1Acct), IsNil)
+	c.Assert(assertstate.Add(s.state, s.dev1AcctKey), IsNil)
+
+	vsetAs1 := s.validationSetAssert(c, "bar", "2", "1")
+	c.Assert(assertstate.Add(s.state, vsetAs1), IsNil)
+
+	vsetAs2 := s.validationSetAssert(c, "bar", "2", "5")
+	err = s.storeSigning.Add(vsetAs2)
+	c.Assert(err, IsNil)
+
+	tr := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   2,
+		PinnedAt:  2,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr)
+
+	err = assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+
+	a, err := assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "2",
+	})
+	c.Assert(err, IsNil)
+	c.Check(a.(*asserts.ValidationSet).Name(), Equals, "bar")
+	c.Check(a.(*asserts.ValidationSet).Sequence(), Equals, 2)
+	c.Check(a.Revision(), Equals, 5)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "validation-set"},
+	})
+
+	// sequence changed in the store to 7
+	vsetAs3 := s.validationSetAssert(c, "bar", "7", "8")
+	err = s.storeSigning.Add(vsetAs3)
+	c.Assert(err, IsNil)
+
+	s.fakeStore.(*fakeStore).requestedTypes = nil
+	err = assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "validation-set"},
+	})
+
+	// new sequence is not available in the db
+	_, err = assertstate.DB(s.state).Find(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "7",
+	})
+	c.Assert(asserts.IsNotFound(err), Equals, true)
+
+	// tracking current remains at 2
+	c.Assert(assertstate.GetValidationSet(s.state, s.dev1Acct.AccountID(), "bar", &tr), IsNil)
+	c.Check(tr.Current, Equals, 2)
+}
+
+func (s *assertMgrSuite) TestRefreshValidationSetAssertionsLocalOnlyFailed(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	// store key already present
+	c.Assert(assertstate.Add(st, s.storeSigning.StoreAccountKey("")), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1Acct), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1AcctKey), check.IsNil)
+
+	// add to local database
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(st, vsetAs1), check.IsNil)
+	vsetAs2 := s.validationSetAssert(c, "baz", "3", "1")
+	c.Assert(assertstate.Add(st, vsetAs2), check.IsNil)
+
+	// vset2 present and updated in the store
+	vsetAs2_2 := s.validationSetAssert(c, "baz", "3", "2")
+	err = s.storeSigning.Add(vsetAs2_2)
+	c.Assert(err, IsNil)
+
+	tr1 := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "bar",
+		Mode:      assertstate.Monitor,
+		Current:   1,
+		PinnedAt:  1,
+		LocalOnly: true,
+	}
+	tr2 := assertstate.ValidationSetTracking{
+		AccountID: s.dev1Acct.AccountID(),
+		Name:      "baz",
+		Mode:      assertstate.Monitor,
+		Current:   3,
+		PinnedAt:  3,
+	}
+	assertstate.UpdateValidationSet(s.state, &tr1)
+	assertstate.UpdateValidationSet(s.state, &tr2)
+
+	err = assertstate.RefreshValidationSetAssertions(s.state, 0)
+	c.Assert(err, IsNil)
+
+	// sanity - local assertion vsetAs1 is the latest
+	a, err := assertstate.DB(s.state).FindSequence(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "bar",
+		"sequence":   "1",
+	}, -1, -1)
+	c.Assert(err, IsNil)
+	vs := a.(*asserts.ValidationSet)
+	c.Check(vs.Name(), Equals, "bar")
+	c.Check(vs.Sequence(), Equals, 1)
+	c.Check(vs.Revision(), Equals, 1)
+
+	// but vsetAs2 was updated with vsetAs2_2
+	a, err = assertstate.DB(s.state).FindSequence(asserts.ValidationSetType, map[string]string{
+		"series":     "16",
+		"account-id": s.dev1Acct.AccountID(),
+		"name":       "baz",
+		"sequence":   "1",
+	}, -1, -1)
+	c.Assert(err, IsNil)
+	vs = a.(*asserts.ValidationSet)
+	c.Check(vs.Name(), Equals, "baz")
+	c.Check(vs.Sequence(), Equals, 3)
+	c.Check(vs.Revision(), Equals, 2)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionForMonitorLocalFallbackForPinned(c *C) {
+	st := s.state
+
+	st.Lock()
+	defer st.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), check.IsNil)
+	c.Assert(assertstate.Add(st, s.storeSigning.StoreAccountKey("")), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1Acct), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1AcctKey), check.IsNil)
+
+	// add to local database
+	vsetAs := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(st, vsetAs), check.IsNil)
+
+	opts := assertstate.ResolveOptions{AllowLocalFallback: true}
+	vs, local, err := assertstate.ValidationSetAssertionForMonitor(st, s.dev1Acct.AccountID(), "bar", 1, true, 0, &opts)
+	c.Assert(err, IsNil)
+	c.Assert(vs, NotNil)
+	c.Assert(local, Equals, true)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionForMonitorPinnedRefreshedFromStore(c *C) {
+	st := s.state
+
+	st.Lock()
+	defer st.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), check.IsNil)
+	c.Assert(assertstate.Add(st, s.storeSigning.StoreAccountKey("")), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1Acct), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1AcctKey), check.IsNil)
+
+	// add to local database
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(st, vsetAs1), check.IsNil)
+
+	// newer revision available in the store
+	vsetAs2 := s.validationSetAssert(c, "bar", "1", "2")
+	c.Assert(s.storeSigning.Add(vsetAs2), check.IsNil)
+
+	vs, local, err := assertstate.ValidationSetAssertionForMonitor(st, s.dev1Acct.AccountID(), "bar", 1, true, 0, nil)
+	c.Assert(err, IsNil)
+	c.Assert(local, Equals, false)
+	c.Check(vs.Revision(), Equals, 2)
+	c.Check(vs.Sequence(), Equals, 1)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionForMonitorUnpinnedRefreshedFromStore(c *C) {
+	st := s.state
+
+	st.Lock()
+	defer st.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), check.IsNil)
+	c.Assert(assertstate.Add(st, s.storeSigning.StoreAccountKey("")), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1Acct), check.IsNil)
+	c.Assert(assertstate.Add(st, s.dev1AcctKey), check.IsNil)
+
+	// add to local database
+	vsetAs1 := s.validationSetAssert(c, "bar", "1", "1")
+	c.Assert(assertstate.Add(st, vsetAs1), check.IsNil)
+
+	// newer assertion available in the store
+	vsetAs2 := s.validationSetAssert(c, "bar", "3", "1")
+	c.Assert(s.storeSigning.Add(vsetAs2), check.IsNil)
+
+	vs, local, err := assertstate.ValidationSetAssertionForMonitor(st, s.dev1Acct.AccountID(), "bar", 0, false, 0, nil)
+	c.Assert(err, IsNil)
+	c.Assert(local, Equals, false)
+	c.Check(vs.Revision(), Equals, 1)
+	c.Check(vs.Sequence(), Equals, 3)
+}
+
+func (s *assertMgrSuite) TestValidationSetAssertionForMonitorUnpinnedNotFound(c *C) {
+	st := s.state
+
+	st.Lock()
+	defer st.Unlock()
+
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	c.Assert(s.storeSigning.Add(storeAs), check.IsNil)
+
+	_, _, err := assertstate.ValidationSetAssertionForMonitor(st, s.dev1Acct.AccountID(), "bar", 0, false, 0, nil)
+	c.Assert(err, check.ErrorMatches, fmt.Sprintf(`cannot fetch and resolve assertions:\n - validation-set/16/%s/bar: validation-set assertion not found.*`, s.dev1Acct.AccountID()))
+}
