@@ -38,9 +38,15 @@ type flags struct {
 	// coreOnlyConfig tells Run/FilesystemOnlyApply to apply the config on core
 	// systems only.
 	coreOnlyConfig bool
-	// validatedOnylStateConfig tells that the config requires only validation,
+	// validatedOnlyStateConfig tells that the config requires only validation,
 	// its options are applied dynamically elsewhere.
 	validatedOnlyStateConfig bool
+	// earlyConfigFilter expresses whether the handler supports
+	// any early configuration options (that can and must be
+	// set before even seeding is finished).
+	// If set the function should copy such options from values
+	// to early.
+	earlyConfigFilter filterFunc
 }
 
 type fsOnlyHandler struct {
@@ -62,7 +68,7 @@ func init() {
 	addFSOnlyHandler(validateWatchdogOptions, handleWatchdogConfiguration, coreOnly)
 
 	// Export experimental.* flags to a place easily accessible from snapd helpers.
-	addFSOnlyHandler(validateExperimentalSettings, doExportExperimentalFlags, nil)
+	addFSOnlyHandler(validateExperimentalSettings, doExportExperimentalFlags, &flags{earlyConfigFilter: earlyExperimentalSettingsFilter})
 
 	// network.disable-ipv6
 	addFSOnlyHandler(validateNetworkSettings, handleNetworkConfiguration, coreOnly)
@@ -89,7 +95,7 @@ func init() {
 	addFSOnlyHandler(validateTimezoneSettings, handleTimezoneConfiguration, coreOnly)
 
 	sysconfig.ApplyFilesystemOnlyDefaultsImpl = func(rootDir string, defaults map[string]interface{}, options *sysconfig.FilesystemOnlyApplyOptions) error {
-		return filesystemOnlyApply(rootDir, plainCoreConfig(defaults), options)
+		return filesystemOnlyApply(rootDir, defaults, options)
 	}
 }
 
@@ -135,12 +141,20 @@ func (h *fsOnlyHandler) handle(cfg config.ConfGetter, opts *fsOnlyContext) error
 // early during boot, before all the configuration is applied as part of
 // normal execution of configure hook.
 // Exposed for use via sysconfig.ApplyFilesystemOnlyDefaults.
-func filesystemOnlyApply(rootDir string, cfg config.ConfGetter, opts *sysconfig.FilesystemOnlyApplyOptions) error {
+func filesystemOnlyApply(rootDir string, values map[string]interface{}, opts *sysconfig.FilesystemOnlyApplyOptions) error {
 	if rootDir == "" {
 		return fmt.Errorf("internal error: root directory for configcore.FilesystemOnlyApply() not set")
 	}
 
-	ctx := &fsOnlyContext{RootDir: rootDir}
+	if opts == nil {
+		opts = &sysconfig.FilesystemOnlyApplyOptions{}
+	}
+
+	cfg := plainCoreConfig(values)
+
+	ctx := &fsOnlyContext{
+		RootDir: rootDir,
+	}
 	for _, h := range handlers {
 		if h.needsState() {
 			continue
