@@ -27,8 +27,8 @@ import (
 
 	. "gopkg.in/check.v1"
 
-	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/configstate/configcore"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
@@ -37,13 +37,31 @@ import (
 
 type servicesSuite struct {
 	configcoreSuite
+	serviceInstalled bool
 }
 
 var _ = Suite(&servicesSuite{})
 
 func (s *servicesSuite) SetUpTest(c *C) {
 	s.configcoreSuite.SetUpTest(c)
+	s.systemctlOutput = func(args ...string) []byte {
+		var output []byte
+		if args[0] == "show" {
+			if args[1] == "--property=ActiveState" {
+				output = []byte("ActiveState=inactive")
+			} else {
+				if s.serviceInstalled {
+					output = []byte(fmt.Sprintf("Id=%s\nType=daemon\nActiveState=inactive\nUnitFileState=enabled\n", args[2]))
+				} else {
+					output = []byte(fmt.Sprintf("Id=%s\nType=\nActiveState=inactive\nUnitFileState=\n", args[2]))
+				}
+			}
+		}
+		return output
+	}
+
 	c.Assert(os.MkdirAll(filepath.Join(dirs.GlobalRootDir, "etc"), 0755), IsNil)
+	s.serviceInstalled = true
 	s.systemctlArgs = nil
 	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
 }
@@ -65,8 +83,9 @@ func (s *servicesSuite) TestConfigureServiceNotDisabled(c *C) {
 	err := configcore.SwitchDisableService("sshd.service", false, nil)
 	c.Assert(err, IsNil)
 	c.Check(s.systemctlArgs, DeepEquals, [][]string{
-		{"--root", dirs.GlobalRootDir, "unmask", "sshd.service"},
-		{"--root", dirs.GlobalRootDir, "enable", "sshd.service"},
+		{"show", "--property=Id,ActiveState,UnitFileState,Type", "sshd.service"},
+		{"unmask", "sshd.service"},
+		{"enable", "sshd.service"},
 		{"start", "sshd.service"},
 	})
 }
@@ -75,8 +94,9 @@ func (s *servicesSuite) TestConfigureServiceDisabled(c *C) {
 	err := configcore.SwitchDisableService("sshd.service", true, nil)
 	c.Assert(err, IsNil)
 	c.Check(s.systemctlArgs, DeepEquals, [][]string{
-		{"--root", dirs.GlobalRootDir, "disable", "sshd.service"},
-		{"--root", dirs.GlobalRootDir, "mask", "sshd.service"},
+		{"show", "--property=Id,ActiveState,UnitFileState,Type", "sshd.service"},
+		{"disable", "sshd.service"},
+		{"mask", "sshd.service"},
 		{"stop", "sshd.service"},
 		{"show", "--property=ActiveState", "sshd.service"},
 	})
@@ -92,12 +112,17 @@ func (s *servicesSuite) TestConfigureServiceDisabledIntegration(c *C) {
 	for _, service := range []struct {
 		cfgName     string
 		systemdName string
+		installed   bool
 	}{
-		{"ssh", "ssh.service"},
-		{"rsyslog", "rsyslog.service"},
+		{"ssh", "ssh.service", true},  // no installed check for ssh
+		{"ssh", "ssh.service", false}, // no installed check for ssh
+		{"rsyslog", "rsyslog.service", true},
+		{"rsyslog", "rsyslog.service", false},
+		{"systemd-resolved", "systemd-resolved.service", true},
+		{"systemd-resolved", "systemd-resolved.service", false},
 	} {
 		s.systemctlArgs = nil
-
+		s.serviceInstalled = service.installed
 		err := configcore.Run(&mockConf{
 			state: s.state,
 			conf: map[string]interface{}{
@@ -116,12 +141,19 @@ func (s *servicesSuite) TestConfigureServiceDisabledIntegration(c *C) {
 				{"show", "--property=ActiveState", srv},
 			})
 		default:
-			c.Check(s.systemctlArgs, DeepEquals, [][]string{
-				{"--root", dirs.GlobalRootDir, "disable", srv},
-				{"--root", dirs.GlobalRootDir, "mask", srv},
-				{"stop", srv},
-				{"show", "--property=ActiveState", srv},
-			})
+			if service.installed {
+				c.Check(s.systemctlArgs, DeepEquals, [][]string{
+					{"show", "--property=Id,ActiveState,UnitFileState,Type", srv},
+					{"disable", srv},
+					{"mask", srv},
+					{"stop", srv},
+					{"show", "--property=ActiveState", srv},
+				})
+			} else {
+				c.Check(s.systemctlArgs, DeepEquals, [][]string{
+					{"show", "--property=Id,ActiveState,UnitFileState,Type", srv},
+				})
+			}
 		}
 	}
 }
@@ -237,7 +269,7 @@ func (s *servicesSuite) TestConfigureConsoleConfEnableDuringInstallMode(c *C) {
 	mockProcCmdline := filepath.Join(c.MkDir(), "cmdline")
 	err := ioutil.WriteFile(mockProcCmdline, []byte("snapd_recovery_mode=install snapd_recovery_system=20201212\n"), 0644)
 	c.Assert(err, IsNil)
-	restore = boot.MockProcCmdline(mockProcCmdline)
+	restore = osutil.MockProcCmdline(mockProcCmdline)
 	defer restore()
 
 	err = configcore.Run(&mockConf{
@@ -260,11 +292,17 @@ func (s *servicesSuite) TestConfigureServiceEnableIntegration(c *C) {
 	for _, service := range []struct {
 		cfgName     string
 		systemdName string
+		installed   bool
 	}{
-		{"ssh", "ssh.service"},
-		{"rsyslog", "rsyslog.service"},
+		{"ssh", "ssh.service", true},  // no installed check for ssh
+		{"ssh", "ssh.service", false}, // no installed check for ssh
+		{"rsyslog", "rsyslog.service", true},
+		{"rsyslog", "rsyslog.service", false},
+		{"systemd-resolved", "systemd-resolved.service", true},
+		{"systemd-resolved", "systemd-resolved.service", false},
 	} {
 		s.systemctlArgs = nil
+		s.serviceInstalled = service.installed
 		err := configcore.Run(&mockConf{
 			state: s.state,
 			conf: map[string]interface{}{
@@ -277,19 +315,26 @@ func (s *servicesSuite) TestConfigureServiceEnableIntegration(c *C) {
 		switch service.cfgName {
 		case "ssh":
 			c.Check(s.systemctlArgs, DeepEquals, [][]string{
-				{"--root", dirs.GlobalRootDir, "unmask", "sshd.service"},
-				{"--root", dirs.GlobalRootDir, "unmask", "ssh.service"},
+				{"unmask", "sshd.service"},
+				{"unmask", "ssh.service"},
 				{"start", srv},
 			})
 			sshCanary := filepath.Join(dirs.GlobalRootDir, "/etc/ssh/sshd_not_to_be_run")
 			_, err := os.Stat(sshCanary)
 			c.Assert(err, ErrorMatches, ".* no such file or directory")
 		default:
-			c.Check(s.systemctlArgs, DeepEquals, [][]string{
-				{"--root", dirs.GlobalRootDir, "unmask", srv},
-				{"--root", dirs.GlobalRootDir, "enable", srv},
-				{"start", srv},
-			})
+			if service.installed {
+				c.Check(s.systemctlArgs, DeepEquals, [][]string{
+					{"show", "--property=Id,ActiveState,UnitFileState,Type", srv},
+					{"unmask", srv},
+					{"enable", srv},
+					{"start", srv},
+				})
+			} else {
+				c.Check(s.systemctlArgs, DeepEquals, [][]string{
+					{"show", "--property=Id,ActiveState,UnitFileState,Type", srv},
+				})
+			}
 		}
 	}
 }
