@@ -31,6 +31,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/interfaces/hotplug"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/hookstate"
@@ -138,6 +139,9 @@ func (s *hotplugSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(c.MkDir())
 	c.Assert(os.MkdirAll(filepath.Dir(dirs.SnapSystemKeyFile), 0755), IsNil)
 
+	restore := osutil.MockMountInfo("")
+	s.AddCleanup(restore)
+
 	s.o = overlord.Mock()
 	s.state = s.o.State()
 
@@ -270,19 +274,10 @@ func (s *hotplugSuite) TearDownTest(c *C) {
 	s.mockSnapCmd.Restore()
 }
 
-func testPlugSlotRefs(c *C, t *state.Task, plugSnap, plugName, slotSnap, slotName string) {
-	var plugRef interfaces.PlugRef
-	var slotRef interfaces.SlotRef
-	c.Assert(t.Get("plug", &plugRef), IsNil)
-	c.Assert(t.Get("slot", &slotRef), IsNil)
-	c.Assert(plugRef, DeepEquals, interfaces.PlugRef{Snap: plugSnap, Name: plugName})
-	c.Assert(slotRef, DeepEquals, interfaces.SlotRef{Snap: slotSnap, Name: slotName})
-}
-
 func testHotplugTaskAttrs(c *C, t *state.Task, ifaceName, hotplugKey string) {
 	iface, key, err := ifacestate.GetHotplugAttrs(t)
 	c.Assert(err, IsNil)
-	c.Assert(key, Equals, hotplugKey)
+	c.Assert(key, Equals, snap.HotplugKey(hotplugKey))
 	c.Assert(iface, Equals, ifaceName)
 }
 
@@ -1123,18 +1118,12 @@ func (s *hotplugSuite) TestUpdateDeviceTasks(c *C) {
 
 	task1 := tss.Tasks()[0]
 	c.Assert(task1.Kind(), Equals, "hotplug-disconnect")
-
-	iface, key, err := ifacestate.GetHotplugAttrs(task1)
-	c.Assert(err, IsNil)
-	c.Assert(iface, Equals, "interface")
-	c.Assert(key, DeepEquals, snap.HotplugKey("key"))
+	testHotplugTaskAttrs(c, task1, "interface", "key")
 
 	task2 := tss.Tasks()[1]
 	c.Assert(task2.Kind(), Equals, "hotplug-update-slot")
-	iface, key, err = ifacestate.GetHotplugAttrs(task2)
-	c.Assert(err, IsNil)
-	c.Assert(iface, Equals, "interface")
-	c.Assert(key, DeepEquals, snap.HotplugKey("key"))
+	testHotplugTaskAttrs(c, task2, "interface", "key")
+
 	var attrs map[string]interface{}
 	c.Assert(task2.Get("slot-attrs", &attrs), IsNil)
 	c.Assert(attrs, DeepEquals, map[string]interface{}{"attr": "value"})
@@ -1155,18 +1144,11 @@ func (s *hotplugSuite) TestRemoveDeviceTasks(c *C) {
 
 	task1 := tss.Tasks()[0]
 	c.Assert(task1.Kind(), Equals, "hotplug-disconnect")
-
-	iface, key, err := ifacestate.GetHotplugAttrs(task1)
-	c.Assert(err, IsNil)
-	c.Assert(iface, Equals, "interface")
-	c.Assert(key, DeepEquals, snap.HotplugKey("key"))
+	testHotplugTaskAttrs(c, task1, "interface", "key")
 
 	task2 := tss.Tasks()[1]
 	c.Assert(task2.Kind(), Equals, "hotplug-remove-slot")
-	iface, key, err = ifacestate.GetHotplugAttrs(task2)
-	c.Assert(err, IsNil)
-	c.Assert(iface, Equals, "interface")
-	c.Assert(key, DeepEquals, snap.HotplugKey("key"))
+	testHotplugTaskAttrs(c, task2, "interface", "key")
 
 	wt := task2.WaitTasks()
 	c.Assert(wt, HasLen, 1)
