@@ -27,7 +27,6 @@ import (
 
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/asserts"
-	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/snapstate/backend"
@@ -35,6 +34,7 @@ import (
 	"github.com/snapcore/snapd/release"
 	seccomp_compiler "github.com/snapcore/snapd/sandbox/seccomp"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdtool"
 )
 
 // featureSet contains the flag values that can be listed in assumes entries
@@ -46,6 +46,9 @@ var featureSet = map[string]bool{
 	"snap-env": true,
 	// Support for the "command-chain" feature for apps and hooks in snap.yaml
 	"command-chain": true,
+	// Support for "kernel-assets" in gadget.yaml. I.e. having volume
+	// content of the style $kernel:ref`
+	"kernel-assets": true,
 }
 
 // supportedSystemUsernames for now contains the hardcoded list of system
@@ -196,16 +199,28 @@ func checkAssumes(si *snap.Info) error {
 	return nil
 }
 
-var versionExp = regexp.MustCompile(`^([1-9][0-9]*)(?:\.([0-9]+)(?:\.([0-9]+))?)?`)
+// regular expression which matches a version expressed as groups of digits
+// separated with dots, with optional non-numbers afterwards
+var versionExp = regexp.MustCompile(`^(?:[1-9][0-9]*)(?:\.(?:[0-9]+))*`)
 
 func checkVersion(version string) bool {
 	// double check that the input looks like a snapd version
-	req := versionExp.FindStringSubmatch(version)
-	if req == nil || req[0] != version {
+	reqVersionNumMatch := versionExp.FindStringSubmatch(version)
+	if reqVersionNumMatch == nil {
+		return false
+	}
+	// this check ensures that no one can use an assumes like snapd2.48.3~pre2
+	// or snapd2.48.5+20.10, as modifiers past the version number are not meant
+	// to be relied on for snaps via assumes, however the check against the real
+	// snapd version number below allows such non-numeric modifiers since real
+	// snapds do have versions like that (for example debian pkg of snapd)
+	if reqVersionNumMatch[0] != version {
 		return false
 	}
 
-	if cmd.Version == "unknown" {
+	req := strings.Split(reqVersionNumMatch[0], ".")
+
+	if snapdtool.Version == "unknown" {
 		return true // Development tree.
 	}
 
@@ -213,16 +228,19 @@ func checkVersion(version string) bool {
 	// this code (see PR#7344). However this would change current
 	// behavior, i.e. "2.41~pre1" would *not* match [snapd2.41] anymore
 	// (which the code below does).
-	cur := versionExp.FindStringSubmatch(cmd.Version)
-	if cur == nil {
+	curVersionNumMatch := versionExp.FindStringSubmatch(snapdtool.Version)
+	if curVersionNumMatch == nil {
 		return false
 	}
+	cur := strings.Split(curVersionNumMatch[0], ".")
 
-	for i := 1; i < len(req); i++ {
-		if req[i] == "" {
-			return true
-		}
-		if cur[i] == "" {
+	for i := range req {
+		if i == len(cur) {
+			// we hit the end of the elements of the current version number and have
+			// more required version numbers left, so this doesn't match, if the
+			// previous element was higher we would have broken out already, so the
+			// only case left here is where we have version requirements that are
+			// not met
 			return false
 		}
 		reqN, err1 := strconv.Atoi(req[i])
@@ -405,7 +423,7 @@ func MockCheckSnapCallbacks(checks []CheckSnapCallback) (restore func()) {
 }
 
 func checkSnapdName(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Container, flags Flags, deviceCtx DeviceContext) error {
-	if snapInfo.GetType() != snap.TypeSnapd {
+	if snapInfo.Type() != snap.TypeSnapd {
 		// not a relevant check
 		return nil
 	}
@@ -417,7 +435,7 @@ func checkSnapdName(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Contai
 }
 
 func checkCoreName(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Container, flags Flags, deviceCtx DeviceContext) error {
-	if snapInfo.GetType() != snap.TypeOS {
+	if snapInfo.Type() != snap.TypeOS {
 		// not a relevant check
 		return nil
 	}
@@ -453,7 +471,7 @@ func checkCoreName(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Contain
 }
 
 func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, snapf snap.Container, flags Flags, deviceCtx DeviceContext) error {
-	typ := snapInfo.GetType()
+	typ := snapInfo.Type()
 	kind := ""
 	var whichName func(*asserts.Model) string
 	switch typ {
@@ -513,7 +531,7 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, snapf sn
 
 func checkBases(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Container, flags Flags, deviceCtx DeviceContext) error {
 	// check if this is relevant
-	if snapInfo.GetType() != snap.TypeApp && snapInfo.GetType() != snap.TypeGadget {
+	if snapInfo.Type() != snap.TypeApp && snapInfo.Type() != snap.TypeGadget {
 		return nil
 	}
 	if snapInfo.Base == "" {
@@ -578,6 +596,18 @@ func earlyEpochCheck(info *snap.Info, snapst *SnapState) error {
 	return checkEpochs(nil, info, cur, nil, Flags{}, nil)
 }
 
+func earlyChecks(st *state.State, snapst *SnapState, update *snap.Info, flags Flags) (Flags, error) {
+	flags, err := ensureInstallPreconditions(st, update, flags, snapst)
+	if err != nil {
+		return flags, err
+	}
+
+	if err := earlyEpochCheck(update, snapst); err != nil {
+		return flags, err
+	}
+	return flags, nil
+}
+
 // check that the listed system users are valid
 var osutilEnsureUserGroup = osutil.EnsureUserGroup
 
@@ -608,7 +638,7 @@ func checkAndCreateSystemUsernames(si *snap.Info) error {
 	}
 
 	// Run /.../snap-seccomp version-info
-	vi, err := seccomp_compiler.CompilerVersionInfo(cmd.InternalToolPath)
+	vi, err := seccomp_compiler.CompilerVersionInfo(snapdtool.InternalToolPath)
 	if err != nil {
 		return fmt.Errorf("cannot obtain seccomp compiler information: %v", err)
 	}
