@@ -22,6 +22,7 @@ package snapstate_test
 import (
 	"errors"
 	"fmt"
+	"os/user"
 
 	. "gopkg.in/check.v1"
 
@@ -117,6 +118,10 @@ var assumesTests = []struct {
 	assumes: "[snapd2.15nono]",
 	version: "unknown",
 	error:   `.* unsupported features: snapd2.15nono .*`,
+}, {
+	assumes: "[snapd2.15~pre1]",
+	version: "unknown",
+	error:   `.* unsupported features: snapd2.15~pre1 .*`,
 }, {
 	assumes: "[snapd2.15]",
 	version: "2.15",
@@ -132,12 +137,49 @@ var assumesTests = []struct {
 }, {
 	assumes: "[snapd2.15.1]",
 	version: "2.16",
+}, {
+	assumes: "[snapd2.15.1]",
+	version: "2.15.1",
+}, {
+	assumes: "[snapd2.15.1.2]",
+	version: "2.15.1.2",
+}, {
+	assumes: "[snapd2.15.1.2]",
+	version: "2.15.1.3",
+}, {
+	// the horror the horror!
+	assumes: "[snapd2.15.1.2.4.5.6.7.8.8]",
+	version: "2.15.1.2.4.5.6.7.8.8",
+}, {
+	assumes: "[snapd2.15.1.2.4.5.6.7.8.8]",
+	version: "2.15.1.2.4.5.6.7.8.9",
+}, {
+	assumes: "[snapd2.15.1.2]",
+	version: "2.15.1.3",
 }, {
 	assumes: "[snapd2.15.2]",
 	version: "2.16.1",
+}, {
+	assumes: "[snapd2.1000]",
+	version: "3.1",
 }, {
 	assumes: "[snapd3]",
 	version: "3.1",
+}, {
+	assumes: "[snapd2]",
+	version: "3.1",
+}, {
+	assumes: "[snapd3]",
+	version: "2.48",
+	error:   `.* unsupported features: snapd3 .*`,
+}, {
+	assumes: "[snapd2.15.1.2]",
+	version: "2.15.1.1",
+	error:   `.* unsupported features: snapd2\.15\.1\.2 .*`,
+}, {
+	assumes: "[snapd2.15.1.2.4.5.6.7.8.8]",
+	version: "2.15.1.2.4.5.6.7.8.1",
+	error:   `.* unsupported features: snapd2\.15\.1\.2\.4\.5\.6\.7\.8\.8 .*`,
 }, {
 	assumes: "[snapd2.16]",
 	version: "2.15",
@@ -157,7 +199,10 @@ var assumesTests = []struct {
 	version: "2.15~pre1",
 }, {
 	assumes: "[command-chain]",
-}}
+}, {
+	assumes: "[kernel-assets]",
+},
+}
 
 func (s *checkSnapSuite) TestCheckSnapAssumes(c *C) {
 	restore := snapdtool.MockVersion("2.15")
@@ -167,16 +212,19 @@ func (s *checkSnapSuite) TestCheckSnapAssumes(c *C) {
 	defer restore()
 
 	for _, test := range assumesTests {
+
 		snapdtool.Version = test.version
 		if snapdtool.Version == "" {
 			snapdtool.Version = "2.15"
 		}
+
+		comment := Commentf("snap assumes %q, but snapd version is %q", test.assumes, snapdtool.Version)
 		release.OnClassic = test.classic
 
 		yaml := fmt.Sprintf("name: foo\nversion: 1.0\nassumes: %s\n", test.assumes)
 
 		info, err := snap.InfoFromSnapYaml([]byte(yaml))
-		c.Assert(err, IsNil)
+		c.Assert(err, IsNil, comment)
 
 		var openSnapFile = func(path string, si *snap.SideInfo) (*snap.Info, snap.Container, error) {
 			return info, emptyContainer(c), nil
@@ -185,9 +233,9 @@ func (s *checkSnapSuite) TestCheckSnapAssumes(c *C) {
 		defer restore()
 		err = snapstate.CheckSnap(s.st, "snap-path", "foo", nil, nil, snapstate.Flags{}, nil)
 		if test.error != "" {
-			c.Check(err, ErrorMatches, test.error)
+			c.Check(err, ErrorMatches, test.error, comment)
 		} else {
-			c.Assert(err, IsNil)
+			c.Assert(err, IsNil, comment)
 		}
 	}
 }
@@ -1132,13 +1180,23 @@ func (s *checkSnapSuite) TestCheckSnapSystemUsernames(c *C) {
 	}
 }
 
-func (s *checkSnapSuite) TestCheckSnapSystemUsernamesCalls(c *C) {
-	// FIXME: this test fails on machines where the user was already
-	// created by the system snapd
-	_, err := osutil.FindUid("snapd-range-524288-root")
-	if err == nil {
-		c.Skip("FIXME")
-	}
+func (s *checkSnapSuite) TestCheckSnapSystemUsernamesCallsSnapDaemon(c *C) {
+	r := osutil.MockFindGid(func(groupname string) (uint64, error) {
+		if groupname == "snap_daemon" || groupname == "snapd-range-524288-root" {
+			return 0, user.UnknownGroupError(groupname)
+		}
+		return 0, fmt.Errorf("unexpected call to FindGid for %s", groupname)
+	})
+	defer r()
+
+	r = osutil.MockFindUid(func(username string) (uint64, error) {
+		if username == "snap_daemon" || username == "snapd-range-524288-root" {
+			return 0, user.UnknownUserError(username)
+		}
+		return 0, fmt.Errorf("unexpected call to FindUid for %s", username)
+	})
+	defer r()
+
 	falsePath := osutil.LookPathDefault("false", "/bin/false")
 	for _, classic := range []bool{false, true} {
 		restore := release.MockOnClassic(classic)
