@@ -20,11 +20,14 @@
 package snapstate
 
 import (
+	"fmt"
 	"time"
 
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/timings"
 )
@@ -71,13 +74,28 @@ func (r *refreshHints) refresh() error {
 	perfTimings := timings.New(map[string]string{"ensure": "refresh-hints"})
 	defer perfTimings.Save(r.state)
 
+	var updates []*snap.Info
+	var ignoreValidationByInstanceName map[string]bool
 	timings.Run(perfTimings, "refresh-candidates", "query store for refresh candidates", func(tm timings.Measurer) {
-		_, _, _, err = refreshCandidates(auth.EnsureContextTODO(), r.state, nil, nil, &store.RefreshOptions{RefreshManaged: refreshManaged})
+		updates, _, ignoreValidationByInstanceName, err = refreshCandidates(auth.EnsureContextTODO(), r.state, nil, nil, &store.RefreshOptions{RefreshManaged: refreshManaged})
 	})
 	// TODO: we currently set last-refresh-hints even when there was an
 	// error. In the future we may retry with a backoff.
 	r.state.Set("last-refresh-hints", time.Now())
-	return err
+
+	if err != nil {
+		return err
+	}
+	deviceCtx, err := DeviceCtxFromState(r.state, nil)
+	if err != nil {
+		return err
+	}
+	hints, err := refreshHintsFromCandidates(r.state, updates, ignoreValidationByInstanceName, deviceCtx)
+	if err != nil {
+		return fmt.Errorf("internal error: cannot get refresh-candidates: %v", err)
+	}
+	r.state.Set("refresh-candidates", hints)
+	return nil
 }
 
 // AtSeed configures hints refresh policies at end of seeding.
@@ -121,3 +139,53 @@ func (r *refreshHints) Ensure() error {
 	}
 	return r.refresh()
 }
+
+func refreshHintsFromCandidates(st *state.State, updates []*snap.Info, ignoreValidationByInstanceName map[string]bool, deviceCtx DeviceContext) (map[string]*refreshCandidate, error) {
+	if ValidateRefreshes != nil && len(updates) != 0 {
+		userID := 0
+		var err error
+		updates, err = ValidateRefreshes(st, updates, ignoreValidationByInstanceName, userID, deviceCtx)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	hints := make(map[string]*refreshCandidate, len(updates))
+	for _, update := range updates {
+		var snapst SnapState
+		if err := Get(st, update.InstanceName(), &snapst); err != nil {
+			return nil, err
+		}
+
+		flags := snapst.Flags
+		flags.IsAutoRefresh = true
+		flags, err := earlyChecks(st, &snapst, update, flags)
+		if err != nil {
+			logger.Debugf("update hint for %q is not applicable: %v", update.InstanceName(), err)
+			continue
+		}
+
+		snapsup := &refreshCandidate{
+			SnapSetup: SnapSetup{
+				Base:      update.Base,
+				Prereq:    defaultContentPlugProviders(st, update),
+				Channel:   snapst.TrackingChannel,
+				CohortKey: snapst.CohortKey,
+				// UserID not set
+				Flags:        flags.ForSnapSetup(),
+				DownloadInfo: &update.DownloadInfo,
+				SideInfo:     &update.SideInfo,
+				Type:         update.Type(),
+				PlugsOnly:    len(update.Slots) == 0,
+				InstanceKey:  update.InstanceKey,
+				auxStoreInfo: auxStoreInfo{
+					Website: update.Website,
+					Media:   update.Media,
+				},
+			},
+			Version: update.Version,
+		}
+		hints[update.InstanceName()] = snapsup
+	}
+	return hints, nil
+}
