@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2019 Canonical Ltd
+ * Copyright (C) 2014-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -63,6 +63,10 @@ type seed20 struct {
 
 	auxInfos map[string]*internal.AuxInfo20
 
+	metaFilesLoaded bool
+
+	essCache map[string]*Snap
+
 	snaps []*Snap
 	// modes holds a matching applicable modes set for each snap in snaps
 	modes             [][]string
@@ -73,7 +77,7 @@ func (s *seed20) LoadAssertions(db asserts.RODatabase, commitTo func(*asserts.Ba
 	if db == nil {
 		// a db was not provided, create an internal temporary one
 		var err error
-		db, commitTo, err = newMemAssertionsDB()
+		db, commitTo, err = newMemAssertionsDB(nil)
 		if err != nil {
 			return err
 		}
@@ -177,11 +181,11 @@ func (s *seed20) LoadAssertions(db asserts.RODatabase, commitTo func(*asserts.Ba
 	return nil
 }
 
-func (s *seed20) Model() (*asserts.Model, error) {
+func (s *seed20) Model() *asserts.Model {
 	if s.model == nil {
-		return nil, fmt.Errorf("internal error: model assertion unset")
+		panic("internal error: model assertion unset (LoadAssertions not called)")
 	}
-	return s.model, nil
+	return s.model
 }
 
 func (s *seed20) Brand() (*asserts.Account, error) {
@@ -213,7 +217,7 @@ func (s *seed20) loadOptions() error {
 func (s *seed20) nextOptSnap(modSnap *asserts.ModelSnap) (optSnap *internal.Snap20, done bool) {
 	// we can merge model snaps and options snaps because
 	// both seed20.go and writer.go follow the order:
-	// system snap, model.AllSnaps()...
+	// system snap, model.EssentialSnaps(), model.SnapsWithoutEssential()
 	if s.optSnapsIdx == len(s.optSnaps) {
 		return nil, true
 	}
@@ -305,7 +309,13 @@ func (s *seed20) lookupVerifiedRevision(snapRef naming.SnapRef, snapsDir string)
 	return snapPath, snapRev, snapDecl, nil
 }
 
-func (s *seed20) addSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, modes []string, channel string, snapsDir string, tm timings.Measurer) (*Snap, error) {
+func (s *seed20) lookupSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, channel string, snapsDir string, cache map[string]*Snap, tm timings.Measurer) (*Snap, error) {
+	snapKey := fmt.Sprintf("%s:%s", snapRef.ID(), snapRef.SnapName())
+	seedSnap := cache[snapKey]
+	if seedSnap != nil {
+		return seedSnap, nil
+	}
+
 	if optSnap != nil && optSnap.Channel != "" {
 		channel = optSnap.Channel
 	}
@@ -343,7 +353,7 @@ func (s *seed20) addSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, modes
 		sideInfo.Contact = auxInfo.Contact
 	}
 
-	seedSnap := &Snap{
+	seedSnap = &Snap{
 		Path: path,
 
 		SideInfo: sideInfo,
@@ -351,6 +361,19 @@ func (s *seed20) addSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, modes
 		Channel: channel,
 	}
 
+	if cache != nil {
+		cache[snapKey] = seedSnap
+	}
+
+	return seedSnap, nil
+}
+
+func (s *seed20) addSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, modes []string, channel string, snapsDir string, cache map[string]*Snap, tm timings.Measurer) (*Snap, error) {
+	seedSnap, err := s.lookupSnap(snapRef, optSnap, channel, snapsDir, cache, tm)
+	if err != nil {
+		return nil, err
+	}
+
 	s.snaps = append(s.snaps, seedSnap)
 	s.modes = append(s.modes, modes)
 
@@ -359,12 +382,12 @@ func (s *seed20) addSnap(snapRef naming.SnapRef, optSnap *internal.Snap20, modes
 
 var errFiltered = errors.New("filtered out")
 
-func (s *seed20) addModelSnap(modelSnap *asserts.ModelSnap, essential bool, filter func(*asserts.ModelSnap) bool, tm timings.Measurer) (*Snap, error) {
+func (s *seed20) addModelSnap(modelSnap *asserts.ModelSnap, essential bool, filter func(*asserts.ModelSnap) bool, cache map[string]*Snap, tm timings.Measurer) (*Snap, error) {
 	optSnap, _ := s.nextOptSnap(modelSnap)
-	if !filter(modelSnap) {
+	if filter != nil && !filter(modelSnap) {
 		return nil, errFiltered
 	}
-	seedSnap, err := s.addSnap(modelSnap, optSnap, modelSnap.Modes, modelSnap.DefaultChannel, "../../snaps", tm)
+	seedSnap, err := s.addSnap(modelSnap, optSnap, modelSnap.Modes, modelSnap.DefaultChannel, "../../snaps", cache, tm)
 	if err != nil {
 		return nil, err
 	}
@@ -380,7 +403,10 @@ func (s *seed20) addModelSnap(modelSnap *asserts.ModelSnap, essential bool, filt
 }
 
 func (s *seed20) LoadMeta(tm timings.Measurer) error {
-	if err := s.loadModelMeta(nil, tm); err != nil {
+	if err := s.loadEssentialMeta(nil, tm); err != nil {
+		return err
+	}
+	if err := s.loadModelRestMeta(tm); err != nil {
 		return err
 	}
 
@@ -392,7 +418,7 @@ func (s *seed20) LoadMeta(tm timings.Measurer) error {
 			break
 		}
 
-		_, err := s.addSnap(optSnap, optSnap, runMode, "latest/stable", "snaps", tm)
+		_, err := s.addSnap(optSnap, optSnap, runMode, "latest/stable", "snaps", nil, tm)
 		if err != nil {
 			return err
 		}
@@ -404,7 +430,7 @@ func (s *seed20) LoadMeta(tm timings.Measurer) error {
 func (s *seed20) LoadEssentialMeta(essentialTypes []snap.Type, tm timings.Measurer) error {
 	filterEssential := essentialSnapTypesToModelFilter(essentialTypes)
 
-	if err := s.loadModelMeta(filterEssential, tm); err != nil {
+	if err := s.loadEssentialMeta(filterEssential, tm); err != nil {
 		return err
 	}
 
@@ -416,10 +442,9 @@ func (s *seed20) LoadEssentialMeta(essentialTypes []snap.Type, tm timings.Measur
 	return nil
 }
 
-func (s *seed20) loadModelMeta(filterEssential func(*asserts.ModelSnap) bool, tm timings.Measurer) error {
-	model, err := s.Model()
-	if err != nil {
-		return err
+func (s *seed20) loadMetaFiles() error {
+	if s.metaFilesLoaded {
+		return nil
 	}
 
 	if err := s.loadOptions(); err != nil {
@@ -430,35 +455,49 @@ func (s *seed20) loadModelMeta(filterEssential func(*asserts.ModelSnap) bool, tm
 		return err
 	}
 
-	essentialOnly := filterEssential != nil
-	filter := filterEssential
-	if !essentialOnly {
-		// no filtering
-		filter = func(*asserts.ModelSnap) bool {
-			return true
-		}
+	s.metaFilesLoaded = true
+	return nil
+}
+
+func (s *seed20) resetSnaps() {
+	// setup essential snaps cache
+	if s.essCache == nil {
+		// 4 = snapd+base+kernel+gadget
+		s.essCache = make(map[string]*Snap, 4)
 	}
 
-	allSnaps := model.AllSnaps()
+	s.optSnapsIdx = 0
+	s.snaps = nil
+	s.modes = nil
+	s.essentialSnapsNum = 0
+}
+
+func (s *seed20) loadEssentialMeta(filterEssential func(*asserts.ModelSnap) bool, tm timings.Measurer) error {
+	model := s.Model()
+
+	if err := s.loadMetaFiles(); err != nil {
+		return err
+	}
+
+	s.resetSnaps()
+
+	essSnaps := model.EssentialSnaps()
+	const essential = true
+
 	// an explicit snapd is the first of all of snaps
-	if allSnaps[0].SnapType != "snapd" {
+	if essSnaps[0].SnapType != "snapd" {
 		snapdSnap := internal.MakeSystemSnap("snapd", "latest/stable", []string{"run", "ephemeral"})
-		if _, err := s.addModelSnap(snapdSnap, true, filter, tm); err != nil && err != errFiltered {
+		if _, err := s.addModelSnap(snapdSnap, essential, filterEssential, s.essCache, tm); err != nil && err != errFiltered {
 			return err
 		}
 	}
 
-	essential := true
-	for _, modelSnap := range allSnaps {
-		seedSnap, err := s.addModelSnap(modelSnap, essential, filter, tm)
+	for _, modelSnap := range essSnaps {
+		seedSnap, err := s.addModelSnap(modelSnap, essential, filterEssential, s.essCache, tm)
 		if err != nil {
 			if err == errFiltered {
 				continue
 			}
-			if _, ok := err.(*noSnapDeclarationError); ok && modelSnap.Presence == "optional" {
-				// skipped optional snap is ok
-				continue
-			}
 			return err
 		}
 		if modelSnap.SnapType == "gadget" {
@@ -472,13 +511,24 @@ func (s *seed20) loadModelMeta(filterEssential func(*asserts.ModelSnap) bool, tm
 			}
 			// TODO: when we allow extend models for classic
 			// we need to add the gadget base here
+		}
+	}
 
-			// done with essential snaps, gadget is the last one
-			essential = false
-			if essentialOnly {
-				// will not find more
-				break
+	return nil
+}
+
+func (s *seed20) loadModelRestMeta(tm timings.Measurer) error {
+	model := s.Model()
+
+	const notEssential = false
+	for _, modelSnap := range model.SnapsWithoutEssential() {
+		_, err := s.addModelSnap(modelSnap, notEssential, nil, nil, tm)
+		if err != nil {
+			if _, ok := err.(*noSnapDeclarationError); ok && modelSnap.Presence == "optional" {
+				// skipped optional snap is ok
+				continue
 			}
+			return err
 		}
 	}
 
