@@ -59,9 +59,10 @@ type CurrentSnap struct {
 }
 
 type AssertionQuery interface {
-	ToResolve() (map[asserts.Grouping][]*asserts.AtRevision, error)
+	ToResolve() (map[asserts.Grouping][]*asserts.AtRevision, map[asserts.Grouping][]*asserts.AtSequence, error)
 
 	AddError(e error, ref *asserts.Ref) error
+	AddSequenceError(e error, atSeq *asserts.AtSequence) error
 	AddGroupingError(e error, grouping asserts.Grouping) error
 }
 
@@ -122,8 +123,8 @@ type snapActionJSON struct {
 	// nil epoch is not an empty interface{}, you'll get the null in the json.
 	Epoch interface{} `json:"epoch,omitempty"`
 	// For assertions
-	Key        string         `json:"key,omitempty"`
-	Assertions []assertAtJSON `json:"assertions,omitempty"`
+	Key        string        `json:"key,omitempty"`
+	Assertions []interface{} `json:"assertions,omitempty"`
 }
 
 type assertAtJSON struct {
@@ -132,6 +133,16 @@ type assertAtJSON struct {
 	IfNewerThan *int     `json:"if-newer-than,omitempty"`
 }
 
+type assertSeqAtJSON struct {
+	Type        string   `json:"type"`
+	SequenceKey []string `json:"sequence-key"`
+	Sequence    int      `json:"sequence,omitempty"`
+	// if-sequence-equal-or-newer-than and sequence are mutually exclusive
+	IfSequenceEqualOrNewerThan *int `json:"if-sequence-equal-or-newer-than,omitempty"`
+	IfSequenceNewerThan        *int `json:"if-sequence-newer-than,omitempty"`
+	IfNewerThan                *int `json:"if-newer-than,omitempty"`
+}
+
 type snapRelease struct {
 	Architecture string `json:"architecture"`
 	Channel      string `json:"channel"`
@@ -141,8 +152,10 @@ type errorListEntry struct {
 	Code    string `json:"code"`
 	Message string `json:"message"`
 	// for assertions
-	Type       string   `json:"type"`
-	PrimaryKey []string `json:"primary-key"`
+	Type string `json:"type"`
+	// either primary-key or sequence-key is expected (but not both)
+	PrimaryKey  []string `json:"primary-key,omitempty"`
+	SequenceKey []string `json:"sequence-key,omitempty"`
 }
 
 type snapActionResult struct {
@@ -196,22 +209,23 @@ func (s *Store) SnapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 	}
 
 	var toResolve map[asserts.Grouping][]*asserts.AtRevision
+	var toResolveSeq map[asserts.Grouping][]*asserts.AtSequence
 	if assertQuery != nil {
 		var err error
-		toResolve, err = assertQuery.ToResolve()
+		toResolve, toResolveSeq, err = assertQuery.ToResolve()
 		if err != nil {
 			return nil, nil, err
 		}
 	}
 
-	if len(currentSnaps) == 0 && len(actions) == 0 && len(toResolve) == 0 {
+	if len(currentSnaps) == 0 && len(actions) == 0 && len(toResolve) == 0 && len(toResolveSeq) == 0 {
 		// nothing to do
 		return nil, nil, &SnapActionError{NoResults: true}
 	}
 
 	authRefreshes := 0
 	for {
-		sars, ars, err := s.snapAction(ctx, currentSnaps, actions, assertQuery, toResolve, user, opts)
+		sars, ars, err := s.snapAction(ctx, currentSnaps, actions, assertQuery, toResolve, toResolveSeq, user, opts)
 
 		if saErr, ok := err.(*SnapActionError); ok && authRefreshes < 2 && len(saErr.Other) > 0 {
 			// do we need to try to refresh auths?, 2 tries
@@ -278,7 +292,7 @@ type AssertionResult struct {
 	StreamURLs []string
 }
 
-func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, actions []*SnapAction, assertQuery AssertionQuery, toResolve map[asserts.Grouping][]*asserts.AtRevision, user *auth.UserState, opts *RefreshOptions) ([]SnapActionResult, []AssertionResult, error) {
+func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, actions []*SnapAction, assertQuery AssertionQuery, toResolve map[asserts.Grouping][]*asserts.AtRevision, toResolveSeq map[asserts.Grouping][]*asserts.AtSequence, user *auth.UserState, opts *RefreshOptions) ([]SnapActionResult, []AssertionResult, error) {
 	requestSalt := ""
 	if opts != nil {
 		requestSalt = opts.PrivacyKey
@@ -317,7 +331,11 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 		}
 	}
 
+	// do not include toResolveSeq len in the initial size since it may have
+	// group keys overlapping with toResolve; the loop over toResolveSeq simply
+	// appends to actionJSONs.
 	actionJSONs := make([]*snapActionJSON, len(actions)+len(toResolve))
+	actionIndex := 0
 
 	// snaps
 	downloadNum := 0
@@ -325,7 +343,7 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 	installs := make(map[string]*SnapAction, len(actions))
 	downloads := make(map[string]*SnapAction, len(actions))
 	refreshes := make(map[string]*SnapAction, len(actions))
-	for i, a := range actions {
+	for _, a := range actions {
 		if !isValidAction(a.Action) {
 			return nil, nil, fmt.Errorf("internal error: unsupported action %q", a.Action)
 		}
@@ -385,30 +403,88 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 
 		aJSON.InstanceKey = instanceKey
 
-		actionJSONs[i] = aJSON
+		actionJSONs[actionIndex] = aJSON
+		actionIndex++
 	}
 
+	groupingsAssertions := make(map[string]*snapActionJSON)
+
 	// assertions
 	var assertMaxFormats map[string]int
 	if len(toResolve) > 0 {
-		i := len(actionJSONs) - len(toResolve)
 		for grp, ats := range toResolve {
 			aJSON := &snapActionJSON{
 				Action: "fetch-assertions",
 				Key:    string(grp),
 			}
-			aJSON.Assertions = make([]assertAtJSON, len(ats))
+			aJSON.Assertions = make([]interface{}, len(ats))
+			groupingsAssertions[aJSON.Key] = aJSON
+
 			for j, at := range ats {
-				aJSON.Assertions[j].Type = at.Type.Name
-				aJSON.Assertions[j].PrimaryKey = at.PrimaryKey
+				aj := &assertAtJSON{
+					Type:       at.Type.Name,
+					PrimaryKey: at.PrimaryKey,
+				}
 				rev := at.Revision
 				if rev != asserts.RevisionNotKnown {
-					aJSON.Assertions[j].IfNewerThan = &rev
+					aj.IfNewerThan = &rev
 				}
+				aJSON.Assertions[j] = aj
 			}
-			actionJSONs[i] = aJSON
-			i++
+			actionJSONs[actionIndex] = aJSON
+			actionIndex++
 		}
+	}
+
+	if len(toResolveSeq) > 0 {
+		for grp, ats := range toResolveSeq {
+			key := string(grp)
+			// append to existing grouping if applicable
+			aJSON := groupingsAssertions[key]
+			existingGroup := aJSON != nil
+			if !existingGroup {
+				aJSON = &snapActionJSON{
+					Action: "fetch-assertions",
+					Key:    key,
+				}
+				aJSON.Assertions = make([]interface{}, 0, len(ats))
+				actionJSONs = append(actionJSONs, aJSON)
+			}
+			for _, at := range ats {
+				aj := assertSeqAtJSON{
+					Type:        at.Type.Name,
+					SequenceKey: at.SequenceKey,
+				}
+				// for pinned we request the assertion ​by the sequence point <sequence-number>​, i.e.
+				// {"type": "validation-set",
+				//  "sequence-key": ["16", "account-id", "name"],
+				//  "sequence": <sequence-number>}
+				if at.Pinned {
+					if at.Sequence <= 0 {
+						return nil, nil, fmt.Errorf("internal error: sequence not set for pinned sequence %s, %v", at.Type.Name, at.SequenceKey)
+					}
+					aj.Sequence = at.Sequence
+				} else {
+					// for not pinned, if sequence is specified, then
+					// use it for "if-sequence-equal-or-newer-than": <sequence-number>
+					if at.Sequence > 0 {
+						aj.IfSequenceEqualOrNewerThan = &at.Sequence
+					} // else - get the latest
+				}
+				rev := at.Revision
+				// revision (if set) goes to "if-newer-than": <assert-revision>
+				if rev != asserts.RevisionNotKnown {
+					if at.Sequence <= 0 {
+						return nil, nil, fmt.Errorf("internal error: sequence not set while revision is known for %s, %v", at.Type.Name, at.SequenceKey)
+					}
+					aj.IfNewerThan = &rev
+				}
+				aJSON.Assertions = append(aJSON.Assertions, aj)
+			}
+		}
+	}
+
+	if len(toResolve) > 0 || len(toResolveSeq) > 0 {
 		assertMaxFormats = asserts.MaxSupportedFormats(1)
 	}
 
@@ -593,8 +669,12 @@ func reportFetchAssertionsError(res *snapActionResult, assertq AssertionQuery) e
 		aType := asserts.Type(ent.Type)
 		return aType != nil && len(ent.PrimaryKey) == len(aType.PrimaryKey)
 	}
+	carryingSeqKey := func(ent *errorListEntry) bool {
+		aType := asserts.Type(ent.Type)
+		return aType != nil && aType.SequenceForming() && len(ent.SequenceKey) == len(aType.PrimaryKey)-1
+	}
 	prio := func(ent *errorListEntry) int {
-		if !carryingRef(ent) {
+		if !carryingRef(ent) && !carryingSeqKey(ent) {
 			return 2
 		}
 		if ent.Code != "not-found" {
@@ -614,20 +694,34 @@ func reportFetchAssertionsError(res *snapActionResult, assertq AssertionQuery) e
 		}
 	}
 	rep := errl[errIdx]
-	if carryingRef(&rep) {
+	notFound := rep.Code == "not-found"
+	switch {
+	case carryingRef(&rep):
 		ref := &asserts.Ref{Type: asserts.Type(rep.Type), PrimaryKey: rep.PrimaryKey}
 		var err error
-		if rep.Code == "not-found" {
+		if notFound {
 			headers, _ := asserts.HeadersFromPrimaryKey(ref.Type, ref.PrimaryKey)
 			err = &asserts.NotFoundError{
 				Type:    ref.Type,
 				Headers: headers,
 			}
-
 		} else {
 			err = fmt.Errorf("%s", rep.Message)
 		}
 		return assertq.AddError(err, ref)
+	case carryingSeqKey(&rep):
+		var err error
+		atSeq := &asserts.AtSequence{Type: asserts.Type(rep.Type), SequenceKey: rep.SequenceKey}
+		if notFound {
+			headers, _ := asserts.HeadersFromSequenceKey(atSeq.Type, atSeq.SequenceKey)
+			err = &asserts.NotFoundError{
+				Type:    atSeq.Type,
+				Headers: headers,
+			}
+		} else {
+			err = fmt.Errorf("%s", rep.Message)
+		}
+		return assertq.AddSequenceError(err, atSeq)
 	}
 
 	return assertq.AddGroupingError(fmt.Errorf("%s", rep.Message), asserts.Grouping(res.Key))
