@@ -85,6 +85,10 @@ func (Store) Assertion(*asserts.AssertionType, []string, *auth.UserState) (asser
 	panic("Store.Assertion not expected")
 }
 
+func (Store) SeqFormingAssertion(*asserts.AssertionType, []string, int, *auth.UserState) (asserts.Assertion, error) {
+	panic("Store.SeqFormingAssertion not expected")
+}
+
 func (Store) DownloadAssertions([]string, *asserts.Batch, *auth.UserState) error {
 	panic("Store.DownloadAssertions not expected")
 }
