@@ -30,6 +30,7 @@ import (
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/squashfs"
 )
@@ -90,6 +91,14 @@ func (s *emulation) Status(units ...string) ([]*UnitStatus, error) {
 	return nil, errNotImplemented
 }
 
+func (s *emulation) InactiveEnterTimestamp(unit string) (time.Time, error) {
+	return time.Time{}, errNotImplemented
+}
+
+func (s *emulation) CurrentMemoryUsage(unit string) (quantity.Size, error) {
+	return 0, errNotImplemented
+}
+
 func (s *emulation) IsEnabled(service string) (bool, error) {
 	return false, errNotImplemented
 }
@@ -176,3 +185,11 @@ func (s *emulation) Unmask(service string) error {
 	_, err := systemctlCmd("--root", s.rootDir, "unmask", service)
 	return err
 }
+
+func (s *emulation) Mount(what, where string, options ...string) error {
+	return errNotImplemented
+}
+
+func (s *emulation) Umount(whatOrWhere string) error {
+	return errNotImplemented
+}
