@@ -21,6 +21,7 @@ package systemd_test
 
 import (
 	"bytes"
+	"encoding/json"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -33,6 +34,7 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/squashfs"
 	"github.com/snapcore/snapd/sandbox/selinux"
@@ -145,25 +147,25 @@ func (s *SystemdTestSuite) myJctl(svcs []string, n int, follow bool) (io.ReadClo
 }
 
 func (s *SystemdTestSuite) TestDaemonReload(c *C) {
-	err := New("", SystemMode, s.rep).DaemonReload()
+	err := New(SystemMode, s.rep).DaemonReload()
 	c.Assert(err, IsNil)
 	c.Assert(s.argses, DeepEquals, [][]string{{"daemon-reload"}})
 }
 
 func (s *SystemdTestSuite) TestDaemonReexec(c *C) {
-	err := New("", SystemMode, s.rep).DaemonReexec()
+	err := New(SystemMode, s.rep).DaemonReexec()
 	c.Assert(err, IsNil)
 	c.Assert(s.argses, DeepEquals, [][]string{{"daemon-reexec"}})
 }
 
 func (s *SystemdTestSuite) TestStart(c *C) {
-	err := New("", SystemMode, s.rep).Start("foo")
+	err := New(SystemMode, s.rep).Start("foo")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"start", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestStartMany(c *C) {
-	err := New("", SystemMode, s.rep).Start("foo", "bar", "baz")
+	err := New(SystemMode, s.rep).Start("foo", "bar", "baz")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"start", "foo", "bar", "baz"}})
 }
@@ -178,7 +180,7 @@ func (s *SystemdTestSuite) TestStop(c *C) {
 		[]byte("ActiveState=inactive\n"),
 	}
 	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New("", SystemMode, s.rep).Stop("foo", 1*time.Second)
+	err := New(SystemMode, s.rep).Stop("foo", 1*time.Second)
 	c.Assert(err, IsNil)
 	c.Assert(s.argses, HasLen, 4)
 	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo"})
@@ -204,6 +206,11 @@ Type=potato
 Id=baz.service
 ActiveState=inactive
 UnitFileState=disabled
+
+Type=
+Id=missing.service
+ActiveState=inactive
+UnitFileState=
 `[1:]),
 		[]byte(`
 Id=some.timer
@@ -216,37 +223,48 @@ UnitFileState=disabled
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service", "bar.service", "baz.service", "some.timer", "other.socket")
+	out, err := New(SystemMode, s.rep).Status("foo.service", "bar.service", "baz.service", "missing.service", "some.timer", "other.socket")
 	c.Assert(err, IsNil)
 	c.Check(out, DeepEquals, []*UnitStatus{
 		{
-			Daemon:   "simple",
-			UnitName: "foo.service",
-			Active:   true,
-			Enabled:  true,
+			Daemon:    "simple",
+			UnitName:  "foo.service",
+			Active:    true,
+			Enabled:   true,
+			Installed: true,
 		}, {
-			Daemon:   "simple",
-			UnitName: "bar.service",
-			Active:   true,
-			Enabled:  true,
+			Daemon:    "simple",
+			UnitName:  "bar.service",
+			Active:    true,
+			Enabled:   true,
+			Installed: true,
 		}, {
-			Daemon:   "potato",
-			UnitName: "baz.service",
-			Active:   false,
-			Enabled:  false,
+			Daemon:    "potato",
+			UnitName:  "baz.service",
+			Active:    false,
+			Enabled:   false,
+			Installed: true,
 		}, {
-			UnitName: "some.timer",
-			Active:   true,
-			Enabled:  true,
+			Daemon:    "",
+			UnitName:  "missing.service",
+			Active:    false,
+			Enabled:   false,
+			Installed: false,
 		}, {
-			UnitName: "other.socket",
-			Active:   true,
-			Enabled:  false,
+			UnitName:  "some.timer",
+			Active:    true,
+			Enabled:   true,
+			Installed: true,
+		}, {
+			UnitName:  "other.socket",
+			Active:    true,
+			Enabled:   false,
+			Installed: true,
 		},
 	})
 	c.Check(s.rep.msgs, IsNil)
 	c.Assert(s.argses, DeepEquals, [][]string{
-		{"show", "--property=Id,ActiveState,UnitFileState,Type", "foo.service", "bar.service", "baz.service"},
+		{"show", "--property=Id,ActiveState,UnitFileState,Type", "foo.service", "bar.service", "baz.service", "missing.service"},
 		{"show", "--property=Id,ActiveState,UnitFileState", "some.timer", "other.socket"},
 	})
 }
@@ -266,7 +284,7 @@ UnitFileState=enabled
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Check(err, ErrorMatches, "cannot get unit status: expected 1 results, got 2")
 	c.Check(out, IsNil)
 	c.Check(s.rep.msgs, IsNil)
@@ -283,7 +301,7 @@ Potatoes
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* bad line "Potatoes" .*`)
 	c.Check(out, IsNil)
 }
@@ -298,7 +316,7 @@ UnitFileState=enabled
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* queried status of "foo.service" but got status of "bar.service"`)
 	c.Check(out, IsNil)
 }
@@ -314,7 +332,7 @@ Potatoes=false
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* unexpected field "Potatoes" .*`)
 	c.Check(out, IsNil)
 }
@@ -327,7 +345,7 @@ ActiveState=active
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* missing UnitFileState, Type .*`)
 	c.Check(out, IsNil)
 }
@@ -340,7 +358,7 @@ ActiveState=active
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.timer")
+	out, err := New(SystemMode, s.rep).Status("foo.timer")
 	c.Assert(err, ErrorMatches, `.* missing UnitFileState .*`)
 	c.Check(out, IsNil)
 }
@@ -356,7 +374,7 @@ UnitFileState=enabled
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* duplicate field "ActiveState" .*`)
 	c.Check(out, IsNil)
 }
@@ -371,7 +389,7 @@ UnitFileState=enabled
 `[1:]),
 	}
 	s.errors = []error{nil}
-	out, err := New("", SystemMode, s.rep).Status("foo.service")
+	out, err := New(SystemMode, s.rep).Status("foo.service")
 	c.Assert(err, ErrorMatches, `.* empty field "Id" .*`)
 	c.Check(out, IsNil)
 }
@@ -379,14 +397,20 @@ UnitFileState=enabled
 func (s *SystemdTestSuite) TestStopTimeout(c *C) {
 	restore := MockStopDelays(time.Millisecond, 25*time.Second)
 	defer restore()
-	err := New("", SystemMode, s.rep).Stop("foo", 10*time.Millisecond)
+	err := New(SystemMode, s.rep).Stop("foo", 10*time.Millisecond)
 	c.Assert(err, FitsTypeOf, &Timeout{})
 	c.Assert(len(s.rep.msgs) > 0, Equals, true)
 	c.Check(s.rep.msgs[0], Equals, "Waiting for foo to stop.")
 }
 
 func (s *SystemdTestSuite) TestDisable(c *C) {
-	err := New("xyzzy", SystemMode, s.rep).Disable("foo")
+	err := New(SystemMode, s.rep).Disable("foo")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{{"disable", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestUnderRootDisable(c *C) {
+	err := NewUnderRoot("xyzzy", SystemMode, s.rep).Disable("foo")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "disable", "foo"}})
 }
@@ -434,19 +458,37 @@ func (s *SystemdTestSuite) TestVersion(c *C) {
 }
 
 func (s *SystemdTestSuite) TestEnable(c *C) {
-	err := New("xyzzy", SystemMode, s.rep).Enable("foo")
+	err := New(SystemMode, s.rep).Enable("foo")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{{"enable", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestEnableUnderRoot(c *C) {
+	err := NewUnderRoot("xyzzy", SystemMode, s.rep).Enable("foo")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "enable", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestMask(c *C) {
-	err := New("xyzzy", SystemMode, s.rep).Mask("foo")
+	err := New(SystemMode, s.rep).Mask("foo")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{{"mask", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestMaskUnderRoot(c *C) {
+	err := NewUnderRoot("xyzzy", SystemMode, s.rep).Mask("foo")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "mask", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestUnmask(c *C) {
-	err := New("xyzzy", SystemMode, s.rep).Unmask("foo")
+	err := New(SystemMode, s.rep).Unmask("foo")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{{"unmask", "foo"}})
+}
+
+func (s *SystemdTestSuite) TestUnmaskUnderRoot(c *C) {
+	err := NewUnderRoot("xyzzy", SystemMode, s.rep).Unmask("foo")
 	c.Assert(err, IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "unmask", "foo"}})
 }
@@ -460,7 +502,7 @@ func (s *SystemdTestSuite) TestRestart(c *C) {
 		nil, // for the "start"
 	}
 	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New("", SystemMode, s.rep).Restart("foo", 100*time.Millisecond)
+	err := New(SystemMode, s.rep).Restart("foo", 100*time.Millisecond)
 	c.Assert(err, IsNil)
 	c.Check(s.argses, HasLen, 3)
 	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo"})
@@ -469,7 +511,7 @@ func (s *SystemdTestSuite) TestRestart(c *C) {
 }
 
 func (s *SystemdTestSuite) TestKill(c *C) {
-	c.Assert(New("", SystemMode, s.rep).Kill("foo", "HUP", ""), IsNil)
+	c.Assert(New(SystemMode, s.rep).Kill("foo", "HUP", ""), IsNil)
 	c.Check(s.argses, DeepEquals, [][]string{{"kill", "foo", "-s", "HUP", "--kill-who=all"}})
 }
 
@@ -481,7 +523,7 @@ func (s *SystemdTestSuite) TestIsTimeout(c *C) {
 func (s *SystemdTestSuite) TestLogErrJctl(c *C) {
 	s.jerrs = []error{&Timeout{}}
 
-	reader, err := New("", SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
+	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
 	c.Check(err, NotNil)
 	c.Check(reader, IsNil)
 	c.Check(s.jns, DeepEquals, []string{"24"})
@@ -496,7 +538,7 @@ func (s *SystemdTestSuite) TestLogs(c *C) {
 `
 	s.jouts = [][]byte{[]byte(expected)}
 
-	reader, err := New("", SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
+	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
 	c.Check(err, IsNil)
 	logs, err := ioutil.ReadAll(reader)
 	c.Assert(err, IsNil)
@@ -507,29 +549,187 @@ func (s *SystemdTestSuite) TestLogs(c *C) {
 	c.Check(s.j, Equals, 1)
 }
 
-func (s *SystemdTestSuite) TestLogPID(c *C) {
+// mustJSONMarshal panic's if the value cannot be marshaled
+func mustJSONMarshal(v interface{}) *json.RawMessage {
+	b, err := json.Marshal(v)
+	if err != nil {
+		panic(fmt.Sprintf("couldn't marshal json in test fixture: %v", err))
+	}
+	msg := json.RawMessage(b)
+	return &msg
+}
+
+func (s *SystemdTestSuite) TestLogPIDWithNonTrivialKeyValues(c *C) {
+	l1 := Log{
+		"_PID": mustJSONMarshal([]string{}),
+	}
+	l2 := Log{
+		"_PID": mustJSONMarshal(6),
+	}
+	l3 := Log{
+		"_PID": mustJSONMarshal([]string{"pid1", "pid2", "pid3"}),
+	}
+	l4 := Log{
+		"SYSLOG_PID": mustJSONMarshal([]string{"pid1", "pid2", "pid3"}),
+	}
+	l5 := Log{
+		"_PID":       mustJSONMarshal("42"),
+		"SYSLOG_PID": mustJSONMarshal([]string{"pid1", "pid2", "pid3"}),
+	}
+	l6 := Log{
+		"_PID":       mustJSONMarshal([]string{"42"}),
+		"SYSLOG_PID": mustJSONMarshal([]string{"pid1", "pid2", "pid3"}),
+	}
+	l7 := Log{
+		"_PID":       mustJSONMarshal([]string{"42", "42"}),
+		"SYSLOG_PID": mustJSONMarshal([]string{"singlepid"}),
+	}
 	c.Check(Log{}.PID(), Equals, "-")
-	c.Check(Log{"_PID": "99"}.PID(), Equals, "99")
-	c.Check(Log{"SYSLOG_PID": "99"}.PID(), Equals, "99")
+	c.Check(l1.PID(), Equals, "-")
+	c.Check(l2.PID(), Equals, "-")
+	c.Check(l3.PID(), Equals, "-")
+	c.Check(l4.PID(), Equals, "-")
 	// things starting with underscore are "trusted", so we trust
 	// them more than the user-settable ones:
-	c.Check(Log{"_PID": "42", "SYSLOG_PID": "99"}.PID(), Equals, "42")
+	c.Check(l5.PID(), Equals, "42")
+	c.Check(l6.PID(), Equals, "42")
+	c.Check(l7.PID(), Equals, "singlepid")
+}
+
+func (s *SystemdTestSuite) TestLogsMessageWithNonUniqueKeys(c *C) {
+
+	tt := []struct {
+		msg     *json.RawMessage
+		exp     string
+		comment string
+	}{
+		{
+			mustJSONMarshal("m1"),
+			"m1",
+			"simple string",
+		},
+		{
+			mustJSONMarshal("Я"),
+			"Я",
+			"simple utf-8 string",
+		},
+		{
+			mustJSONMarshal([]rune{65, 66, 67, 192, 69}),
+			"ABC\xc0E",
+			"invalid utf-8 bytes",
+		},
+		{
+			mustJSONMarshal(""),
+			"",
+			"empty string",
+		},
+		{
+			mustJSONMarshal([]string{"m1", "m2", "m3"}),
+			"m1\nm2\nm3",
+			"slice of strings",
+		},
+		{
+			// this is just "hello" in ascii
+			mustJSONMarshal([]rune{104, 101, 108, 108, 111}),
+			"hello",
+			"rune arrays are converted to strings",
+		},
+		{
+			// this is "hello\r" in ascii, the \r char is unprintable
+			mustJSONMarshal([]rune{104, 101, 108, 108, 111, 13}),
+			"hello\r",
+			"rune arrays are converted to strings",
+		},
+		{
+			// this is "hel" and "lo" in ascii
+			mustJSONMarshal([][]rune{
+				{104, 101, 108},
+				{108, 111},
+			}),
+			"hel\nlo",
+			"arrays of rune arrays are converted to arrays of strings",
+		},
+		{
+			// this is invalid utf-8 string followed by a valid one
+			mustJSONMarshal([][]byte{
+				{65, 66, 67, 192, 69},
+				{104, 101, 108, 108, 111},
+			}),
+			"ABC\xc0E\nhello",
+			"arrays of bytes, some are invalid utf-8 strings",
+		},
+		{
+			mustJSONMarshal(5),
+			"- (error decoding original message: unsupported JSON encoding format)",
+			"invalid message format of raw scalar number",
+		},
+		{
+			mustJSONMarshal(map[string]int{"hello": 1}),
+			"- (error decoding original message: unsupported JSON encoding format)",
+			"invalid message format of map object",
+		},
+	}
+
+	// trivial case
+	c.Check(Log{}.Message(), Equals, "-")
+
+	// case where the JSON has a "null" JSON value for the key, which happens if
+	// the actual message is too large for journald to send
+	// we can't use the mustJSONMarshal helper for this in the test table
+	// because that gets decoded by Go differently than a verbatim nil here, it
+	// gets interpreted as the empty string rather than nil directly
+	c.Check(Log{"MESSAGE": nil}.Message(), Equals, "- (error decoding original message: message key \"MESSAGE\" truncated)")
+
+	for _, t := range tt {
+		if t.msg == nil {
+
+		}
+		c.Check(Log{
+			"MESSAGE": t.msg,
+		}.Message(), Equals, t.exp, Commentf(t.comment))
+	}
+}
+
+func (s *SystemdTestSuite) TestLogSID(c *C) {
+	c.Check(Log{}.SID(), Equals, "-")
+	c.Check(Log{"SYSLOG_IDENTIFIER": mustJSONMarshal("abcdef")}.SID(), Equals, "abcdef")
+	c.Check(Log{"SYSLOG_IDENTIFIER": mustJSONMarshal([]string{"abcdef"})}.SID(), Equals, "abcdef")
+	// multiple string values are not supported
+	c.Check(Log{"SYSLOG_IDENTIFIER": mustJSONMarshal([]string{"abc", "def"})}.SID(), Equals, "-")
+
+}
+
+func (s *SystemdTestSuite) TestLogPID(c *C) {
+	c.Check(Log{}.PID(), Equals, "-")
+	c.Check(Log{"_PID": mustJSONMarshal("99")}.PID(), Equals, "99")
+	c.Check(Log{"SYSLOG_PID": mustJSONMarshal("99")}.PID(), Equals, "99")
+	// things starting with underscore are "trusted", so we trust
+	// them more than the user-settable ones:
+	c.Check(Log{
+		"_PID":       mustJSONMarshal("42"),
+		"SYSLOG_PID": mustJSONMarshal("99"),
+	}.PID(), Equals, "42")
 }
 
 func (s *SystemdTestSuite) TestTime(c *C) {
 	t, err := Log{}.Time()
 	c.Check(t.IsZero(), Equals, true)
-	c.Check(err, ErrorMatches, "no timestamp")
+	c.Check(err, ErrorMatches, "key \"__REALTIME_TIMESTAMP\" missing from message")
 
-	t, err = Log{"__REALTIME_TIMESTAMP": "what"}.Time()
+	// multiple timestampe keys mean we don't have a timestamp
+	t, err = Log{"__REALTIME_TIMESTAMP": mustJSONMarshal([]string{"1", "2"})}.Time()
+	c.Check(t.IsZero(), Equals, true)
+	c.Check(err, ErrorMatches, `no timestamp`)
+
+	t, err = Log{"__REALTIME_TIMESTAMP": mustJSONMarshal("what")}.Time()
 	c.Check(t.IsZero(), Equals, true)
 	c.Check(err, ErrorMatches, `timestamp not a decimal number: "what"`)
 
-	t, err = Log{"__REALTIME_TIMESTAMP": "0"}.Time()
+	t, err = Log{"__REALTIME_TIMESTAMP": mustJSONMarshal("0")}.Time()
 	c.Check(err, IsNil)
 	c.Check(t.String(), Equals, "1970-01-01 00:00:00 +0000 UTC")
 
-	t, err = Log{"__REALTIME_TIMESTAMP": "42"}.Time()
+	t, err = Log{"__REALTIME_TIMESTAMP": mustJSONMarshal("42")}.Time()
 	c.Check(err, IsNil)
 	c.Check(t.String(), Equals, "1970-01-01 00:00:00.000042 +0000 UTC")
 }
@@ -554,7 +754,7 @@ func (s *SystemdTestSuite) TestAddMountUnit(c *C) {
 	mockSnapPath := filepath.Join(c.MkDir(), "/var/lib/snappy/snaps/foo_1.0.snap")
 	makeMockFile(c, mockSnapPath)
 
-	mountUnitName, err := New(rootDir, SystemMode, nil).AddMountUnitFile("foo", "42", mockSnapPath, "/snap/snapname/123", "squashfs")
+	mountUnitName, err := NewUnderRoot(rootDir, SystemMode, nil).AddMountUnitFile("foo", "42", mockSnapPath, "/snap/snapname/123", "squashfs")
 	c.Assert(err, IsNil)
 	defer os.Remove(mountUnitName)
 
@@ -562,12 +762,13 @@ func (s *SystemdTestSuite) TestAddMountUnit(c *C) {
 [Unit]
 Description=Mount unit for foo, revision 42
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
 Where=/snap/snapname/123
 Type=squashfs
-Options=nodev,ro,x-gdu.hide
+Options=nodev,ro,x-gdu.hide,x-gvfs-hide
 LazyUnmount=yes
 
 [Install]
@@ -587,7 +788,7 @@ func (s *SystemdTestSuite) TestAddMountUnitForDirs(c *C) {
 
 	// a directory instead of a file produces a different output
 	snapDir := c.MkDir()
-	mountUnitName, err := New("", SystemMode, nil).AddMountUnitFile("foodir", "x1", snapDir, "/snap/snapname/x1", "squashfs")
+	mountUnitName, err := New(SystemMode, nil).AddMountUnitFile("foodir", "x1", snapDir, "/snap/snapname/x1", "squashfs")
 	c.Assert(err, IsNil)
 	defer os.Remove(mountUnitName)
 
@@ -595,12 +796,13 @@ func (s *SystemdTestSuite) TestAddMountUnitForDirs(c *C) {
 [Unit]
 Description=Mount unit for foodir, revision x1
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
 Where=/snap/snapname/x1
 Type=none
-Options=nodev,ro,x-gdu.hide,bind
+Options=nodev,ro,x-gdu.hide,x-gvfs-hide,bind
 LazyUnmount=yes
 
 [Install]
@@ -609,7 +811,7 @@ WantedBy=multi-user.target
 
 	c.Assert(s.argses, DeepEquals, [][]string{
 		{"daemon-reload"},
-		{"--root", "", "enable", "snap-snapname-x1.mount"},
+		{"enable", "snap-snapname-x1.mount"},
 		{"start", "snap-snapname-x1.mount"},
 	})
 }
@@ -628,7 +830,7 @@ func (s *SystemdTestSuite) TestWriteSELinuxMountUnit(c *C) {
 	err = ioutil.WriteFile(mockSnapPath, nil, 0644)
 	c.Assert(err, IsNil)
 
-	mountUnitName, err := New("", SystemMode, nil).AddMountUnitFile("foo", "42", mockSnapPath, "/snap/snapname/123", "squashfs")
+	mountUnitName, err := New(SystemMode, nil).AddMountUnitFile("foo", "42", mockSnapPath, "/snap/snapname/123", "squashfs")
 	c.Assert(err, IsNil)
 	defer os.Remove(mountUnitName)
 
@@ -636,12 +838,13 @@ func (s *SystemdTestSuite) TestWriteSELinuxMountUnit(c *C) {
 [Unit]
 Description=Mount unit for foo, revision 42
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
 Where=/snap/snapname/123
 Type=squashfs
-Options=nodev,context=system_u:object_r:snappy_snap_t:s0,ro,x-gdu.hide
+Options=nodev,context=system_u:object_r:snappy_snap_t:s0,ro,x-gdu.hide,x-gvfs-hide
 LazyUnmount=yes
 
 [Install]
@@ -671,7 +874,7 @@ exit 0
 	err = ioutil.WriteFile(mockSnapPath, nil, 0644)
 	c.Assert(err, IsNil)
 
-	mountUnitName, err := New("", SystemMode, nil).AddMountUnitFile("foo", "x1", mockSnapPath, "/snap/snapname/123", "squashfs")
+	mountUnitName, err := New(SystemMode, nil).AddMountUnitFile("foo", "x1", mockSnapPath, "/snap/snapname/123", "squashfs")
 	c.Assert(err, IsNil)
 	defer os.Remove(mountUnitName)
 
@@ -679,12 +882,13 @@ exit 0
 [Unit]
 Description=Mount unit for foo, revision x1
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
 Where=/snap/snapname/123
 Type=fuse.squashfuse
-Options=nodev,ro,x-gdu.hide,allow_other
+Options=nodev,ro,x-gdu.hide,x-gvfs-hide,allow_other
 LazyUnmount=yes
 
 [Install]
@@ -710,7 +914,7 @@ exit 0
 	err = ioutil.WriteFile(mockSnapPath, nil, 0644)
 	c.Assert(err, IsNil)
 
-	mountUnitName, err := New("", SystemMode, nil).AddMountUnitFile("foo", "x1", mockSnapPath, "/snap/snapname/123", "squashfs")
+	mountUnitName, err := New(SystemMode, nil).AddMountUnitFile("foo", "x1", mockSnapPath, "/snap/snapname/123", "squashfs")
 	c.Assert(err, IsNil)
 	defer os.Remove(mountUnitName)
 
@@ -718,12 +922,13 @@ exit 0
 [Unit]
 Description=Mount unit for foo, revision x1
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
 Where=/snap/snapname/123
 Type=squashfs
-Options=nodev,ro,x-gdu.hide
+Options=nodev,ro,x-gdu.hide,x-gvfs-hide
 LazyUnmount=yes
 
 [Install]
@@ -751,6 +956,19 @@ func (s *SystemdTestSuite) TestJctl(c *C) {
 	c.Check(args, DeepEquals, []string{"-o", "json", "--no-pager", "--no-tail", "-u", "foo", "-u", "bar"})
 }
 
+func (s *SystemdTestSuite) TestIsActiveUnderRoot(c *C) {
+	sysErr := &Error{}
+	// manpage states that systemctl returns exit code 3 for inactive
+	// services, however we should check any non-0 exit status
+	sysErr.SetExitCode(1)
+	sysErr.SetMsg([]byte("inactive\n"))
+	s.errors = []error{sysErr}
+
+	_, err := NewUnderRoot("xyzzy", SystemMode, s.rep).IsActive("foo")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "is-active", "foo"}})
+}
+
 func (s *SystemdTestSuite) TestIsActiveIsInactive(c *C) {
 	sysErr := &Error{}
 	// manpage states that systemctl returns exit code 3 for inactive
@@ -759,10 +977,10 @@ func (s *SystemdTestSuite) TestIsActiveIsInactive(c *C) {
 	sysErr.SetMsg([]byte("inactive\n"))
 	s.errors = []error{sysErr}
 
-	active, err := New("xyzzy", SystemMode, s.rep).IsActive("foo")
+	active, err := New(SystemMode, s.rep).IsActive("foo")
 	c.Assert(active, Equals, false)
 	c.Assert(err, IsNil)
-	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "is-active", "foo"}})
+	c.Check(s.argses, DeepEquals, [][]string{{"is-active", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestIsActiveIsFailed(c *C) {
@@ -772,19 +990,19 @@ func (s *SystemdTestSuite) TestIsActiveIsFailed(c *C) {
 	sysErr.SetMsg([]byte("failed\n"))
 	s.errors = []error{sysErr}
 
-	active, err := New("xyzzy", SystemMode, s.rep).IsActive("foo")
+	active, err := New(SystemMode, s.rep).IsActive("foo")
 	c.Assert(active, Equals, false)
 	c.Assert(err, IsNil)
-	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "is-active", "foo"}})
+	c.Check(s.argses, DeepEquals, [][]string{{"is-active", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestIsActiveIsActive(c *C) {
 	s.errors = []error{nil}
 
-	active, err := New("xyzzy", SystemMode, s.rep).IsActive("foo")
+	active, err := New(SystemMode, s.rep).IsActive("foo")
 	c.Assert(active, Equals, true)
 	c.Assert(err, IsNil)
-	c.Check(s.argses, DeepEquals, [][]string{{"--root", "xyzzy", "is-active", "foo"}})
+	c.Check(s.argses, DeepEquals, [][]string{{"is-active", "foo"}})
 }
 
 func (s *SystemdTestSuite) TestIsActiveUnexpectedErr(c *C) {
@@ -793,7 +1011,7 @@ func (s *SystemdTestSuite) TestIsActiveUnexpectedErr(c *C) {
 	sysErr.SetMsg([]byte("random-failure\n"))
 	s.errors = []error{sysErr}
 
-	active, err := New("xyzzy", SystemMode, s.rep).IsActive("foo")
+	active, err := NewUnderRoot("xyzzy", SystemMode, s.rep).IsActive("foo")
 	c.Assert(active, Equals, false)
 	c.Assert(err, ErrorMatches, ".* failed with exit status 1: random-failure\n")
 }
@@ -814,7 +1032,7 @@ func (s *SystemdTestSuite) TestRemoveMountUnit(c *C) {
 
 	mountDir := rootDir + "/snap/foo/42"
 	mountUnit := makeMockMountUnit(c, mountDir)
-	err := New(rootDir, SystemMode, nil).RemoveMountUnitFile(mountDir)
+	err := NewUnderRoot(rootDir, SystemMode, nil).RemoveMountUnitFile(mountDir)
 
 	c.Assert(err, IsNil)
 	// the file is gone
@@ -832,7 +1050,7 @@ func (s *SystemdTestSuite) TestDaemonReloadMutex(c *C) {
 
 func (s *SystemdTestSuite) testDaemonReloadMutex(c *C, reload func(Systemd) error) {
 	rootDir := dirs.GlobalRootDir
-	sysd := New(rootDir, SystemMode, nil)
+	sysd := NewUnderRoot(rootDir, SystemMode, nil)
 
 	mockSnapPath := filepath.Join(c.MkDir(), "/var/lib/snappy/snaps/foo_1.0.snap")
 	makeMockFile(c, mockSnapPath)
@@ -869,7 +1087,7 @@ func (s *SystemdTestSuite) TestDaemonReexecMutex(c *C) {
 
 func (s *SystemdTestSuite) TestUserMode(c *C) {
 	rootDir := dirs.GlobalRootDir
-	sysd := New(rootDir, UserMode, nil)
+	sysd := NewUnderRoot(rootDir, UserMode, nil)
 
 	c.Assert(sysd.Enable("foo"), IsNil)
 	c.Check(s.argses[0], DeepEquals, []string{"--user", "--root", rootDir, "enable", "foo"})
@@ -879,7 +1097,7 @@ func (s *SystemdTestSuite) TestUserMode(c *C) {
 
 func (s *SystemdTestSuite) TestGlobalUserMode(c *C) {
 	rootDir := dirs.GlobalRootDir
-	sysd := New(rootDir, GlobalUserMode, nil)
+	sysd := NewUnderRoot(rootDir, GlobalUserMode, nil)
 
 	c.Assert(sysd.Enable("foo"), IsNil)
 	c.Check(s.argses[0], DeepEquals, []string{"--user", "--global", "--root", rootDir, "enable", "foo"})
@@ -901,14 +1119,51 @@ func (s *SystemdTestSuite) TestGlobalUserMode(c *C) {
 	c.Check(func() { sysd.Stop("foo", 0) }, Panics, "cannot call stop with GlobalUserMode")
 	c.Check(func() { sysd.Restart("foo", 0) }, Panics, "cannot call restart with GlobalUserMode")
 	c.Check(func() { sysd.Kill("foo", "HUP", "") }, Panics, "cannot call kill with GlobalUserMode")
-	c.Check(func() { sysd.Status("foo") }, Panics, "cannot call status with GlobalUserMode")
 	c.Check(func() { sysd.IsActive("foo") }, Panics, "cannot call is-active with GlobalUserMode")
 }
 
+func (s *SystemdTestSuite) TestStatusGlobalUserMode(c *C) {
+	output := []byte("enabled\ndisabled\nstatic\n")
+	sysdErr := &Error{}
+	sysdErr.SetExitCode(1)
+	sysdErr.SetMsg(output)
+
+	s.outs = [][]byte{output, nil, output}
+	s.errors = []error{nil, sysdErr, nil}
+
+	rootDir := dirs.GlobalRootDir
+	sysd := NewUnderRoot(rootDir, GlobalUserMode, nil)
+	sts, err := sysd.Status("foo", "bar", "baz")
+	c.Check(err, IsNil)
+	c.Check(sts, DeepEquals, []*UnitStatus{
+		{UnitName: "foo", Enabled: true},
+		{UnitName: "bar", Enabled: false},
+		{UnitName: "baz", Enabled: true},
+	})
+	c.Check(s.argses[0], DeepEquals, []string{"--user", "--global", "--root", rootDir, "is-enabled", "foo", "bar", "baz"})
+
+	// Output is collected if systemctl has a non-zero exit status
+	sts, err = sysd.Status("one", "two", "three")
+	c.Check(err, IsNil)
+	c.Check(sts, DeepEquals, []*UnitStatus{
+		{UnitName: "one", Enabled: true},
+		{UnitName: "two", Enabled: false},
+		{UnitName: "three", Enabled: true},
+	})
+	c.Check(s.argses[1], DeepEquals, []string{"--user", "--global", "--root", rootDir, "is-enabled", "one", "two", "three"})
+
+	// An error is returned if the wrong number of statuses are returned
+	sts, err = sysd.Status("one")
+	c.Check(err, ErrorMatches, "cannot get enabled status of services: expected 1 results, got 3")
+	c.Check(sts, IsNil)
+	c.Check(s.argses[2], DeepEquals, []string{"--user", "--global", "--root", rootDir, "is-enabled", "one"})
+}
+
 const unitTemplate = `
 [Unit]
 Description=Mount unit for foo, revision 42
 Before=snapd.service
+After=zfs-mount.service
 
 [Mount]
 What=%s
@@ -940,10 +1195,10 @@ func (s *SystemdTestSuite) TestPreseedModeAddMountUnit(c *C) {
 	// systemd was not called
 	c.Check(s.argses, HasLen, 0)
 	// mount was called
-	c.Check(mockMountCmd.Calls()[0], DeepEquals, []string{"mount", "-t", "squashfs", mockSnapPath, "/snap/snapname/123", "-o", "nodev,ro,x-gdu.hide"})
+	c.Check(mockMountCmd.Calls()[0], DeepEquals, []string{"mount", "-t", "squashfs", mockSnapPath, "/snap/snapname/123", "-o", "nodev,ro,x-gdu.hide,x-gvfs-hide"})
 	// unit was enabled with a symlink
 	c.Check(osutil.IsSymlink(filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", mountUnitName)), Equals, true)
-	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(unitTemplate[1:], mockSnapPath, "squashfs", "nodev,ro,x-gdu.hide"))
+	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(unitTemplate[1:], mockSnapPath, "squashfs", "nodev,ro,x-gdu.hide,x-gvfs-hide"))
 }
 
 func (s *SystemdTestSuite) TestPreseedModeAddMountUnitWithFuse(c *C) {
@@ -963,7 +1218,7 @@ func (s *SystemdTestSuite) TestPreseedModeAddMountUnitWithFuse(c *C) {
 	defer os.Remove(mountUnitName)
 
 	c.Check(mockMountCmd.Calls()[0], DeepEquals, []string{"mount", "-t", "fuse.squashfuse", mockSnapPath, "/snap/snapname/123", "-o", "nodev,a,b,c"})
-	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(unitTemplate[1:], mockSnapPath, "squashfs", "nodev,ro,x-gdu.hide"))
+	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(unitTemplate[1:], mockSnapPath, "squashfs", "nodev,ro,x-gdu.hide,x-gvfs-hide"))
 }
 
 func (s *SystemdTestSuite) TestPreseedModeMountError(c *C) {
@@ -1086,3 +1341,167 @@ func (s *SystemdTestSuite) TestUnmaskInEmulationMode(c *C) {
 	c.Check(s.argses, DeepEquals, [][]string{
 		{"--root", "/path", "unmask", "foo"}})
 }
+
+func (s *SystemdTestSuite) TestMountHappy(c *C) {
+	sysd := New(SystemMode, nil)
+
+	cmd := testutil.MockCommand(c, "systemd-mount", "")
+	defer cmd.Restore()
+
+	c.Assert(sysd.Mount("foo", "bar"), IsNil)
+	c.Check(cmd.Calls(), DeepEquals, [][]string{
+		{"systemd-mount", "foo", "bar"},
+	})
+	cmd.ForgetCalls()
+	c.Assert(sysd.Mount("foo", "bar", "-o", "bind"), IsNil)
+	c.Check(cmd.Calls(), DeepEquals, [][]string{
+		{"systemd-mount", "-o", "bind", "foo", "bar"},
+	})
+}
+
+func (s *SystemdTestSuite) TestMountErr(c *C) {
+	sysd := New(SystemMode, nil)
+
+	cmd := testutil.MockCommand(c, "systemd-mount", `echo "failed"; exit 111`)
+	defer cmd.Restore()
+
+	err := sysd.Mount("foo", "bar")
+	c.Assert(err, ErrorMatches, "failed")
+	c.Check(cmd.Calls(), DeepEquals, [][]string{
+		{"systemd-mount", "foo", "bar"},
+	})
+}
+
+func (s *SystemdTestSuite) TestUmountHappy(c *C) {
+	sysd := New(SystemMode, nil)
+
+	cmd := testutil.MockCommand(c, "systemd-mount", "")
+	defer cmd.Restore()
+
+	c.Assert(sysd.Umount("bar"), IsNil)
+	c.Check(cmd.Calls(), DeepEquals, [][]string{
+		{"systemd-mount", "--umount", "bar"},
+	})
+}
+
+func (s *SystemdTestSuite) TestUmountErr(c *C) {
+	sysd := New(SystemMode, nil)
+
+	cmd := testutil.MockCommand(c, "systemd-mount", `echo "failed"; exit 111`)
+	defer cmd.Restore()
+
+	err := sysd.Umount("bar")
+	c.Assert(err, ErrorMatches, "failed")
+	c.Check(cmd.Calls(), DeepEquals, [][]string{
+		{"systemd-mount", "--umount", "bar"},
+	})
+}
+
+func (s *SystemdTestSuite) TestCurrentMemoryUsageInactive(c *C) {
+	s.outs = [][]byte{
+		[]byte(`MemoryCurrent=[not set]`),
+	}
+	sysd := New(SystemMode, s.rep)
+	_, err := sysd.CurrentMemoryUsage("bar.service")
+	c.Assert(err, ErrorMatches, "memory usage unavailable")
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "MemoryCurrent", "bar.service"},
+	})
+}
+
+func (s *SystemdTestSuite) TestCurrentMemoryUsageInvalid(c *C) {
+	s.outs = [][]byte{
+		[]byte(`MemoryCurrent=blahhhhhhhhhhhh`),
+	}
+	sysd := New(SystemMode, s.rep)
+	_, err := sysd.CurrentMemoryUsage("bar.service")
+	c.Assert(err, ErrorMatches, `invalid property value from systemd for MemoryCurrent: cannot parse "blahhhhhhhhhhhh" as an integer`)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "MemoryCurrent", "bar.service"},
+	})
+}
+
+func (s *SystemdTestSuite) TestCurrentMemoryUsage(c *C) {
+	s.outs = [][]byte{
+		[]byte(`MemoryCurrent=1024`),
+	}
+	sysd := New(SystemMode, s.rep)
+	usage, err := sysd.CurrentMemoryUsage("bar.service")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "MemoryCurrent", "bar.service"},
+	})
+	c.Check(usage, Equals, quantity.SizeKiB)
+}
+
+func (s *SystemdTestSuite) TestInactiveEnterTimestampZero(c *C) {
+	s.outs = [][]byte{
+		[]byte(`InactiveEnterTimestamp=`),
+	}
+	sysd := New(SystemMode, s.rep)
+	stamp, err := sysd.InactiveEnterTimestamp("bar.service")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "InactiveEnterTimestamp", "bar.service"},
+	})
+	c.Check(stamp.IsZero(), Equals, true)
+}
+
+func (s *SystemdTestSuite) TestInactiveEnterTimestampValidWhitespace(c *C) {
+	s.outs = [][]byte{
+		[]byte(`InactiveEnterTimestamp=Fri 2021-04-16 15:32:21 UTC
+`),
+	}
+
+	stamp, err := New(SystemMode, s.rep).InactiveEnterTimestamp("bar.service")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "InactiveEnterTimestamp", "bar.service"},
+	})
+	c.Check(stamp.Equal(time.Date(2021, time.April, 16, 15, 32, 21, 0, time.UTC)), Equals, true)
+}
+
+func (s *SystemdTestSuite) TestInactiveEnterTimestampValid(c *C) {
+	s.outs = [][]byte{
+		[]byte(`InactiveEnterTimestamp=Fri 2021-04-16 15:32:21 UTC`),
+	}
+
+	stamp, err := New(SystemMode, s.rep).InactiveEnterTimestamp("bar.service")
+	c.Assert(err, IsNil)
+	c.Check(s.argses, DeepEquals, [][]string{
+		{"show", "--property", "InactiveEnterTimestamp", "bar.service"},
+	})
+	c.Check(stamp.Equal(time.Date(2021, time.April, 16, 15, 32, 21, 0, time.UTC)), Equals, true)
+}
+
+func (s *SystemdTestSuite) TestInactiveEnterTimestampFailure(c *C) {
+	s.outs = [][]byte{
+		[]byte(`mocked failure`),
+	}
+	s.errors = []error{
+		fmt.Errorf("mocked failure"),
+	}
+	stamp, err := New(SystemMode, s.rep).InactiveEnterTimestamp("bar.service")
+	c.Assert(err, ErrorMatches, "mocked failure")
+	c.Check(stamp.IsZero(), Equals, true)
+}
+
+func (s *SystemdTestSuite) TestInactiveEnterTimestampMalformed(c *C) {
+	s.outs = [][]byte{
+		[]byte(`InactiveEnterTimestamp`),
+		[]byte(``),
+		[]byte(`some random garbage
+with newlines`),
+		[]byte(`InactiveEnterTimestamp=0`), // 0 is valid for InactiveEnterTimestampMonotonic
+	}
+	sysd := New(SystemMode, s.rep)
+	for i := 0; i < len(s.outs); i++ {
+		s.argses = nil
+		stamp, err := sysd.InactiveEnterTimestamp("bar.service")
+		c.Assert(err, ErrorMatches, `(?s)internal error: systemctl (time )?output \(.*\) is malformed`)
+		c.Check(s.argses, DeepEquals, [][]string{
+			{"show", "--property", "InactiveEnterTimestamp", "bar.service"},
+		})
+		c.Check(stamp.IsZero(), Equals, true)
+	}
+}
