@@ -12,7 +12,6 @@ NESTED_RUNTIME_DIR="$NESTED_WORK_DIR/runtime"
 NESTED_ASSETS_DIR="$NESTED_WORK_DIR/assets"
 NESTED_LOGS_DIR="$NESTED_WORK_DIR/logs"
 
-
 NESTED_VM=nested-vm
 NESTED_SSH_PORT=8022
 NESTED_MON_PORT=8888
@@ -117,6 +116,55 @@ nested_prepare_ssh() {
     nested_exec_as external ubuntu "sudo true"
 }
 
+
+nested_is_kvm_enabled() {
+    if [ -n "$NESTED_ENABLE_KVM" ]; then
+        [ "$NESTED_ENABLE_KVM" = true ]
+    fi
+    return 0
+
+}
+
+nested_is_tpm_enabled() {
+    if [ -n "$NESTED_ENABLE_TPM" ]; then
+        [ "$NESTED_ENABLE_TPM" = true ]
+    else
+        case "${SPREAD_SYSTEM:-}" in
+            ubuntu-1*)
+                return 1
+                ;;
+            ubuntu-2*)
+                # TPM enabled by default on 20.04 and later
+                return 0
+                ;;
+            *)
+                echo "unsupported system"
+                exit 1
+                ;;
+        esac
+    fi
+}
+
+nested_is_secure_boot_enabled() {
+    if [ -n "$NESTED_ENABLE_SECURE_BOOT" ]; then
+        [ "$NESTED_ENABLE_SECURE_BOOT" = true ]
+    else
+        case "${SPREAD_SYSTEM:-}" in
+            ubuntu-1*)
+                return 1
+                ;;
+            ubuntu-2*)
+                # secure boot enabled by default on 20.04 and later
+                return 0
+                ;;
+            *)
+                echo "unsupported system"
+                exit 1
+                ;;
+        esac
+    fi
+}
+
 nested_create_assertions_disk() {
     mkdir -p "$NESTED_ASSETS_DIR"
     local ASSERTIONS_DISK LOOP_DEV
@@ -178,22 +226,25 @@ nested_qemu_name() {
 nested_get_google_image_url_for_vm() {
     case "${1:-$SPREAD_SYSTEM}" in
         ubuntu-16.04-64)
-            echo "https://storage.googleapis.com/spread-snapd-tests/images/cloudimg/xenial-server-cloudimg-amd64-disk1.img"
+            echo "https://storage.googleapis.com/snapd-spread-tests/images/cloudimg/xenial-server-cloudimg-amd64-disk1.img"
             ;;
         ubuntu-18.04-64)
-            echo "https://storage.googleapis.com/spread-snapd-tests/images/cloudimg/bionic-server-cloudimg-amd64.img"
+            echo "https://storage.googleapis.com/snapd-spread-tests/images/cloudimg/bionic-server-cloudimg-amd64.img"
             ;;
         ubuntu-20.04-64)
-            echo "https://storage.googleapis.com/spread-snapd-tests/images/cloudimg/focal-server-cloudimg-amd64.img"
+            echo "https://storage.googleapis.com/snapd-spread-tests/images/cloudimg/focal-server-cloudimg-amd64.img"
             ;;
         ubuntu-20.10-64*)
-            echo "https://storage.googleapis.com/spread-snapd-tests/images/cloudimg/groovy-server-cloudimg-amd64.img"
+            echo "https://storage.googleapis.com/snapd-spread-tests/images/cloudimg/groovy-server-cloudimg-amd64.img"
+            ;;
+        ubuntu-21.04-64*)
+            echo "https://storage.googleapis.com/snapd-spread-tests/images/cloudimg/hirsute-server-cloudimg-amd64.img"
             ;;
         *)
             echo "unsupported system"
             exit 1
             ;;
-        esac
+    esac
 }
 
 # shellcheck disable=SC2120
@@ -211,6 +262,9 @@ nested_get_ubuntu_image_url_for_vm() {
         ubuntu-20.10-64*)
             echo "https://cloud-images.ubuntu.com/groovy/current/groovy-server-cloudimg-amd64.img"
             ;;
+        ubuntu-21.04-64*)
+            echo "https://cloud-images.ubuntu.com/hirsute/current/hirsute-server-cloudimg-amd64.img"
+            ;;
         *)
             echo "unsupported system"
             exit 1
@@ -238,9 +292,15 @@ nested_get_cdimage_current_image_url() {
 nested_get_snap_rev_for_channel() {
     local SNAP=$1
     local CHANNEL=$2
-    # This should be executed on remote system but as nested architecture is the same than the
-    # host then the snap info is executed in the host
-    snap info "$SNAP" | grep "$CHANNEL" | awk '{ print $4 }' | sed 's/.*(\(.*\))/\1/' | tr -d '\n'
+
+    curl -s \
+         -H "Snap-Device-Architecture: ${NESTED_ARCHITECTURE:-amd64}" \
+         -H "Snap-Device-Series: 16" \
+         -X POST \
+         -H "Content-Type: application/json" \
+         --data "{\"context\": [], \"actions\": [{\"action\": \"install\", \"name\": \"$SNAP\", \"channel\": \"$CHANNEL\", \"instance-key\": \"1\"}]}" \
+         https://api.snapcraft.io/v2/snaps/refresh | \
+        jq '.results[0].snap.revision'
 }
 
 nested_is_nested_system() {
@@ -436,7 +496,8 @@ nested_model_authority() {
 
 nested_ensure_ubuntu_save() {
     local GADGET_DIR="$1"
-    "$TESTSLIB"/ensure_ubuntu_save.py "$GADGET_DIR"/meta/gadget.yaml > /tmp/gadget-with-save.yaml
+    shift
+    "$TESTSLIB"/ensure_ubuntu_save.py "$@" "$GADGET_DIR"/meta/gadget.yaml > /tmp/gadget-with-save.yaml
     if [ "$(cat /tmp/gadget-with-save.yaml)" != "" ]; then
         mv /tmp/gadget-with-save.yaml "$GADGET_DIR"/meta/gadget.yaml
     else
@@ -455,10 +516,11 @@ nested_create_core_vm() {
 
     mkdir -p "$NESTED_IMAGES_DIR"
 
-    if [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME".xz ]; then
-        nested_uncompress_image "$IMAGE_NAME"
-    elif [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ]; then
+    if [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine" ]; then
+        cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine" "$NESTED_IMAGES_DIR/$IMAGE_NAME"
+        return
 
+    elif [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ]; then
         if [ -n "$NESTED_CUSTOM_IMAGE_URL" ]; then
             # download the ubuntu-core image from $CUSTOM_IMAGE_URL
             nested_download_image "$NESTED_CUSTOM_IMAGE_URL" "$IMAGE_NAME"
@@ -492,7 +554,15 @@ nested_create_core_vm() {
 
                 elif nested_is_core_20_system; then
                     snap download --basename=pc-kernel --channel="20/edge" pc-kernel
-                    uc20_build_initramfs_kernel_snap "$PWD/pc-kernel.snap" "$NESTED_ASSETS_DIR"
+
+                    # set the unix bump time if the NESTED_* var is set, 
+                    # otherwise leave it empty
+                    local epochBumpTime
+                    epochBumpTime=${NESTED_CORE20_INITRAMFS_EPOCH_TIMESTAMP:-}
+                    if [ -n "$epochBumpTime" ]; then
+                        epochBumpTime="--epoch-bump-time=$epochBumpTime"
+                    fi
+                    uc20_build_initramfs_kernel_snap "$PWD/pc-kernel.snap" "$NESTED_ASSETS_DIR" "$epochBumpTime"
                     rm -f "$PWD/pc-kernel.snap"
 
                     # Prepare the pc kernel snap
@@ -523,13 +593,22 @@ nested_create_core_vm() {
                         snap download --basename=pc --channel="20/edge" pc
                         unsquashfs -d pc-gadget pc.snap
                         nested_secboot_sign_gadget pc-gadget "$SNAKEOIL_KEY" "$SNAKEOIL_CERT"
-                        # TODO:UC20: until https://github.com/snapcore/pc-amd64-gadget/pull/51/
-                        # lands there is no ubuntu-save in the gadget, make sure we have one
-                        nested_ensure_ubuntu_save pc-gadget
+                        case "${NESTED_UBUNTU_SAVE:-}" in
+                            add)
+                                # ensure that ubuntu-save is present
+                                nested_ensure_ubuntu_save pc-gadget --add
+                                touch ubuntu-save-added
+                                ;;
+                            remove)
+                                # ensure that ubuntu-save is removed
+                                nested_ensure_ubuntu_save pc-gadget --remove
+                                touch ubuntu-save-removed
+                                ;;
+                        esac
 
-                        # also make logging persistent for easier debugging of 
-                        # test failures, otherwise we have no way to see what 
-                        # happened during a failed nested VM boot where we 
+                        # also make logging persistent for easier debugging of
+                        # test failures, otherwise we have no way to see what
+                        # happened during a failed nested VM boot where we
                         # weren't able to login to a device
                         cat >> pc-gadget/meta/gadget.yaml << EOF
 defaults:
@@ -591,6 +670,7 @@ EOF
             else 
                 UBUNTU_IMAGE_CHANNEL_ARG=""
             fi
+            # ubuntu-image creates sparse image files
             "$UBUNTU_IMAGE" snap --image-size 10G "$NESTED_MODEL" \
                 "$UBUNTU_IMAGE_CHANNEL_ARG" \
                 --output "$NESTED_IMAGES_DIR/$IMAGE_NAME" \
@@ -612,9 +692,8 @@ EOF
         nested_create_assertions_disk
     fi
 
-    # Save a compressed copy of the image
-    # TODO: analyze if it is better to compress just when the image is generic
-    nested_compress_image "$IMAGE_NAME"
+    # Save a copy of the image
+    cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME" "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine"
 }
 
 nested_configure_cloud_init_on_core_vm() {
@@ -790,7 +869,7 @@ nested_start_core_vm_unit() {
         exit 1
     fi
 
-    local PARAM_DISPLAY PARAM_NETWORK PARAM_MONITOR PARAM_USB PARAM_CD PARAM_RANDOM PARAM_CPU PARAM_TRACE PARAM_LOG PARAM_SERIAL
+    local PARAM_DISPLAY PARAM_NETWORK PARAM_MONITOR PARAM_USB PARAM_CD PARAM_RANDOM PARAM_CPU PARAM_TRACE PARAM_LOG PARAM_SERIAL PARAM_RTC
     PARAM_DISPLAY="-nographic"
     PARAM_NETWORK="-net nic,model=virtio -net user,hostfwd=tcp::$NESTED_SSH_PORT-:22"
     PARAM_MONITOR="-monitor tcp:127.0.0.1:$NESTED_MON_PORT,server,nowait"
@@ -800,6 +879,8 @@ nested_start_core_vm_unit() {
     PARAM_CPU=""
     PARAM_TRACE="-d cpu_reset"
     PARAM_LOG="-D $NESTED_LOGS_DIR/qemu.log"
+    PARAM_RTC="${NESTED_PARAM_RTC:-}"
+
     # Open port 7777 on the host so that failures in the nested VM (e.g. to
     # create users) can be debugged interactively via
     # "telnet localhost 7777". Also keeps the logs
@@ -819,7 +900,7 @@ nested_start_core_vm_unit() {
     # Set kvm attribute
     local ATTR_KVM
     ATTR_KVM=""
-    if [ "$NESTED_ENABLE_KVM" = "true" ]; then
+    if nested_is_kvm_enabled; then
         ATTR_KVM=",accel=kvm"
         # CPU can be defined just when kvm is enabled
         PARAM_CPU="-cpu host"
@@ -873,17 +954,17 @@ nested_start_core_vm_unit() {
             PARAM_BIOS="-bios /usr/share/OVMF/OVMF_CODE.fd"
         fi
         
-        if [ "$NESTED_ENABLE_SECURE_BOOT" = "true" ]; then
+        if nested_is_secure_boot_enabled; then
             cp -f "/usr/share/OVMF/OVMF_VARS.$OVMF_VARS.fd" "$NESTED_ASSETS_DIR/OVMF_VARS.$OVMF_VARS.fd"
             PARAM_BIOS="-drive file=/usr/share/OVMF/OVMF_CODE.$OVMF_CODE.fd,if=pflash,format=raw,unit=0,readonly -drive file=$NESTED_ASSETS_DIR/OVMF_VARS.$OVMF_VARS.fd,if=pflash,format=raw"
             PARAM_MACHINE="-machine q35${ATTR_KVM} -global ICH9-LPC.disable_s3=1"
         fi
 
-        if [ "$NESTED_ENABLE_TPM" = "true" ]; then
-            if snap list swtpm-mvo; then
+        if nested_is_tpm_enabled; then
+            if snap list swtpm-mvo >/dev/null; then
                 # reset the tpm state
                 rm /var/snap/swtpm-mvo/current/tpm2-00.permall
-                snap restart swtpm-mvo
+                snap restart swtpm-mvo > /dev/null
             else
                 snap install swtpm-mvo --beta
             fi
@@ -907,6 +988,7 @@ nested_start_core_vm_unit() {
         ${PARAM_MEM} \
         ${PARAM_TRACE} \
         ${PARAM_LOG} \
+        ${PARAM_RTC} \
         ${PARAM_MACHINE} \
         ${PARAM_DISPLAY} \
         ${PARAM_NETWORK} \
@@ -923,14 +1005,19 @@ nested_start_core_vm_unit() {
     # wait for the nested-vm service to appear active
     wait_for_service "$NESTED_VM"
 
-    # Wait until ssh is ready
-    nested_wait_for_ssh
-    # Wait for the snap command to be available
-    nested_wait_for_snap_command
-    # Wait for snap seeding to be done
-    nested_exec "sudo snap wait system seed.loaded"
-    # Wait for cloud init to be done
-    nested_exec "cloud-init status --wait"
+    local EXPECT_SHUTDOWN
+    EXPECT_SHUTDOWN=${NESTED_EXPECT_SHUTDOWN:-}
+
+    if [ "$EXPECT_SHUTDOWN" != "1" ]; then
+        # Wait until ssh is ready
+        nested_wait_for_ssh
+        # Wait for the snap command to be available
+        nested_wait_for_snap_command
+        # Wait for snap seeding to be done
+        nested_exec "sudo snap wait system seed.loaded"
+        # Wait for cloud init to be done
+        nested_exec "cloud-init status --wait"
+    fi
 }
 
 nested_get_current_image_name() {
@@ -953,35 +1040,31 @@ nested_start_core_vm() {
         # exists
         local IMAGE_NAME
         IMAGE_NAME="$(nested_get_image_name core)"
-        if ! [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.xz" ] && ! [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ]; then
+        if ! [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ]; then
             echo "No image found to be started"
             exit 1
         fi
 
-        # First time the image is used $IMAGE_NAME exists so it is used, otherwise
-        # the saved image from previous run is uncompressed
-        if ! [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ]; then
-            nested_uncompress_image "$IMAGE_NAME"
-        fi
-        mv "$NESTED_IMAGES_DIR/$IMAGE_NAME" "$CURRENT_IMAGE"
+        # images are created as sparse files, simple cp should preserve that
+        # property
+        cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME" "$CURRENT_IMAGE"
 
         # Start the nested core vm
         nested_start_core_vm_unit "$CURRENT_IMAGE"
 
-        if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.xz.configured" ]; then
+        if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.configured" ]; then
             # configure ssh for first time
             nested_prepare_ssh
             sync
 
-            # compress the current image if it is a generic image
+            # keep a copy of the current image if it is a generic image
             if nested_is_generic_image && [ "$NESTED_CONFIGURE_IMAGES" = "true" ]; then
                 # Stop the current image and compress it
                 nested_shutdown
-                nested_compress_image "$CURRENT_NAME"
 
                 # Save the image with the name of the original image
-                mv "${CURRENT_IMAGE}.xz" "$NESTED_IMAGES_DIR/$IMAGE_NAME.xz"
-                touch "$NESTED_IMAGES_DIR/$IMAGE_NAME.xz.configured"
+                cp -v "${CURRENT_IMAGE}" "$NESTED_IMAGES_DIR/$IMAGE_NAME"
+                touch "$NESTED_IMAGES_DIR/$IMAGE_NAME.configured"
 
                 # Start the current image again and wait until it is ready
                 nested_start
@@ -1013,18 +1096,6 @@ nested_start() {
     nested_wait_for_ssh
 }
 
-nested_compress_image() {
-    local IMAGE_NAME=$1
-    if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME".xz ]; then
-        xz -k0 "$NESTED_IMAGES_DIR/$IMAGE_NAME"
-    fi
-}
-
-nested_uncompress_image() {
-    local IMAGE_NAME=$1
-    unxz -kf "$NESTED_IMAGES_DIR/$IMAGE_NAME".xz
-}
-
 nested_create_classic_vm() {
     local IMAGE_NAME
     IMAGE_NAME="$(nested_get_image_name classic)"
@@ -1042,8 +1113,8 @@ nested_create_classic_vm() {
         cloud-localds -H "$(hostname)" "$NESTED_ASSETS_DIR/seed.img" "$NESTED_ASSETS_DIR/seed"
     fi
 
-    # Save a compressed copy of the image
-    nested_compress_image "$IMAGE_NAME"
+    # Save a copy of the image
+    cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME" "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine"
 }
 
 nested_start_classic_vm() {
@@ -1051,8 +1122,8 @@ nested_start_classic_vm() {
     QEMU="$(nested_qemu_name)"
     IMAGE_NAME="$(nested_get_image_name classic)"
 
-    if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ] && [ -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.xz" ]; then
-        nested_uncompress_image "$IMAGE_NAME"
+    if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ] ; then
+        cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine" "$IMAGE_NAME"
     fi
 
     # Now qemu parameters are defined
@@ -1118,7 +1189,8 @@ nested_start_classic_vm() {
     # save logs from previous runs
     nested_save_serial_log
 
-    # Systemd unit is created, it is important to respect the qemu parameters order
+    # Systemd unit is created, it is important to respect the qemu parameters 
+    # order
     systemd_create_and_start_unit "$NESTED_VM" "${QEMU}  \
         ${PARAM_SMP} \
         ${PARAM_CPU} \
@@ -1206,7 +1278,7 @@ nested_get_core_revision_installed() {
 nested_fetch_spread() {
     if [ ! -f "$NESTED_WORK_DIR/spread" ]; then
         mkdir -p "$NESTED_WORK_DIR"
-        curl https://niemeyer.s3.amazonaws.com/spread-amd64.tar.gz | tar -xzv -C "$NESTED_WORK_DIR"
+        curl https://storage.googleapis.com/snapd-spread-tests/spread/spread-amd64.tar.gz | tar -xzv -C "$NESTED_WORK_DIR"
         # make sure spread really exists
         test -x "$NESTED_WORK_DIR/spread"
         echo "$NESTED_WORK_DIR/spread"
