@@ -5,12 +5,6 @@ SNAPD_STATE_FILE="$SPREAD_PATH/tests/snapd-state/snapd-state.tar"
 RUNTIME_STATE_PATH="$SPREAD_PATH/tests/runtime-state"
 SNAPD_ACTIVE_UNITS="$RUNTIME_STATE_PATH/snapd-active-units"
 
-# shellcheck source=tests/lib/dirs.sh
-. "$TESTSLIB/dirs.sh"
-
-# shellcheck source=tests/lib/boot.sh
-. "$TESTSLIB/boot.sh"
-
 # shellcheck source=tests/lib/systemd.sh
 . "$TESTSLIB/systemd.sh"
 
@@ -35,7 +29,7 @@ is_snapd_state_saved() {
 
 save_snapd_state() {
     if os.query is-core; then
-        boot_path="$(get_boot_path)"
+        boot_path="$("$TESTSTOOLS"/boot-state boot-path)"
         test -n "$boot_path" || return 1
 
         mkdir -p "$SNAPD_STATE_PATH"/system-units
@@ -49,6 +43,7 @@ save_snapd_state() {
         cp -a /var/snap/* "$SNAPD_STATE_PATH"/var-snap/
     else
         systemctl daemon-reload
+        SNAP_MOUNT_DIR="$(os.paths snap-mount-dir)"
         escaped_snap_mount_dir="$(systemd-escape --path "$SNAP_MOUNT_DIR")"
         units="$(systemctl list-unit-files --full | grep -e "^${escaped_snap_mount_dir}[-.].*\\.mount" -e "^${escaped_snap_mount_dir}[-.].*\\.service" | cut -f1 -d ' ')"
         for unit in $units; do
@@ -89,7 +84,7 @@ restore_snapd_state() {
     if os.query is-core; then
         # we need to ensure that we also restore the boot environment
         # fully for tests that break it
-        boot_path="$(get_boot_path)"
+        boot_path="$("$TESTSTOOLS"/boot-state boot-path)"
         test -n "$boot_path" || return 1
 
         restore_snapd_lib
