@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -23,6 +23,7 @@ import (
 	"bytes"
 	"context"
 	"fmt"
+	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
@@ -31,6 +32,7 @@ import (
 	"text/template"
 	"time"
 
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
@@ -38,6 +40,7 @@ import (
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/randutil"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/quota"
 	"github.com/snapcore/snapd/strutil"
 	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timeout"
@@ -61,6 +64,9 @@ func serviceStopTimeout(app *snap.AppInfo) time.Duration {
 	return time.Duration(tout)
 }
 
+// TODO: this should not accept AddSnapServicesOptions, it should use some other
+// subset of options, specifically it should not accept Preseeding as an option
+// here
 func generateSnapServiceFile(app *snap.AppInfo, opts *AddSnapServicesOptions) ([]byte, error) {
 	if err := snap.ValidateApp(app); err != nil {
 		return nil, err
@@ -69,6 +75,26 @@ func generateSnapServiceFile(app *snap.AppInfo, opts *AddSnapServicesOptions) ([
 	return genServiceFile(app, opts)
 }
 
+// generateGroupSliceFile generates a systemd slice unit definition for the
+// specified quota group.
+func generateGroupSliceFile(grp *quota.Group) ([]byte, error) {
+	buf := bytes.Buffer{}
+
+	template := `[Unit]
+Description=Slice for snap quota group %[1]s
+Before=slices.target
+
+[Slice]
+# Always enable memory accounting otherwise the MemoryMax setting does nothing.
+MemoryAccounting=true
+MemoryMax=%[2]d
+`
+
+	fmt.Fprintf(&buf, template, grp.Name, grp.MemoryLimit)
+
+	return buf.Bytes(), nil
+}
+
 func stopUserServices(cli *client.Client, inter interacter, services ...string) error {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(timeout.DefaultTimeout))
 	defer cancel()
@@ -204,12 +230,61 @@ type StartServicesFlags struct {
 // are services. Service units will be started in the order provided by the
 // caller.
 func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServicesFlags, inter interacter, tm timings.Measurer) (err error) {
+	if flags == nil {
+		flags = &StartServicesFlags{}
+	}
+
 	systemSysd := systemd.New(systemd.SystemMode, inter)
 	userSysd := systemd.New(systemd.GlobalUserMode, inter)
 	cli := client.New()
 
+	var disableEnabledServices func()
+
+	defer func() {
+		if err == nil {
+			return
+		}
+		if disableEnabledServices != nil {
+			disableEnabledServices()
+		}
+	}()
+
+	var toEnable []*snap.AppInfo
 	systemServices := make([]string, 0, len(apps))
 	userServices := make([]string, 0, len(apps))
+
+	// gather all non-sockets, non-timers, and non-dbus activated
+	// services to enable first
+	for _, app := range apps {
+		// they're *supposed* to be all services, but checking doesn't hurt
+		if !app.IsService() {
+			continue
+		}
+		// sockets and timers are enabled and started separately (and unconditionally) further down.
+		// dbus activatable services are started on first use.
+		if len(app.Sockets) == 0 && app.Timer == nil && len(app.ActivatesOn) == 0 {
+			if strutil.ListContains(disabledSvcs, app.Name) {
+				continue
+			}
+			svcName := app.ServiceName()
+			switch app.DaemonScope {
+			case snap.SystemDaemon:
+				systemServices = append(systemServices, svcName)
+			case snap.UserDaemon:
+				userServices = append(userServices, svcName)
+			}
+			if flags.Enable {
+				toEnable = append(toEnable, app)
+			}
+		}
+	}
+
+	disableEnabledServices, err = enableServices(toEnable, inter)
+	if err != nil {
+		return err
+	}
+
+	// handle sockets and timers
 	for _, app := range apps {
 		// they're *supposed* to be all services, but checking doesn't hurt
 		if !app.IsService() {
@@ -228,6 +303,7 @@ func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServ
 			if err == nil {
 				return
 			}
+
 			if e := stopService(sysd, app, inter); e != nil {
 				inter.Notify(fmt.Sprintf("While trying to stop previously started service %q: %v", app.ServiceName(), e))
 			}
@@ -245,29 +321,10 @@ func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServ
 			}
 		}(app)
 
-		if len(app.Sockets) == 0 && app.Timer == nil {
-			// check if the service is disabled, if so don't start it up
-			// this could happen for example if the service was disabled in
-			// the install hook by snapctl or if the service was disabled in
-			// the previous installation
-			isEnabled, err := sysd.IsEnabled(app.ServiceName())
-			if err != nil {
-				return err
-			}
-			if isEnabled {
-				switch app.DaemonScope {
-				case snap.SystemDaemon:
-					systemServices = append(systemServices, app.ServiceName())
-				case snap.UserDaemon:
-					userServices = append(userServices, app.ServiceName())
-				}
-			}
-		}
-
 		for _, socket := range app.Sockets {
 			socketService := filepath.Base(socket.File())
 			// enable the socket
-			if err := sysd.Enable(socketService); err != nil {
+			if err = sysd.Enable(socketService); err != nil {
 				return err
 			}
 
@@ -289,7 +346,7 @@ func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServ
 		if app.Timer != nil {
 			timerService := filepath.Base(app.Timer.File())
 			// enable the timer
-			if err := sysd.Enable(timerService); err != nil {
+			if err = sysd.Enable(timerService); err != nil {
 				return err
 			}
 
@@ -324,6 +381,7 @@ func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServ
 			return err
 		}
 	}
+
 	if len(userServices) != 0 {
 		timings.Run(tm, "start-user-services", "start user services", func(nested timings.Measurer) {
 			err = startUserServices(cli, inter, userServices...)
@@ -343,145 +401,297 @@ func userDaemonReload() error {
 	return cli.ServicesDaemonReload(ctx)
 }
 
-type AddSnapServicesOptions struct {
-	Preseeding   bool
-	VitalityRank int
+func tryFileUpdate(path string, desiredContent []byte) (old *osutil.MemoryFileState, modified bool, err error) {
+	newFileState := osutil.MemoryFileState{
+		Content: desiredContent,
+		Mode:    os.FileMode(0644),
+	}
+
+	// get the existing content (if any) of the file to have something to
+	// rollback to if we have any errors
+
+	// note we can't use FileReference here since we may be modifying
+	// the file, and the FileReference.State() wouldn't be evaluated
+	// until _after_ we attempted modification
+	oldFileState := osutil.MemoryFileState{}
+
+	if st, err := os.Stat(path); err == nil {
+		b, err := ioutil.ReadFile(path)
+		if err != nil {
+			return nil, false, err
+		}
+		oldFileState.Content = b
+		oldFileState.Mode = st.Mode()
+		newFileState.Mode = st.Mode()
+
+		// save the old state of the file
+		old = &oldFileState
+	}
+
+	if mkdirErr := os.MkdirAll(filepath.Dir(path), 0755); mkdirErr != nil {
+		return nil, false, mkdirErr
+	}
+	ensureErr := osutil.EnsureFileState(path, &newFileState)
+	switch ensureErr {
+	case osutil.ErrSameState:
+		// didn't change the file
+		return old, false, nil
+	case nil:
+		// we successfully modified the file
+		return old, true, nil
+	default:
+		// some other fatal error trying to write the file
+		return nil, false, ensureErr
+	}
 }
 
-// AddSnapServices adds service units for the applications from the snap which are services.
-func AddSnapServices(s *snap.Info, disabledSvcs []string, opts *AddSnapServicesOptions, inter interacter) (err error) {
-	if s.Type() == snap.TypeSnapd {
-		return fmt.Errorf("internal error: adding explicit services for snapd snap is unexpected")
-	}
+type SnapServiceOptions struct {
+	// VitalityRank is the rank of all services in the specified snap used by
+	// the OOM killer when OOM conditions are reached.
+	VitalityRank int
 
-	if opts == nil {
-		opts = &AddSnapServicesOptions{}
-	}
+	// QuotaGroup is the quota group for all services in the specified snap.
+	QuotaGroup *quota.Group
+}
 
-	// check if any previously disabled services are now no longer services and
-	// log messages about that
-	for _, svc := range disabledSvcs {
-		app, ok := s.Apps[svc]
-		if !ok {
-			logger.Noticef("previously disabled service %s no longer exists", svc)
-		} else if !app.IsService() {
-			logger.Noticef("previously disabled service %s is now an app and not a service", svc)
-		}
-	}
+// ObserveChangeCallback can be invoked by EnsureSnapServices to observe
+// the previous content of a unit and the new on a change.
+// unitType can be "service", "socket", "timer". name is empty for a timer.
+type ObserveChangeCallback func(app *snap.AppInfo, grp *quota.Group, unitType string, name, old, new string)
 
-	// TODO: remove once services get enabled on start and not when created.
-	preseeding := opts.Preseeding
+// EnsureSnapServicesOptions is the set of options applying to the
+// EnsureSnapServices operation. It does not include per-snap specific options
+// such as VitalityRank or RequireMountedSnapdSnap from AddSnapServiceOptions,
+// since those are expected to be provided in the snaps argument.
+type EnsureSnapServicesOptions struct {
+	// Preseeding is whether the system is currently being preseeded, in which
+	// case there is not a running systemd for EnsureSnapServicesOptions to
+	// issue commands like systemctl daemon-reload to.
+	Preseeding bool
 
+	// RequireMountedSnapdSnap is whether the generated units should depend on
+	// the snapd snap being mounted, this is specific to systems like UC18 and
+	// UC20 which have the snapd snap and need to have units generated
+	RequireMountedSnapdSnap bool
+}
+
+// EnsureSnapServices will ensure that the specified snap services' file states
+// are up to date with the specified options and infos. It will add new services
+// if those units don't already exist, but it does not delete existing service
+// units that are not present in the snap's Info structures.
+// There are two sets of options; there are global options which apply to the
+// entire transaction and to every snap service that is ensured, and options
+// which are per-snap service and specified in the map argument.
+// If any errors are encountered trying to update systemd units, then all
+// changes performed up to that point are rolled back, meaning newly written
+// units are deleted and modified units are attempted to be restored to their
+// previous state.
+// To observe which units were added or modified a
+// ObserveChangeCallback calllback can be provided. The callback is
+// invoked while processing the changes. Because of that it should not
+// produce immediate side-effects, as the changes are in effect only
+// if the function did not return an error.
+func EnsureSnapServices(snaps map[*snap.Info]*SnapServiceOptions, opts *EnsureSnapServicesOptions, observeChange ObserveChangeCallback, inter interacter) (err error) {
 	// note, sysd is not used when preseeding
 	sysd := systemd.New(systemd.SystemMode, inter)
-	var written []string
-	var writtenSystem, writtenUser bool
-	var disableEnabledServices func()
+
+	if opts == nil {
+		opts = &EnsureSnapServicesOptions{}
+	}
+
+	// we only consider the global EnsureSnapServicesOptions to decide if we
+	// are preseeding or not to reduce confusion about which set of options
+	// determines whether we are preseeding or not during the ensure operation
+	preseeding := opts.Preseeding
+
+	// modifiedUnitsPreviousState is the set of units that were modified and the previous
+	// state of the unit before modification that we can roll back to if there
+	// are any issues.
+	// note that the rollback is best effort, if we are rebooted in the middle,
+	// there is no guarantee about the state of files, some may have been
+	// updated and some may have been rolled back, higher level tasks/changes
+	// should have do/undo handlers to properly handle the case where this
+	// function is interrupted midway
+	modifiedUnitsPreviousState := make(map[string]*osutil.MemoryFileState)
+	var modifiedSystem, modifiedUser bool
 
 	defer func() {
 		if err == nil {
 			return
 		}
-		if disableEnabledServices != nil {
-			disableEnabledServices()
-		}
-		for _, s := range written {
-			if e := os.Remove(s); e != nil {
-				inter.Notify(fmt.Sprintf("while trying to remove %s due to previous failure: %v", s, e))
+		for file, state := range modifiedUnitsPreviousState {
+			if state == nil {
+				// we don't have anything to rollback to, so just remove the
+				// file
+				if e := os.Remove(file); e != nil {
+					inter.Notify(fmt.Sprintf("while trying to remove %s due to previous failure: %v", file, e))
+				}
+			} else {
+				// rollback the file to the previous state
+				if e := osutil.EnsureFileState(file, state); e != nil {
+					inter.Notify(fmt.Sprintf("while trying to rollback %s due to previous failure: %v", file, e))
+				}
 			}
 		}
-		if writtenSystem && !preseeding {
+		if modifiedSystem && !preseeding {
 			if e := sysd.DaemonReload(); e != nil {
 				inter.Notify(fmt.Sprintf("while trying to perform systemd daemon-reload due to previous failure: %v", e))
 			}
 		}
-		if writtenUser && !preseeding {
+		if modifiedUser && !preseeding {
 			if e := userDaemonReload(); e != nil {
 				inter.Notify(fmt.Sprintf("while trying to perform user systemd daemon-reload due to previous failure: %v", e))
 			}
 		}
 	}()
 
-	var toEnable []*snap.AppInfo
-
-	// create services first; this doesn't trigger systemd
-	for _, app := range s.Apps {
-		if !app.IsService() {
-			continue
-		}
-		// Generate service file
-		content, err := generateSnapServiceFile(app, opts)
+	handleFileModification := func(app *snap.AppInfo, unitType string, name, path string, content []byte) error {
+		old, modifiedFile, err := tryFileUpdate(path, content)
 		if err != nil {
 			return err
 		}
-		svcFilePath := app.ServiceFile()
-		os.MkdirAll(filepath.Dir(svcFilePath), 0755)
-		if err := osutil.AtomicWriteFile(svcFilePath, content, 0644, 0); err != nil {
-			return err
-		}
-		written = append(written, svcFilePath)
-		switch app.DaemonScope {
-		case snap.SystemDaemon:
-			writtenSystem = true
-		case snap.UserDaemon:
-			writtenUser = true
-		}
 
-		// Generate systemd .socket files if needed
-		socketFiles, err := generateSnapSocketFiles(app)
-		if err != nil {
-			return err
-		}
-		for path, content := range *socketFiles {
-			os.MkdirAll(filepath.Dir(path), 0755)
-			if err := osutil.AtomicWriteFile(path, content, 0644, 0); err != nil {
-				return err
+		if modifiedFile {
+			if observeChange != nil {
+				var oldContent []byte
+				if old != nil {
+					oldContent = old.Content
+				}
+				observeChange(app, nil, unitType, name, string(oldContent), string(content))
+			}
+			modifiedUnitsPreviousState[path] = old
+
+			// also mark that we need to reload either the system or
+			// user instance of systemd
+			switch app.DaemonScope {
+			case snap.SystemDaemon:
+				modifiedSystem = true
+			case snap.UserDaemon:
+				modifiedUser = true
 			}
-			written = append(written, path)
 		}
 
-		if app.Timer != nil {
-			content, err := generateSnapTimerFile(app)
+		return nil
+	}
+
+	neededQuotaGrps := &quota.QuotaGroupSet{}
+
+	for s, snapSvcOpts := range snaps {
+		if s.Type() == snap.TypeSnapd {
+			return fmt.Errorf("internal error: adding explicit services for snapd snap is unexpected")
+		}
+
+		// always use RequireMountedSnapdSnap options from the global options
+		genServiceOpts := &AddSnapServicesOptions{
+			RequireMountedSnapdSnap: opts.RequireMountedSnapdSnap,
+		}
+		if snapSvcOpts != nil {
+			// and if there are per-snap options specified, use that for
+			// VitalityRank
+			genServiceOpts.VitalityRank = snapSvcOpts.VitalityRank
+			genServiceOpts.QuotaGroup = snapSvcOpts.QuotaGroup
+
+			if snapSvcOpts.QuotaGroup != nil {
+				if err := neededQuotaGrps.AddAllNecessaryGroups(snapSvcOpts.QuotaGroup); err != nil {
+					// this error can basically only be a circular reference
+					// in the quota group tree
+					return err
+				}
+			}
+		}
+		// note that the Preseeding option is not used here at all
+
+		for _, app := range s.Apps {
+			if !app.IsService() {
+				continue
+			}
+
+			// create services first; this doesn't trigger systemd
+
+			// Generate new service file state
+			path := app.ServiceFile()
+			content, err := generateSnapServiceFile(app, genServiceOpts)
 			if err != nil {
 				return err
 			}
-			path := app.Timer.File()
-			os.MkdirAll(filepath.Dir(path), 0755)
-			if err := osutil.AtomicWriteFile(path, content, 0644, 0); err != nil {
+
+			if err := handleFileModification(app, "service", app.Name, path, content); err != nil {
 				return err
 			}
-			written = append(written, path)
-		}
 
-		if app.Timer != nil || len(app.Sockets) != 0 {
-			// service is socket or timer activated, not during the
-			// boot
-			continue
-		}
-		// XXX: this may become quadratic, optimize.
-		// When preseeding services get enabled in doMarkPresseeded instead at
-		// the moment.
-		if strutil.ListContains(disabledSvcs, app.Name) || preseeding {
-			continue
-		}
+			// Generate systemd .socket files if needed
+			socketFiles, err := generateSnapSocketFiles(app)
+			if err != nil {
+				return err
+			}
+			for name, content := range socketFiles {
+				path := app.Sockets[name].File()
+				if err := handleFileModification(app, "socket", name, path, content); err != nil {
+					return err
+				}
+			}
 
-		toEnable = append(toEnable, app)
+			if app.Timer != nil {
+				content, err := generateSnapTimerFile(app)
+				if err != nil {
+					return err
+				}
+				path := app.Timer.File()
+				if err := handleFileModification(app, "timer", "", path, content); err != nil {
+					return err
+				}
+			}
+		}
 	}
 
-	// this is no-op when preseeding because of empty toEnable list
-	disableEnabledServices, err = enableServices(toEnable, inter)
-	if err != nil {
-		return err
+	handleSliceModification := func(grp *quota.Group, path string, content []byte) error {
+		old, modifiedFile, err := tryFileUpdate(path, content)
+		if err != nil {
+			return err
+		}
+
+		if modifiedFile {
+			if observeChange != nil {
+				var oldContent []byte
+				if old != nil {
+					oldContent = old.Content
+				}
+				observeChange(nil, grp, "slice", grp.Name, string(oldContent), string(content))
+			}
+
+			modifiedUnitsPreviousState[path] = old
+
+			// also mark that we need to reload the system instance of systemd
+			// TODO: also handle reloading the user instance of systemd when
+			// needed
+			modifiedSystem = true
+		}
+
+		return nil
+	}
+
+	// now make sure that all of the slice units exist
+	for _, grp := range neededQuotaGrps.AllQuotaGroups() {
+		content, err := generateGroupSliceFile(grp)
+		if err != nil {
+			return err
+		}
+
+		sliceFileName := grp.SliceFileName()
+		path := filepath.Join(dirs.SnapServicesDir, sliceFileName)
+		if err := handleSliceModification(grp, path, content); err != nil {
+			return err
+		}
 	}
 
 	if !preseeding {
-		if writtenSystem {
-			if err := sysd.DaemonReload(); err != nil {
+		if modifiedSystem {
+			if err = sysd.DaemonReload(); err != nil {
 				return err
 			}
 		}
-		if writtenUser {
-			if err := userDaemonReload(); err != nil {
+		if modifiedUser {
+			if err = userDaemonReload(); err != nil {
 				return err
 			}
 		}
@@ -490,20 +700,47 @@ func AddSnapServices(s *snap.Info, disabledSvcs []string, opts *AddSnapServicesO
 	return nil
 }
 
-// EnableSnapServices enables all services of the snap; the main use case for this is
-// the first boot of a pre-seeded image with service files already in place but not enabled.
-// XXX: it should go away once services are fixed and enabled on start.
-func EnableSnapServices(s *snap.Info, inter interacter) (err error) {
-	sysd := systemd.New(systemd.SystemMode, inter)
-	for _, app := range s.Apps {
-		if app.IsService() {
-			svcName := app.ServiceName()
-			if err := sysd.Enable(svcName); err != nil {
-				return err
-			}
-		}
+// AddSnapServicesOptions is a struct for controlling the generated service
+// definition for a snap service.
+type AddSnapServicesOptions struct {
+	// VitalityRank is the rank of all services in the specified snap used by
+	// the OOM killer when OOM conditions are reached.
+	VitalityRank int
+
+	// QuotaGroup is the quota group for all services in the specified snap.
+	QuotaGroup *quota.Group
+
+	// RequireMountedSnapdSnap is whether the generated units should depend on
+	// the snapd snap being mounted, this is specific to systems like UC18 and
+	// UC20 which have the snapd snap and need to have units generated
+	RequireMountedSnapdSnap bool
+
+	// Preseeding is whether the system is currently being preseeded, in which
+	// case there is not a running systemd for EnsureSnapServicesOptions to
+	// issue commands like systemctl daemon-reload to.
+	Preseeding bool
+}
+
+// AddSnapServices adds service units for the applications from the snap which
+// are services. The services do not get enabled or started.
+func AddSnapServices(s *snap.Info, opts *AddSnapServicesOptions, inter interacter) error {
+	m := map[*snap.Info]*SnapServiceOptions{
+		s: {},
 	}
-	return nil
+	ensureOpts := &EnsureSnapServicesOptions{}
+	if opts != nil {
+		// set the per-snap service options
+		m[s].VitalityRank = opts.VitalityRank
+		m[s].QuotaGroup = opts.QuotaGroup
+
+		// copy the globally applicable opts from AddSnapServicesOptions to
+		// EnsureSnapServicesOptions, since those options override the per-snap opts
+		// we put in the map argument
+		ensureOpts.Preseeding = opts.Preseeding
+		ensureOpts.RequireMountedSnapdSnap = opts.RequireMountedSnapdSnap
+	}
+
+	return EnsureSnapServices(m, ensureOpts, nil, inter)
 }
 
 // StopServicesFlags carries extra flags for StopServices.
@@ -591,6 +828,33 @@ func ServicesEnableState(s *snap.Info, inter interacter) (map[string]bool, error
 	return snapSvcsState, nil
 }
 
+// RemoveQuotaGroup ensures that the slice file for a quota group is removed. It
+// assumes that the slice corresponding to the group is not in use anymore by
+// any services or sub-groups of the group when it is invoked.
+// group with sub-groups, one must remove all the sub-groups first.
+func RemoveQuotaGroup(grp *quota.Group, inter interacter) error {
+	// TODO: it only works on leaf sub-groups currently
+	if len(grp.SubGroups) != 0 {
+		return fmt.Errorf("internal error: cannot remove quota group with sub-groups")
+	}
+
+	systemSysd := systemd.New(systemd.SystemMode, inter)
+
+	// remove the slice file
+	err := os.Remove(filepath.Join(dirs.SnapServicesDir, grp.SliceFileName()))
+	if err != nil && !os.IsNotExist(err) {
+		return err
+	}
+
+	if err == nil {
+		// we deleted the slice unit, so we need to daemon-reload
+		if err := systemSysd.DaemonReload(); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // RemoveSnapServices disables and removes service units for the applications
 // from the snap which are services. The optional flag indicates whether
 // services are removed as part of undoing of first install of a given snap.
@@ -679,6 +943,9 @@ func genServiceNames(snap *snap.Info, appNames []string) []string {
 	return names
 }
 
+// TODO: this should not accept AddSnapServicesOptions, it should use some other
+// subset of options, specifically it should not accept Preseeding as an option
+// here
 func genServiceFile(appInfo *snap.AppInfo, opts *AddSnapServicesOptions) ([]byte, error) {
 	if opts == nil {
 		opts = &AddSnapServicesOptions{}
@@ -731,6 +998,10 @@ After={{ stringsJoin .After " " }}
 {{- if .Before}}
 Before={{ stringsJoin .Before " "}}
 {{- end}}
+{{- if .CoreMountedSnapdSnapDep}}
+Wants={{ stringsJoin .CoreMountedSnapdSnapDep " "}}
+After={{ stringsJoin .CoreMountedSnapdSnapDep " "}}
+{{- end}}
 X-Snappy=yes
 
 [Service]
@@ -761,8 +1032,8 @@ Type={{.App.Daemon}}
 {{- if .Remain}}
 RemainAfterExit={{.Remain}}
 {{- end}}
-{{- if .App.BusName}}
-BusName={{.App.BusName}}
+{{- if .BusName}}
+BusName={{.BusName}}
 {{- end}}
 {{- if .App.WatchdogTimeout}}
 WatchdogSec={{.App.WatchdogTimeout.Seconds}}
@@ -779,7 +1050,10 @@ OOMScoreAdjust={{.OOMAdjustScore}}
 {{- if .InterfaceServiceSnippets}}
 {{.InterfaceServiceSnippets}}
 {{- end}}
-{{- if not .App.Sockets}}
+{{- if .SliceUnit}}
+Slice={{.SliceUnit}}
+{{- end}}
+{{- if not (or .App.Sockets .App.Timer .App.ActivatesOn) }}
 
 [Install]
 WantedBy={{.ServicesTarget}}
@@ -820,6 +1094,18 @@ WantedBy={{.ServicesTarget}}
 		killMode = "process"
 	}
 
+	var busName string
+	if appInfo.Daemon == "dbus" {
+		busName = appInfo.BusName
+		if busName == "" && len(appInfo.ActivatesOn) != 0 {
+			slot := appInfo.ActivatesOn[len(appInfo.ActivatesOn)-1]
+			if err := slot.Attr("name", &busName); err != nil {
+				// This should be impossible for a valid AppInfo
+				logger.Noticef("Cannot get 'name' attribute of dbus slot %q: %v", slot.Name, err)
+			}
+		}
+	}
+
 	wrapperData := struct {
 		App *snap.AppInfo
 
@@ -834,12 +1120,16 @@ WantedBy={{.ServicesTarget}}
 		KillMode                 string
 		KillSignal               string
 		OOMAdjustScore           int
+		BusName                  string
 		Before                   []string
 		After                    []string
 		InterfaceServiceSnippets string
+		SliceUnit                string
 
 		Home    string
 		EnvVars string
+
+		CoreMountedSnapdSnapDep []string
 	}{
 		App: appInfo,
 
@@ -852,6 +1142,7 @@ WantedBy={{.ServicesTarget}}
 		KillMode:       killMode,
 		KillSignal:     appInfo.StopMode.KillSignal(),
 		OOMAdjustScore: oomAdjustScore,
+		BusName:        busName,
 
 		Before: genServiceNames(appInfo.Snap, appInfo.Before),
 		After:  genServiceNames(appInfo.Snap, appInfo.After),
@@ -875,6 +1166,11 @@ WantedBy={{.ServicesTarget}}
 		panic("unknown snap.DaemonScope")
 	}
 
+	// check the quota group slice
+	if opts.QuotaGroup != nil {
+		wrapperData.SliceUnit = opts.QuotaGroup.SliceFileName()
+	}
+
 	// Add extra "After" targets
 	if wrapperData.PrerequisiteTarget != "" {
 		wrapperData.After = append([]string{wrapperData.PrerequisiteTarget}, wrapperData.After...)
@@ -883,6 +1179,14 @@ WantedBy={{.ServicesTarget}}
 		wrapperData.After = append([]string{wrapperData.MountUnit}, wrapperData.After...)
 	}
 
+	if opts.RequireMountedSnapdSnap {
+		// on core 18+ systems, the snapd tooling is exported
+		// into the host system via a special mount unit, which
+		// also adds an implicit dependency on the snapd snap
+		// mount thus /usr/bin/snap points
+		wrapperData.CoreMountedSnapdSnapDep = []string{SnapdToolingMountUnit}
+	}
+
 	if err := t.Execute(&templateOut, wrapperData); err != nil {
 		// this can never happen, except we forget a variable
 		logger.Panicf("Unable to execute template: %v", err)
@@ -950,16 +1254,16 @@ WantedBy={{.SocketsTarget}}
 	return templateOut.Bytes()
 }
 
-func generateSnapSocketFiles(app *snap.AppInfo) (*map[string][]byte, error) {
+func generateSnapSocketFiles(app *snap.AppInfo) (map[string][]byte, error) {
 	if err := snap.ValidateApp(app); err != nil {
 		return nil, err
 	}
 
 	socketFiles := make(map[string][]byte)
-	for name, socket := range app.Sockets {
-		socketFiles[socket.File()] = genServiceSocketFile(app, name)
+	for name := range app.Sockets {
+		socketFiles[name] = genServiceSocketFile(app, name)
 	}
-	return &socketFiles, nil
+	return socketFiles, nil
 }
 
 func renderListenStream(socket *snap.SocketInfo) string {
